<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\AppController;
use App\Models\Category;
use App\Http\Controllers\Controller;
use App\Models\AssignedTraining;
use App\Models\Training;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class AdminController extends Controller
{
    /**
     * index
     */
    public function index(Request $request, $locale) 
    {
        $categories = Category::orderBy('cat_num', 'asc')
                        ->select('cat_num', 'new_name', 'image', 'id', 'code')
                        ->get();
        $tutorial_video = "videos/$locale-tutorial.mp4";
        $tutorial_video = Storage::url($tutorial_video);
        
        $trainings = Training::where('company_id', $this->company->id)
                        ->select('id', 'name', 'prefix', 'image', 'disable', 'category_id', 'platform_id')
                        ->get();

        $assigned_trainings = AssignedTraining::where('user_id', $this->user->id)->get();

        $online_training_and_evaluation = 0;
        $external_presentiel = 0;
        $online_reading_procedures = 0;
        $interactive_apprentice_booklet = 0;

        $training_catalogs = [];
        $training_ids = [];
        $i = 0;
        foreach ($trainings as $training)
        {
            $training_ids[$training->id] = $training->id;
            if ($training->category->id == 1)
            {
                $j = $i % 3;
                $image = $this->unserialize($training->image);
                $training->image = $image;
                $training_catalogs[$j][] = $training;
                $i++;
            }

            if ($training->disable)
            {
                switch ($training->category->id)
                {
                    case 1:
                        $online_training_and_evaluation++;
                        break;
                    case 2:
                        $online_reading_procedures++;
                        break;
                    case 3:
                        $interactive_apprentice_booklet++;
                        break;
                }
                if ($training->platform->id == 2) 
                {
                    $external_presentiel++;
                }
            }
        }

        $assign_trainings = $assigned_trainings->count();
        $completed_trainings = 0;
        $timespent = 0;

        foreach ($assigned_trainings as $assign_training)
        {
            $training = $assign_training->training;
            $user_training = $assign_training->user_training;
            $completed_info = AppController::completed_info($user_training);
            if ($completed_info['completed'])
            {
                // $completed_trainings++;
            }
            $timespent += $completed_info['duration'];
        }
        $timespent = $this->sec_to_time_full($timespent);

        foreach ($assigned_trainings as $assigned_training)
        {
            if ($assigned_training->status == 'Completed')
            {
                $completed_trainings++;
            }
        }

        $nothing_assigned_category = 0;
        $nothing_assigned_training = 0;
        $not_started_category = 0;
        $not_started_training = 0;
        $in_progress_category = 0;
        $in_progress_training = 0;
        $all_done_category = 0;
        $all_done_training = 0;
        
        $data = compact(
            'categories', 
            'tutorial_video',
            'nothing_assigned_category',
            'nothing_assigned_training',
            'not_started_category',
            'not_started_training',
            'in_progress_category',
            'in_progress_training',
            'all_done_category',
            'all_done_training',
            'assign_trainings',
            'completed_trainings',
            'timespent',
            'online_training_and_evaluation',
            'external_presentiel',
            'online_reading_procedures',
            'interactive_apprentice_booklet',
            'training_catalogs'
        );
        return view('admin.dashboard', $data);
    }

    /**
     * create
     */
    public function create(Request $request)
    {
        return view('admin.register');
    }

    /**
     * report
     */
    public function report(Request $request)
    {
        return view('admin.report');
    }
}
