<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Presentiel;
use App\Models\Trainer;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;

class CalendarController extends Controller
{
    /**
     * index
     */
    public function index(Request $request, $locale)
    {
        $data = [];
        $request_data = $request->all();
        $parameters = $request_data;
        $parameters['locale'] = $locale;

        $trainin_ids = $request_data['training_id'] ?? false;
        $department_id = $request_data['department_id'] ?? false;
        $subdepartment_id = $request_data['subdepartment_id'] ?? false;
        $location_id = $request_data['location_id'] ?? false;
        $trainer_id = $request_data['trainer_id'] ?? false;

        $presentiels_qb = Presentiel::where('presentiel.company_id', $this->company->id);
        $presentiels_qb = $presentiels_qb->leftJoin('training', 'training.id', '=', 'presentiel.training_id');
        $presentiels_qb = $trainin_ids ? $presentiels_qb->whereIn('presentiel.training_id', $trainin_ids) : $presentiels_qb;
        $presentiels_qb = $department_id ? $presentiels_qb->where('training.department_id', $department_id) : $presentiels_qb;
        $presentiels_qb = $subdepartment_id ? $presentiels_qb->where('training.subdepartment_id', $subdepartment_id) : $presentiels_qb;
        $presentiels_qb = $location_id ? $presentiels_qb->where('training.location_id', $location_id) : $presentiels_qb;
        $presentiels_qb = $trainer_id ? $presentiels_qb->where('presentiel.trainer_id', $trainer_id) : $presentiels_qb;
        $presentiels = $presentiels_qb->get();

        $event_list = [];
        $reccuring_training = 0;
        $non_reccuring_training = 0;
        $trainings = [];
        $departments = [];
        $subdepartments = [];
        $locations = [];
        $trainers = [];
        foreach ($presentiels as $presentiel)
        {
            $assigned_to = [];
            foreach ($presentiel->activities_presentiel as $activity_presentiel)
            {
                foreach ($activity_presentiel->activities as $activity)
                {
                    $assigned_training = $activity->assigned_training;
                    $assigned_to[$assigned_training->user->id] = $assigned_training->user->fname . ' ' . $assigned_training->user->lname;
                }
            }
            $description = implode(" - ", $assigned_to);
            $event = [
                'groupId' => $presentiel->training->id,
                'id'  => $presentiel->id,
                'title' => $presentiel->training->name,
                'start' => $presentiel->start_date . 'T' . $presentiel->time_start,
                'end' => $presentiel->end_date .  'T' . $presentiel->time_stop,
                'extendedProps' => [
                    'calendar' => $presentiel->reccuring_calendar ? 'Warning' : 'Primary',
                    'description' => $description
                ]
            ];
            $event_list[] = $event;
            if ($presentiel->reccuring_calendar)
            {
                $reccuring_training++;
            }
            else
            {
                $non_reccuring_training++;
            }
            $training = $presentiel->training;
            $trainings[$training->id] = $training;
            $departments[$training->department->id] = $training->department;
            $subdepartments[$training->subdepartment->id] = $training->subdepartment;
            $locations[$training->location->id] = $training->location;
            $trainers[$presentiel->trainer->id] = $presentiel->trainer;
        }
        $data['parameters'] = $parameters;
        $data['presentiels'] = $presentiels;
        $data['event_list'] = json_encode($event_list);
        $data['reccuring_training'] = $reccuring_training;
        $data['non_reccuring_training'] = $non_reccuring_training;
        $data['trainings'] = $trainings;
        $data['departments'] = $departments;
        $data['subdepartments'] = $subdepartments;
        $data['locations'] = $locations;
        $data['trainers'] = $trainers;

        return view('admin.calendar', $data);
    }

    /**
     * training_details
     */
    public function training_details(Request $request, $locale)
    {
        $data = [];

        $presentiel_id = $request->id;
        $presentiel = Presentiel::find($presentiel_id);
        if ($presentiel)
        {
            $this->check_company($presentiel);
        }
        else
        {
            $this->not_found();
        }

        $presentiel->duration = $this->minute_to_time($presentiel->duration);
        $documents = $this->unserialize($presentiel->documents);
        $data['documents'] = $documents;
        Session::put('downloads', $documents);
        
        $trainees = $this->unserialize($presentiel->trainees);
        $presentiel->trainees = User::whereIn('id', $trainees)->get();
        
        $data['activities_presentiel'] = [];
        $data['assigned_training'] = [];
        foreach ($presentiel->activities_presentiel as $i => $activity_presentiel)
        {
            $activity_presentiel->duration = $this->minute_to_time($activity_presentiel->duration);
            $data['activities_presentiel'][$i] = $activity_presentiel;
            foreach ($activity_presentiel->activities as $activity)
            {
                if ($activity->assigned_training)
                {
                    $data['assigned_training'][$activity->assigned_training->id] = $activity->assigned_training;
                }
            }
        }
        foreach ($presentiel->assigned_training as $assigned_training)
        {
            $data['assigned_training'][$assigned_training->id] = $assigned_training;
        }

        $data['presentiel'] = $presentiel;
        $data['is_modal'] = true;

        return view('training.presentiel.view-tpl', $data);
    }
}
