<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Company;
use App\Models\GroupsCompany;
use Illuminate\Http\Request;

class CompaniesController extends Controller
{
    public function index()
    {
        $companies = Company::all();
        $groups = GroupsCompany::all();
        return view('admin.companies.company', compact('companies','groups'));
    }

    public function create($locale)
    {
        $companies = Company::all();
        $groups = GroupsCompany::all();
        $companies = Company::with('groupsCompany')->get();
        return view('admin.companies.company', compact('companies', 'groups'));
    }

    public function store($locale,Request $request)
    {
        $validatedData = $request->validate([
            'name' => 'required|string|max:255',
            'license_type' => 'required|integer',
            'max_users' => 'required|integer',
            'display' => 'required|boolean',
            'stripe_customer_id' => 'nullable|string|max:64',
            'group_company_id' => 'nullable|integer',
            'logo' => 'nullable|image|mimes:jpg,jpeg,png,gif,svg|max:2048',
            'logo2' => 'nullable|image|mimes:jpg,jpeg,png,gif,svg|max:2048',
        ]);

    
        // Handle logo upload

        if ($request->hasFile('logo')) {
            $validatedData['logo'] = $request->file('logo')->store('images', 'public');
        }


        // Handle logo2 upload
        if ($request->hasFile('logo2')) {
            $validatedData['logo2'] = $request->file('logo2')->store('images', 'public');
        }

        Company::create($validatedData);

        return redirect()->route('companies.create',$locale)->with('success', 'Company created successfully.');
    }

    public function show(string $id)
    {
        // Votre logique ici
    }

    public function edit($locale,string $id)
    {
        // Votre logique ici
        $groups = GroupsCompany::all();
        $company = Company::all();
        $companies = Company::findOrFail($id);
        return view('admin.companies.updatecompanies', compact('companies', 'company', 'groups'));
    }

    public function update($locale,Request $request, string $id)
    {
        // Valider les données du formulaire
        $validatedData = $request->validate([
            'name' => 'required|string|max:255',
            'license_type' => 'required|integer',
            'max_users' => 'required|integer',
            'display' => 'required|boolean',
            'stripe_customer_id' => 'nullable|string|max:64',
            'group_company_id' => 'nullable|integer',
            'logo' => 'nullable|image|mimes:jpg,jpeg,png,gif,svg|max:2048',
            'logo2' => 'nullable|image|mimes:jpg,jpeg,png,gif,svg|max:2048',
        ]);

        // Récupérer l'entreprise à mettre à jour par son ID
        $company = Company::findOrFail($id);

        // Handle logo upload

        if ($request->hasFile('logo')) {
            $validatedData['logo'] = $request->file('logo')->store('images', 'public');
        }


        // Handle logo2 upload
        if ($request->hasFile('logo2')) {
            $validatedData['logo2'] = $request->file('logo2')->store('images', 'public');
        }

        // Mettre à jour les attributs de l'entreprise
        $company->update($validatedData);

        // Redirection avec un message de succès
        return redirect()->route('companies.index',$locale)->with('success', 'Company updated successfully.');
    }

    public function destroy($locale,$id)
    {
        $company = Company::findOrFail($id);
        $company->delete();

        return redirect()->route('companies.index',$locale)->with('success', 'Company deleted successfully.');
    }
}
