<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use App\Http\Requests\Auth\LoginRequest;
use App\Models\Company;
use Carbon\Carbon;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Auth;
use Illuminate\View\View;
use App\Models\User;
class AuthenticatedSessionController extends Controller
{
    /**
     * Display the login view.
     */
    public function create(): View
    {
        // Récupérer toutes les entreprises depuis la base de données
        $Companies = Company::all();

        // Passer les entreprises à la vue
        return view('auth.login', compact('Companies'));
    }

    /**
     * Handle an incoming authentication request.
     */
    public function store(LoginRequest $request): RedirectResponse
    {
        $request->authenticate();

        $request->session()->regenerate();

        $request->user()->update([
            'last_login_at' => Carbon::now()->toDateTimeString(),
            'last_login_ip' => $request->getClientIp()
        ]);

        App::setlocale(Auth::user()->lang);
        
        if (Auth::user()->supervisor == 'Administrator') {
            return redirect(route('admin.dashboard'));
        }

        return redirect()->intended(route('admin.dashboard', ['locale' => App::currentLocale()], absolute: false));
    }

    /**
     * Destroy an authenticated session.
     */
    public function destroy(Request $request): RedirectResponse
    {
        // If we're logged in as another user, return to the original user instead of logging out
        if ($request->session()->has('original_user_id')) {
            $originalUser = User::find($request->session()->get('original_user_id'));
            Auth::login($originalUser);
            $request->session()->forget('original_user_id');
            return redirect()->route('admin.dashboard', ['locale' => App::currentLocale()]);
        }
        Auth::guard('web')->logout();

        $request->session()->invalidate();

        $request->session()->regenerateToken();

        return redirect('/public/');
    }
}
