<?php 
namespace App\Http\Controllers;

use App\Models\Category;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;

class CategoryController extends Controller
{
    public function index()
    {
        $categories = Category::all();
        return view('categorie.categorie', compact('categories'));
    }

    public function create($locale)
    {
        $categories = Category::all();
        return view('categorie.categorie', compact('categories'));
    }

    public function store($locale, Request $request)
    {
        $validatedData = $request->validate([
            'cat_name' => 'required|string|max:60',
            'cat_num' => 'required|string|max:6',
            'new_name' => 'required|string|max:60',
            'image' => 'nullable|image|max:20048',
            'video_fr' => 'nullable|mimes:mp4,mov,ogg,qt|max:20000',
            'video_en' => 'nullable|mimes:mp4,mov,ogg,qt|max:20000',
        ]);
    
        Log::info('Validated Data:', $validatedData); // Ajoutez ceci pour voir les données validées
    
        if ($request->hasFile('image')) {
            $validatedData['image'] = $request->file('image')->store('images', 'public');
        }
    
        if ($request->hasFile('video_fr')) {
            $validatedData['video_fr'] = $request->file('video_fr')->store('videos', 'public');
        }
    
        if ($request->hasFile('video_en')) {
            $validatedData['video_en'] = $request->file('video_en')->store('videos', 'public');
        }
    
        Log::info('Data to be inserted:', $validatedData); // Ajoutez ceci pour voir les données avant l'insertion
    
        try {
            $category = Category::create($validatedData);
            Log::info('Category created successfully:', $category->toArray());
        } catch (\Exception $e) {
            Log::error('Error creating category:', ['message' => $e->getMessage()]);
        }
    
        return redirect()->route('categorie.create', $locale)->with('success', 'Catégorie ajoutée avec succès.');
    }
    

    public function edit($locale, $id)
    {
        $category = Category::findOrFail($id);
        return view('categorie.edit', compact('category'));
    }

    public function update(Request $request, $locale, $id)
    {
        $category = Category::findOrFail($id);

        $validatedData = $request->validate([
            'cat_name' => 'required|string|max:60',
            'cat_num' => 'required|string|max:6',
            'new_name' => 'required|string|max:60',
            'image' => 'nullable|image|max:20048',
            'video_fr' => 'nullable|mimes:mp4,mov,ogg,qt|max:50000',
            'video_en' => 'nullable|mimes:mp4,mov,ogg,qt|max:50000',
        ]);

        if ($request->hasFile('image')) {
            $validatedData['image'] = $request->file('image')->store('images', 'public');
            // Supprimer l'ancienne image si elle existe
            if ($category->image) {
                Storage::delete('public/' . $category->image);
            }
        } else {
            $validatedData['image'] = $category->image;
        }

        if ($request->hasFile('video_fr')) {
            $validatedData['video_fr'] = $request->file('video_fr')->store('videos', 'public');
            // Supprimer l'ancienne vidéo si elle existe
            if ($category->video_fr) {
                Storage::delete('public/' . $category->video_fr);
            }
        } else {
            $validatedData['video_fr'] = $category->video_fr;
        }

        if ($request->hasFile('video_en')) {
            $validatedData['video_en'] = $request->file('video_en')->store('videos', 'public');
            // Supprimer l'ancienne vidéo si elle existe
            if ($category->video_en) {
                Storage::delete('public/' . $category->video_en);
            }
        } else {
            $validatedData['video_en'] = $category->video_en;
        }

        $category->update($validatedData);

        return redirect()->route('categorie.index', $locale)->with('success', 'Catégorie mise à jour avec succès.');
    }

    public function destroy($locale, $id)
    {
        $category = Category::findOrFail($id);
        if ($category->image) {
            Storage::delete('public/' . $category->image);
        }
        if ($category->video_fr) {
            Storage::delete('public/' . $category->video_fr);
        }
        if ($category->video_en) {
            Storage::delete('public/' . $category->video_en);
        }
        $category->delete();

        return redirect()->route('categorie.index', $locale)->with('success', 'Catégorie supprimée avec succès.');
    }
}
