<?php

namespace App\Http\Controllers\CompetencyCards;

use App\Http\Controllers\Controller;
use App\Models\TrainingService;
use App\Models\User;
use App\Models\Competency; // assuming you have a Competency model
use App\Models\CompetencyCard;
use Illuminate\Http\Request;

class CompetencyCardsRequestController extends Controller
{
    /**
     * Display a listing of the resource.
     */

    public function index()
    {
        //
        $users = User::all();
        //$competencies = competency::all();
        $competencies = CompetencyCard::all();

        return view('management.competency_cards.competency_cards_requests_create', compact('competencies','users'));
    }

    /**   
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
        return view('management.competancy_cards.competency_cards_requests_create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store($locale,Request $request)
    {
        //
        try {
            $request->validate([
                'competency_name' => 'required|string|max:255',
                'training_service' => 'required|string|max:255',
                'coursePlan' => 'nullable|mimes:pdf,docx,jpg,jpeg,png|max:2048',
                'approved' => 'nullable|boolean',
                'approved_by' => 'nullable|exists:users,id',
                'created_by' => 'nullable|exists:users,id',
            ]);
            $competencycards = new CompetencyCard();
            $competencycards->competency_name = $request->input('competency_name');
            $competencycards->training_service = $request->input('training_service');
            $competencycards->approved = $request->input('approved');
            $competencycards->approved_by = $request->input('approved_by');
            $competencycards->created_by = $request->input('created_by');
            $competencycards->company_id = auth()->user()->company_id;
            // Gestion du fichier 'coursePlan'
            if ($request->hasFile('coursePlan')) {
                $file = $request->file('coursePlan');
                $fileExtension = $file->getClientOriginalExtension();
                $filePath = '';

                if (in_array($fileExtension, ['pdf', 'docx'])) {
                    $filePath = $file->store("course-plans/{$fileExtension}s", 'public');
                } elseif (in_array($fileExtension, ['jpg', 'jpeg', 'png'])) {
                    $filePath = $file->store('course-plans/images', 'public');
                }

                $competencycards->curriculum = $filePath;
            }
            $competencycards->save();
            if ($request->ajax()) {
                # code...
                return response()->json($competencycards);
            }
            return redirect()->route('competencycardrequests.index', $locale)->with('success', 'Carte Competence créé avec succès!');
        } catch (\Exception $e) {
            // Capture et affichage de l'erreur
            return redirect()->back()->with('error', 'Une erreur est survenue : ' . $e->getMessage());
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        //
        $competencies = Competency::all(); // assuming you have a Competency model
        return view('management.competancy_cards.competencycardrequests.edit', compact('competencies'));
    
    }

    /**
     * Update the specified resource in storage.
     */
    public function update($locale, Request $request, $id)
{
    try {
        // Validation des données
        $request->validate([
            'competency_name' => 'required|string|max:255',
            'training_service' => 'required|string|max:255',
            'coursePlan' => 'nullable|mimes:pdf,docx,jpg,jpeg,png|max:2048',
            'approved' => 'nullable|boolean',
            'approved_by' => 'nullable|exists:users,id',
            'created_by' => 'nullable|exists:users,id',
        ]);

        // Recherche de la carte de compétence
        $competencycards = CompetencyCard::findOrFail($id);

        // Mise à jour des champs
        $competencycards->competency_name = $request->input('competency_name');
        $competencycards->training_service = $request->input('training_service');
        $competencycards->approved = $request->input('approved');
        $competencycards->approved_by = $request->input('approved_by');
        $competencycards->created_by = $request->input('created_by');
        $competencycards->company_id = auth()->user()->company_id;

        // Gestion du fichier 'coursePlan'
        if ($request->hasFile('coursePlan')) {
            $file = $request->file('coursePlan');
            $fileExtension = $file->getClientOriginalExtension();
            $filePath = '';

            // Définir le chemin de stockage en fonction de l'extension
            if (in_array($fileExtension, ['pdf', 'docx'])) {
                $filePath = $file->store("course-plans/{$fileExtension}s", 'public');
            } elseif (in_array($fileExtension, ['jpg', 'jpeg', 'png'])) {
                $filePath = $file->store('course-plans/images', 'public');
            }

            // Assigner le chemin du fichier au champ 'curriculum'
            $competencycards->curriculum = $filePath;
        }

        // Sauvegarder les changements
        $competencycards->save();

        // Retourner une réponse JSON si la requête est AJAX
        if ($request->ajax()) {
            return response()->json([
                'message' => 'Carte de compétence mise à jour avec succès',
                'competencycards' => $competencycards,
            ]);
        }

        // Redirection en cas de succès
        return redirect()->route('competencycardrequests.index', $locale)
                         ->with('success', 'Carte Compétence mise à jour avec succès!');
    } catch (\Exception $e) {
        // Gestion des erreurs
        return redirect()->back()->with('error', 'Une erreur est survenue : ' . $e->getMessage());
    }
}

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        //
        $competencycards = CompetencyCard::find($id);
        $competencycards->delete();
        return redirect()->route('competencycardrequests.index');
    }
}
