<?php

namespace App\Http\Controllers\CompetencyCards;

use App\Http\Controllers\Controller;
use App\Models\Competency;
use App\Models\TrainingService;
use Illuminate\Http\Request;

class CompetencyServiceController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        //
        $services = TrainingService::all();
        $competencies = Competency::all();
        return view('management.competency_cards.competency_service', compact('competencies', 'services'));
    }


    public function list()
    {
        //

        $services = TrainingService::all();
        $competencies = Competency::all();
        $services = TrainingService::with('competencies')->get();
        
        return view('management.competency_cards.competency_service_list', compact('services', 'competencies'));
    }

    /**

     * Show the form for creating a new resource.

     */

    public function create($locale)

    {

        // Get all competencies to populate the select dropdown

        $competencies = Competency::all();
        $services = TrainingService::all();

        return view('management.competency_cards.competency_service', compact('competencies', 'services'));
    }


    /**
 
     * Store a newly created resource in storage.
 
     */

    public function store($locale, Request $request)

    {

        $request->validate([

            'competence' => 'required',

            'existingService' => 'required',

            'newService' => 'required|required_without:existingService',

            'coursePlan' => 'nullable|mimes:pdf,docx,jpg,jpeg,png|max:2048', // adjust the mime type and max size as needed

        ]);

        $trainingService = new TrainingService();

        $trainingService->name = $request->input('newService') ?? $request->input('existingService');

        $trainingService->competency_id = $request->input('competence');

        $trainingService->company_id = auth()->user()->company_id; // assuming you have a company ID associated with the user

        if ($request->hasFile('coursePlan')) {

            $file = $request->file('coursePlan');

            $fileName = $file->getClientOriginalName();

            $fileExtension = $file->getClientOriginalExtension();

            $filePath = '';



            switch ($fileExtension) {

                case 'pdf':

                    $filePath = $file->store('course-plans/pdfs', 'public');

                    break;

                case 'docx':

                    $filePath = $file->store('course-plans/docx', 'public');

                    break;

                case 'jpg':

                case 'jpeg':

                    $filePath = $file->store('course-plans/images', 'public');

                    break;
                case 'png':

                    $filePath = $file->store('course-plans/images', 'public');

                    break;

                default:

                    // Gérer les erreurs pour les types de fichiers non autorisés

                    break;
            }


            $trainingService->curriculum = $filePath;
            // Récupérer l'URL du fichier uploadé
            $fileUrl = asset('storage/' . $filePath);
        }


        $trainingService->save();

        if ($request->ajax()) {
            return response()->json($trainingService);
        }
        return redirect()->route('competencyservice.list', $locale)->with('success', 'Service créé avec succès!');
    }
    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($locale, string $id)
    {
        // Votre logique ici
        $competencies = Competency::all();
        $trainingService = TrainingService::findOrFail($id);
        return view('management.competency_cards.competency_service', compact('trainingService', 'competencies'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update($locale, Request $request, string $id)
    {
        $request->validate([

            'competence' => 'required',

            'existingService' => 'required',

            'newService' => 'required|required_without:existingService',

            'coursePlan' => 'nullable|mimes:pdf,docx,jpg,jpeg,png|max:2048', // adjust the mime type and max size as needed


        ]);


        $trainingService = TrainingService::findOrFail($id);
        $trainingService->name = $request->input('newService') ?? $request->input('existingService');

        $trainingService->competency_id = $request->input('competence');

        $trainingService->company_id = auth()->user()->company_id; // assuming you have a company ID associated with the user

        if ($request->hasFile('coursePlan')) {

            $file = $request->file('coursePlan');

            $fileName = $file->getClientOriginalName();

            $fileExtension = $file->getClientOriginalExtension();

            $filePath = '';


            switch ($fileExtension) {

                case 'pdf':

                    $filePath = $file->store('course-plans/pdfs', 'public');

                    break;

                case 'docx':

                    $filePath = $file->store('course-plans/docx', 'public');

                    break;

                case 'jpg':

                case 'jpeg':

                    $filePath = $file->store('course-plans/images', 'public');

                    break;
                case 'png':

                    $filePath = $file->store('course-plans/images', 'public');

                    break;

                default:

                    // Gérer les erreurs pour les types de fichiers non autorisés

                    break;
            }


            $trainingService->curriculum = $filePath;
        }

        $trainingService->save();



        return redirect()->route('competencyservice.list', $locale)->with('success', 'department updated successfully');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy($locale, string $id)
    {
        $services = TrainingService::findOrFail($id);
        $services->delete();

        return redirect()->route('competencyservice.list', $locale)->with('success', 'Company deleted successfully.');
    }
}
