<?php

namespace App\Http\Controllers\CompetencyCards;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\CompetencyUser;
use App\Models\TrainingService;
use App\Models\User;
use App\Models\Competency; // assuming you have a Competency model

class CompetencyUserController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function list()
    {
        $competencyUsers = CompetencyUser::with(['competency', 'service', 'user'])
            ->where('company_id', $this->company->id)
            ->orderBy('id', 'asc') // Tri par le prénom de l'utilisateur
            ->get();
        $competencies = Competency::all();
        $services = TrainingService::all();
        $users = User::all();

        return view('management.competency_cards.competency_users_list', compact('competencies', 'competencyUsers', 'services', 'users'));
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index($id)
    {
        //$competencyUsers = CompetencyUser::all();
        $competencyUsers = CompetencyUser::with(['competency', 'service', 'user'])
            ->where('company_id', $this->company->id)
            ->get();
        $competencies = Competency::all();
        $services = TrainingService::all();
        $users = User::all();
        return view('management.competency_cards.competency_users', compact('competencyUsers', 'users', 'competencies', 'services'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $users = User::all();
        $competencies = Competency::all(); // assuming you have a Competency model
        $services = TrainingService::all();
        return view('management.competency_cards.competency_users', compact('users', 'competencies', 'services'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store($locale, Request $request)
    {
        try {
            // Validation des données
            $request->validate([
                'user_id' => 'nullable',
                'competency_id' => 'required',
                'service_id' => 'required',
                'date' => 'required|date',
                'validity' => 'required',
                'coursePlan' => 'nullable|mimes:pdf,docx,jpg,jpeg,png|max:2048',
            ]);

            // Création d'un nouvel objet CompetencyUser
            $competencyUser = new CompetencyUser();
            $competencyUser->user_id = $request->input('user_id');
            $competencyUser->competency_id = $request->input('competency_id');
            $competencyUser->service_id = $request->input('service_id');
            $competencyUser->date = $request->input('date');
            $competencyUser->validity = $request->input('validity');
            $competencyUser->company_id = auth()->user()->company_id;

            // Gestion du fichier 'coursePlan'
            if ($request->hasFile('coursePlan')) {
                $file = $request->file('coursePlan');
                $fileExtension = $file->getClientOriginalExtension();
                $filePath = '';

                if (in_array($fileExtension, ['pdf', 'docx'])) {
                    $filePath = $file->store("course-plans/{$fileExtension}s", 'public');
                } elseif (in_array($fileExtension, ['jpg', 'jpeg', 'png'])) {
                    $filePath = $file->store('course-plans/images', 'public');
                }

                $competencyUser->certification = $filePath;
            }

            // Sauvegarde des données
            $competencyUser->save();

            // Réponse Ajax ou redirection
            if ($request->ajax()) {
                return response()->json($competencyUser);
            }

            return redirect()->route('competencyuser.list', $locale)->with('success', 'Service créé avec succès!');
        } catch (\Exception $e) {
            // Capture et affichage de l'erreur
            return redirect()->back()->with('error', 'Une erreur est survenue : ' . $e->getMessage());
        }
    }


    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $competencyUser = CompetencyUser::find($id);
        $services = TrainingService::all();
        $users = User::all();
        $competencies = Competency::all(); // assuming you have a Competency model
        return view('management.competancy_cards.competencyuser.edit', compact('competencyUser', 'users', 'competencies', 'services'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update($locale, Request $request, $id)
    {
        try {
            // Validation des données
            $request->validate([
                'user_id' => 'nullable',
                'competency_id' => 'required',
                'service_id' => 'required',
                'date' => 'required|date',
                'validity' => 'required',
                'coursePlan' => 'nullable|mimes:pdf,docx,jpg,jpeg,png|max:2048',
            ]);

            // Récupérer l'objet CompetencyUser existant
            $competencyUser = CompetencyUser::findOrFail($id); // Trouve ou échoue si non trouvé

            // Mise à jour des champs
            $competencyUser->user_id = $request->input('user_id');
            $competencyUser->competency_id = $request->input('competency_id');
            $competencyUser->service_id = $request->input('service_id');
            $competencyUser->date = $request->input('date');
            $competencyUser->validity = $request->input('validity');
            $competencyUser->company_id = auth()->user()->company_id;

            // Gestion du fichier 'coursePlan'
            if ($request->hasFile('coursePlan')) {
                $file = $request->file('coursePlan');
                $fileExtension = $file->getClientOriginalExtension();
                $filePath = '';

                if (in_array($fileExtension, ['pdf', 'docx'])) {
                    $filePath = $file->store("course-plans/{$fileExtension}s", 'public');
                } elseif (in_array($fileExtension, ['jpg', 'jpeg', 'png'])) {
                    $filePath = $file->store('course-plans/images', 'public');
                }

                // Mettre à jour le chemin du fichier de certification
                $competencyUser->certification = $filePath;
            }

            // Sauvegarde des données mises à jour
            $competencyUser->save();

            // Réponse Ajax ou redirection
            if ($request->ajax()) {
                return response()->json($competencyUser);
            }

            return redirect()->route('competencyuser.list', $locale)->with('success', 'Service mis à jour avec succès!');
        } catch (\Exception $e) {
            // Capture et affichage de l'erreur
            return redirect()->back()->with('error', 'Une erreur est survenue : ' . $e->getMessage());
        }
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $competencyUser = CompetencyUser::find($id);
        $competencyUser->delete();
        return redirect()->route('competencyuser.index');
    }
}
