<?php

namespace App\Http\Controllers;

use App\Models\Company;
use App\Models\Setting;
use App\Models\UserSetting;
use Carbon\Carbon;
use Carbon\CarbonInterval;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Lang;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\View;
use ZipArchive;

use function PHPUnit\Framework\isNull;

abstract class Controller
{
    protected $setting;
    protected $locale;
    protected $user_setting;
    protected $company;
    protected $user;

    protected $path_documents;
    protected $path_images;
    protected $path_videos;
    protected $path_directories;
    protected $path_scorms;

    protected $request_data;

    /**
     * contruct
     */
    public function __construct(Request $request)
    {
        // locale
        $locale = $request->segment(1);
        if (in_array($locale, config('languages')))
        {
            App::setLocale($locale);
        }
        else
        {
            $locale = env('APP_LOCALE', 'en');
            App::setLocale($locale);
        }
        View::share('locale', $locale);
        $this->locale = $locale;

        // setting
        $setting = Setting::where('status', '=', 1)->first();
        $this->setting = $setting;
        View::share('setting', $setting);

        // user_setting
        $users_id = [0];
        if (Auth::check())
        {
            $user = Auth::user();
            $this->user = $user;
            $users_id[] = $user->id;
            View::share('user', $user);
            $company = Company::where('id', '=', $user->company_id)->first();
            $this->company = $company;
            View::share('company', $company);
        }
        $user_setting = UserSetting::whereIn('user_id', $users_id)->orderBy('user_id', 'DESC')->first();
        if ($user_setting->user_id == 0 && Auth::check())
        {
            $user_id = Auth::user()->id;
            $user_setting = new UserSetting(['id' => $user_id, 'user_id' => $user_id]);
            $user_setting->save();
        }
        $this->user_setting = $user_setting;
        View::share('user_setting', $user_setting);

        $this->set_path();
    }

    /**
     * sec_to_time
     */
    public function sec_to_time($seconds)
    {
        return gmdate("H:i", $seconds);
        // CarbonInterval::seconds(3600)->cascade()->forHumans('His');
        $duration = Carbon::now()->subSeconds($seconds);
        $placeholder = [
            'h' => $duration->format('H'),
            'm' => $duration->format('i'),
            's' => $duration->format('s')
        ];
        return __(':h::m::s', $placeholder);
    }

    /**
     * sec_to_time_full
     */
    public function sec_to_time_full($seconds)
    {
        return gmdate("H:i:s", $seconds);
        // CarbonInterval::seconds(3600)->cascade()->forHumans('His');
        $duration = Carbon::now()->subSeconds($seconds);
        $placeholder = [
            'h' => $duration->format('H'),
            'm' => $duration->format('i'),
            's' => $duration->format('s')
        ];
        return __(':h::m::s', $placeholder);
    }

    /**
     * second_to_time
     */
    public function second_to_time($seconds)
    {
        $minute = floor($seconds / 60);
        return $this->minute_to_time($minute);
    }

    /**
     * time_to_second
     */
    public function time_to_second($time) 
    {
        $list_time = explode(':', $time, 3);
        if (count($list_time) == 1) 
        {
            $list_time[] = 0;
            $list_time[] = 0;
        }
        else if (count($list_time) == 2) 
        {
            $list_time[] = 0;
        }
        list($hours, $minutes, $seconds) = $list_time;
        return ($hours * 3600) + ($minutes * 60) + $seconds;
    }

    /**
     * minute_to_time
     */
    public function minute_to_time($minutes)
    {
        $hours = floor($minutes / 60);
        $remainingMinutes = $minutes % 60;
        $formattedTime = sprintf("%02d:%02d:00", $hours, $remainingMinutes);
        return $formattedTime;
    }

    /**
     * time_to_minute
     */
    public function time_to_minute($time) 
    {
        $list_time = explode(':', $time, 3);
        if (count($list_time) == 1) 
        {
            $list_time[] = 0;
            $list_time[] = 0;
        }
        else if (count($list_time) == 2) 
        {
            $list_time[] = 0;
        }
        list($hours, $minutes, $seconds) = $list_time;
        return ($hours * 60) + $minutes;
    }

    /**
     * get_not_null
     */
    public function get_not_null($data=array())
    {
        $new_data = array();
        foreach ($data as $key => $val)
        {
            if ($val != null) 
            {
                $new_data[$key] = $val;
            }  
        }
        return $new_data;
    }

    /**
     * check_company
     */
    protected function check_company($object)
    {
        if (isset($object->company_id)) 
        {
            if ($object->company_id != $this->company->id)
            {
                abort(403, __("message.unauthorized_access"));
            }
        }
    }

    /**
     * not found
     */
    protected function not_found()
    {
        abort(404, __("message.not_found"));
    }

    /**
     * create directory for company
     */
    public function set_path()
    {
        $path_public = "public";
        $path_company = md5("PAELO*"."0"."-Company/2024");
        if (isset($this->company) && $this->company->id)
        {
            $path_company = md5("PAELO*".$this->company->id."-Company/2024");
        }
        $path_documents = "$path_public/documents/$path_company/";
        $path_images = "$path_public/images/$path_company/";
        $path_videos = "$path_public/videos/$path_company/";
        $path_directories = "$path_public/directories/$path_company/";
        $path_scorms = "$path_public/scorms/$path_company/";
        if (!Storage::directoryExists($path_documents)) Storage::makeDirectory($path_documents);
        if (!Storage::directoryExists($path_images)) Storage::makeDirectory($path_images);
        if (!Storage::directoryExists($path_videos)) Storage::makeDirectory($path_videos);
        if (!Storage::directoryExists($path_directories)) Storage::makeDirectory($path_directories);
        if (!Storage::directoryExists($path_scorms)) Storage::makeDirectory($path_scorms);
        $this->path_documents = $path_documents;
        $this->path_images = $path_images;
        $this->path_videos = $path_videos;
        $this->path_directories = $path_directories;
        $this->path_scorms = $path_scorms;
        return [
            "documents" => $path_documents,
            "images" => $path_images,
            "videos" => $path_videos,
            "directories" => $path_directories,
            "scorms" => $path_scorms,
        ];
    }

    /**
     * get directory for company
     */
    public function get_path($path)
    {
        $full_path = $this->path_documents;
        eval('$full_path = $this->path_' . $path . ';');
        return $full_path;
    }

    /**
     * store files
     */
    public function store_files($files, $path)
    {
        switch ($path)
        {
            case 'images':
                $path = $this->path_images;
                break;
            case 'videos':
                $path = $this->path_videos;
                break;
            case 'documents':
                $path = $this->path_documents;
                break;
            case 'directories':
                $path = $this->path_directories;
                break;
            case 'scorms':
                $path = $this->path_scorms;
                break;
            default:
                $path = $this->path_documents;
                break;
        }
        if (!is_array($files))
        {
            $files = [$files];
        }
        $file_paths = [];
        foreach ($files as $file) 
        {
            $file_name = $file->getClientOriginalName();
            // $extension = $file->getClientOriginalExtension();
            $store_name = $file_name/* . '.' . $extension*/;
            $file_path = $file->store($path);
            $file_path = str_replace("//", "/", $file_path);
            $file_paths[$store_name] = $file_path;
        }
        return $file_paths;
    }

    /**
     * delete paths
     */
    public function delete_paths($paths)
    {
        $results = [];
        if (!is_array($paths))
        {
            $paths = [$paths];
        }
        foreach ($paths as $path)
        {
            $result = "not_found";
            if (Storage::exists($path) && Storage::fileExists($path))
            {
                // $path = Storage::path($path);
                $result = Storage::delete($path);
            }
            if (Storage::exists($path) && Storage::directoryExists($path)) 
            {
                // $path = Storage::path($path);
                $result = Storage::deleteDirectory($path);
            }
            $results[$path] = $result;
        }
        return $results;
    }

    /**
     * File Download
     */
    public function download($locale, $filename)
    {
        $downloads = Session::get('downloads', []);
        if (array_key_exists($filename, $downloads))
        {
            $filepath = $downloads[$filename];
            if (Storage::exists($filepath)) 
            {
                return Storage::download($filepath, $filename);
            }
        }
        abort(404, __('message.file_not_found'));
    }


    /**
     * unserialize
     */
    public function unserialize($string)
    {
        try 
        {
            $array = unserialize($string);
        }
        catch (\Exception $e) 
        {
            $array = [];
        }

        if ($array == false)
        {
            $array = [];
        }
        return $array;
    }

    /**
     * set_qb
     */
    protected function set_qb($key, $column, $operator, &$query_builder) 
    {
        if ($operator == 'like')
        {
            $value = $this->request_data[$key] ?? false;
            $query_builder = $value ? $query_builder->where($column, $operator, "%$value%") : $query_builder;
        }
        else if ($operator == '=')
        {
            $value = $this->request_data[$key] ?? false;
            $query_builder = $value != "" ? $query_builder->where($column, $operator, $value) : $query_builder;
        }
        else if ($operator == 'between')
        {
            $start = $this->request_data[$key]['start'] ?? false;
            $end = $this->request_data[$key]['end'] ?? false;
            if (in_array($key, ['completion_date']))
            {
                if ($start && $end)
                {
                    $query_builder = $query_builder->where(function ($query) use ($start, $end) {
                        $query->whereBetween('assigned_training.cert_date', [$start, $end])
                            ->orwhereBetween('assigned_training.attest_date', [$start, $end]);
                    });
                }
                else if ($start)
                {
                    $query_builder = $query_builder->where(function ($query) use ($start, $end) {
                        $query->whereBetween('assigned_training.cert_date', '>=', $start)
                            ->orwhereBetween('assigned_training.attest_date', '>=', $start);
                    });
                }
                else if ($end)
                {
                    $query_builder = $query_builder->where(function ($query) use ($start, $end) {
                        $query->whereBetween('assigned_training.cert_date', '<=', $end)
                            ->orwhereBetween('assigned_training.attest_date', '<=', $end);
                    });
                }
            }
            else
            {
                if ($start && $end)
                {
                    $query_builder = $query_builder->whereBetween($column, [$start, $end]);
                }
                else if ($start)
                {
                    $query_builder = $query_builder->where($column, '>=', $start);
                }
                else if ($end)
                {
                    $query_builder = $query_builder->where($column, '<=', $end);
                }
            }
        }
        
        return $query_builder;
    }
}
