<?php

namespace App\Http\Controllers\Lockouts;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\DeviceIcons;
use Storage;

class DeviceIconsController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $deviceIcons = DeviceIcons::latest()->paginate(env('DEFAULT_PAGINATION', 10));
        return view('lockouts.device_icons.index', compact('deviceIcons'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('lockouts.device_icons.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'icon' => 'required|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
            'description_en' => 'required|string',
            'description_fr' => 'required|string',
        ]);

        if($validatedData) {
            $iconName = time() . '_' . $request->file('icon')->getClientOriginalName();
            $request->file('icon')->storeAs(DeviceIcons::STORAGE, $iconName);

            $deviceIcon = new DeviceIcons();
            $deviceIcon->icon = $iconName;
            $deviceIcon->description_en = $validatedData['description_en'];
            $deviceIcon->description_fr = $validatedData['description_fr'];
            $deviceIcon->company_id = auth()->user()->company_id;
            $deviceIcon->save();
        }

        return redirect()->route('device_icons.index', app()->getLocale())->with('success', __('device_icons.message.success.create'));
    }

    /**
     * Display the specified resource.
     */
    public function show($locale, DeviceIcons $deviceIcon)
    {
        return view('lockouts.device_icons.show', compact('deviceIcon'));
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($locale, DeviceIcons $deviceIcon)
    {
        return view('lockouts.device_icons.edit', compact('deviceIcon'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update($locale, Request $request, DeviceIcons $deviceIcon)
    {
        $validatedData = $request->validate([
            'icon' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
            'description_en' => 'required|string',
            'description_fr' => 'required|string',
        ]);

        if($validatedData) {

            // Gérer l'icône si une nouvelle est fournie
            if ($request->hasFile('icon')) {
                $iconFile = $request->file('icon');

                // Vérifier si le fichier est une image valide
                if ($iconFile->isValid()) {
                    // Supprimer l'ancienne icône si elle existe
                    if ($deviceIcon->icon && $deviceIcon->icon !== DeviceIcons::DEFAULT_ICON) {
                        Storage::delete(DeviceIcons::STORAGE . '/' . $deviceIcon->icon);
                    }

                    // Stocker la nouvelle icône
                    $iconName = time() . '_' . $iconFile->getClientOriginalName();
                    $iconFile->storeAs(DeviceIcons::STORAGE, $iconName);
                    $deviceIcon->icon = $iconName;
                } else {
                    return back()->withErrors(['icon' => 'Le fichier d\'icône n\'est pas valide.'])->withInput();
                }
            }

            $deviceIcon->description_en = $validatedData['description_en'];
            $deviceIcon->description_fr = $validatedData['description_fr'];
            $deviceIcon->save();
        }

        return redirect()->route('device_icons.index', app()->getLocale())->with('success', __('device_icons.message.success.update'));
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy($locale, DeviceIcons $deviceIcon)
    {
        $deviceIcon->delete();
        return redirect()->route('device_icons.index', $locale)->with('success', __('device_icons.message.success.delete'));
    }
}
