<?php

namespace App\Http\Controllers\Lockouts;

use App\Http\Controllers\Controller;
use App\Models\Energies;
use App\Models\Energy;
use Illuminate\Http\Request;
use Illuminate\Support\Env;

class EnergiesController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $energies = Energies::latest('id')->paginate(env('PAGINATION_DEFAULT_LIMIT'));
        return view('energies.index', compact('energies'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('energies.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'label_fr' => 'required|string',
            'label_en' => 'required|string',
            'type' => 'required|in:ELECTRIQUE,MECANIQUE',
        ]);

        if($validatedData) {
            $energy = new Energies();
            $energy->label_fr = $validatedData['label_fr'];
            $energy->label_en = $validatedData['label_en'];
            $energy->type = $validatedData['type'];
            $energy->company_id = auth()->user()->company_id;
            $energy->save();
        }

        return redirect()->route('energies.index', app()->getLocale())->with('success', __('energies.create_success'));
    }

    /**
     * Display the specified resource.
     */
    public function show($locale, Energies $energy)
    {
        return view('energies.show', compact('energy'));
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($locale, Energies $energy)
    {
        return view('energies.edit', compact('energy'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $locale, Energies $energy)
    {
        $validatedData = $request->validate([
            'label_fr' => 'required|string',
            'label_en' => 'required|string',
            'type' => 'required|in:ELECTRIQUE,MECANIQUE',
        ]);

        if($validatedData) {
            $energy->label_fr = $validatedData['label_fr'];
            $energy->label_en = $validatedData['label_en'];
            $energy->type = $validatedData['type'];
            $energy->save();
        }

        return redirect()->route('energies.index', app()->getLocale())->with('success', __('energies.update_success'));
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy($locale, Energies $energy)
    {
        $energy->delete();

        return redirect()->route('energies.index', app()->getLocale())->with('success', __('energies.delete_success'));
    }
}
