<?php

namespace App\Http\Controllers\Lockouts;

use App\Http\Controllers\Controller;
use App\Models\Instructions;
use Illuminate\Http\Request;

class InstructionsController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $instructions = Instructions::latest()->paginate(env('PAGINATION_DEFAULT_LIMIT'));
        return view('lockouts.instructions.index', compact('instructions'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('lockouts.instructions.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'label_fr' => 'required|string',
            'label_en' => 'required|string',
        ]);

        if($validatedData) {
            $validatedData['company_id'] = auth()->user()->company->id;

            Instructions::create($validatedData);
        }

        return redirect()->route('instructions.index', app()->getLocale())->with('success', 'Instruction créée avec succès.');
    }

    /**
     * Display the specified resource.
     */
    public function show($locale, Instructions $instruction)
    {
        return view('lockouts.instructions.show', compact('instruction'));
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($locale, Instructions $instruction)
    {
        return view('lockouts.instructions.edit', compact('instruction'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $locale, Instructions $instruction)
    {
        $validatedData = $request->validate([
            'label_fr' => 'required|string',
            'label_en' => 'required|string',
        ]);

        if($validatedData) {
            $instruction->update($validatedData);
        }

        return redirect()->route('instructions.index', app()->getLocale())->with('success', 'Instruction modifiée avec succès.');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy($locale, Instructions $instruction)
    {
        $instruction->delete();
        return redirect()->route('instructions.index', app()->getLocale())->with('success', 'Instruction supprimée avec succès.');
    }
}
