<?php

namespace App\Http\Controllers\Lockouts;

use App;
use App\Http\Controllers\Controller;
use App\Models\Department;
use App\Models\Devices;
use App\Models\Location;
use App\Models\LockoutItems;
use App\Models\Lockouts;
use App\Models\Notes;
use App\Models\Subdepartment;
use App\Models\Trades;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use PHPUnit\Framework\MockObject\Generator\DuplicateMethodException;
use Storage;

class LockoutsController extends Controller
{
    public function index()
    {
        $lockouts = Lockouts::latest()->paginate(env('PAGINATION_DEFAULT_LIMIT', 10));

        return view('lockouts.index', [
            'lockouts' => $lockouts
        ]);
    }

    public function downloadPdf()
    {
        //download a pdf file from project directory
        $file = public_path(). "/lockouts/lockout_blank.pdf";
        //rename the file to be downloaded
        $headers = array(
            'Content-Type: application/pdf',
        );

        if(!file_exists($file)) {
            return redirect()->back()->with('error', trans('app.file_not_found', ['file' => 'lockout_vierge.pdf']));
        }
        return response()->download($file, trans('lockouts.lockout_blank_filename'), $headers);
    }

    public function create()
    {
        $data = $this->getDependencies();

        return view('lockouts.create', $data);
    }

    public function store(Request $request)
    {
        $validated = $request->validate([
            'equipment_id' => 'required|string|max:255',
            'equipment_description' => 'required|string|max:255',
            'equipment_image' => 'required|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
            'equipment_location' => 'required|string|max:255',
            'code' => 'required|string|max:255',
            'version' => 'required|string|max:255',
            'title' => 'required|string|max:255',
            'department_id' => 'required|exists:departments,id',
            'sub_department_id' => 'required|exists:subdepartments,id',
            'location_id' => 'required|exists:locations,id',
        ]);

        if($validated) {
            $imageName = time().'.'.$request->equipment_image->extension();
            $request->equipment_image->move(Lockouts::STORAGE, $imageName);

            $lockout = new Lockouts();
            $lockout->equipment_id = $validated['equipment_id'];
            $lockout->equipment_description = $validated['equipment_description'];
            $lockout->equipment_image = $imageName;
            $lockout->code = $validated['code'];
            $lockout->version = $validated['version'];
            $lockout->title = $validated['title'];
            $lockout->status = 'draft';
            $lockout->created_by = auth()->user()->id;
            $lockout->company_id = auth()->user()->company_id;
            $lockout->department_id = $validated['department_id'];
            $lockout->sub_department_id = $validated['sub_department_id'];
            $lockout->location_id = $validated['location_id'];
            $lockout->save();
        }

        return redirect()->route('lockouts.index', app()->getLocale())->with('success', trans('lockouts.message.create_lockout_success'));
    }

    public function edit($locale, Lockouts $lockout)
    {
        $data = $this->getDependencies();
        return view('lockouts.edit', $data + compact('lockout'));
    }

    public function update(Request $request, $locale, Lockouts $lockout)
    {
        $validated = $request->validate([
            'equipment_id' => 'required|string|max:255',
            'equipment_description' => 'required|string|max:255',
            'equipment_image' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
            'equipment_location' => 'required|string|max:255',
            'code' => 'required|string|max:255',
            'version' => 'required|string|max:255',
            'title' => 'required|string|max:255',
            'department_id' => 'required|exists:departments,id',
            'sub_department_id' => 'required|exists:subdepartments,id',
            'location_id' => 'required|exists:locations,id',
        ]);

        if($validated) {
            if ($request->hasFile('equipment_image')) {
                $equipment_image = $request->file('equipment_image');

                if ($equipment_image->isValid()) {
                    if ($lockout->equipment_image && $lockout->equipment_image !== Lockouts::DEFAULT_IMAGE) {
                        Storage::delete(Lockouts::STORAGE . '/' . $lockout->equipment_image);
                    }

                    $imageName = time() . '_' . $equipment_image->getClientOriginalName();

                    $equipment_image->storeAs(Lockouts::STORAGE, $imageName);
                    $validated['equipment_image'] = $imageName;
                } else {
                    return back()->withErrors(['equipment_image' => 'L\'image de la fiche n\'est pas valide.'])->withInput();
                }
            }

            $lockout->update($validated);
        }



        return redirect()->route('lockouts.index', app()->getLocale())->with('success', trans('lockouts.message.update_lockout_success'));
    }

    public function updateHeader(Request $request, $locale, Lockouts $lockout)
    {
        $validated = $request->validate([
            'equipment_id' => 'required|string|max:255',
            'equipment_description' => 'required|string|max:255',
            'equipment_image' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
            'equipment_location' => 'required|string|max:255',
            'code' => 'required|string|max:255',
            'version' => 'required|string|max:255',
            'title' => 'required|string|max:255',
            'department_id' => 'required|exists:departments,id',
            'sub_department_id' => 'required|exists:subdepartments,id',
            'location_id' => 'required|exists:locations,id',
            'status' => 'required|string|max:255',
        ]);

        

        if($validated) {
            if ($request->hasFile('equipment_image')) {
                $equipment_image = $request->file('equipment_image');

                if ($equipment_image->isValid()) {
                    if ($lockout->equipment_image && $lockout->equipment_image !== Lockouts::DEFAULT_IMAGE) {
                        Storage::delete(Lockouts::STORAGE . '/' . $lockout->equipment_image);
                    }

                    $imageName = time() . '_' . $equipment_image->getClientOriginalName();
                    $equipment_image->storeAs(Lockouts::STORAGE, $imageName);
                    $lockout->equipment_image = $imageName;
                } else {
                    return back()->withErrors(['equipment_image' => 'L\'image de la fiche n\'est pas valide.'])->withInput();
                }
            }

            if($lockout->status == 'draft' && $validated['status'] == 'writed') {
                $lockout->status = 'writed';
            }

            $lockout->update($validated);
        }



        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.update_lockout_success'))
            ->withFragment('header');
    }

    public function show($locale, Lockouts $lockout)
    {
        return view('lockouts.show', compact('lockout'));
    }

    public function print($locale, Lockouts $lockout)
    {
        $pdf = App::make('snappy.pdf');
        $pdf->setOption('enable-local-file-access', true);
        $pdf->setOption('encoding', 'UTF-8');
        $pdf->setOption('no-stop-slow-scripts', true);
        
        // Add header and footer options
        $pdf->setOption('header-html', view('lockouts.print.pdf_header', compact('lockout'))->render());
        $pdf->setOption('header-spacing', 10); // Increased spacing
        $pdf->setOption('footer-html', view('lockouts.print.pdf_footer', compact('lockout'))->render());
        $pdf->setOption('footer-spacing', 10); // Increased spacing
        $pdf->setOption('margin-top', 35); // Increased margin
        $pdf->setOption('margin-bottom', 35); // Increased margin
        $pdf->setOption('header-font-size', 12); // Added font size
        $pdf->setOption('footer-font-size', 12); // Added font size

        $html = view('lockouts.print', compact('lockout'))->render();

        $output = $pdf->getOutputFromHtml($html);
        $filename = $lockout->code . '_' . $lockout->version . '.pdf';

        return new Response($output, 200, [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'attachment; filename="' . $filename . '"'
        ]);
    }

    public function copy($locale, Lockouts $lockout)
    {
        return view('lockouts.copy', compact('lockout'));
    }

    public function link($locale, Lockouts $lockout)
    {
        return view('lockouts.link', compact('lockout'));
    }

    public function merge($locale, Lockouts $lockout)
    {
        return view('lockouts.merge', compact('lockout'));
    }

    public function configure($locale, Lockouts $lockout)
    {
        //filter trades where not in the lockout
        $trades = Trades::where('company_id', auth()->user()->company_id)->whereDoesntHave('lockouts', function ($query) use ($lockout) {
            $query->where('lockouts_trades.lockout_id', $lockout->id);
        })->get();

        //filter for lockouts_notes.type = PREPARATION only
        $notesPreparation = Notes::where('company_id', auth()->user()->company_id)->whereDoesntHave('lockouts', function ($query) use ($lockout) {
            $query->where('lockouts_notes.lockout_id', $lockout->id)->where('lockouts_notes.type', 'PREPA');
        })->get();

        //filter for lockouts_notes.type = DECONSIGNATION only
        $notesDeconsignation = Notes::where('company_id', auth()->user()->company_id)->whereDoesntHave('lockouts', function ($query) use ($lockout) {
            $query->where('lockouts_notes.lockout_id', $lockout->id)->where('lockouts_notes.type', 'DECAD');
        })->get();  

        $devices = Devices::where('company_id', auth()->user()->company_id)->whereDoesntHave('lockoutItems', function ($query) use ($lockout) {
            $query->where('lockout_items.lockout_id', $lockout->id);
        })->get();

        $device_notes = Notes::where('company_id', auth()->user()->company_id)->whereDoesntHave('lockoutItems', function ($query) use ($lockout) {
            $query->where('lockout_items.lockout_id', $lockout->id);
        })->get();

        $mode = 'configure';
        $statuses = Lockouts::STATUSES;
        $data = $this->getDependencies();

        return view('lockouts.configure', $data + compact('lockout', 'trades', 'notesPreparation', 'notesDeconsignation', 'devices', 'device_notes', 'mode', 'statuses'));
    }

    public function storeNotesDeconsignation($locale, Lockouts $lockout, Request $request)
    {
        $validated = $request->validate([
            'note_id' => 'required|exists:notes,id',
            'order' => 'nullable|integer|min:1'
        ]);

        if(isset($validated['order'])) {
            $nextOrder = intval($validated['order']);
        } else {
            $nextOrder = ($lockout->notesDeconsignation()->max('order') ?? 0) + 1;
        }

        $lockout->notesDeconsignation()->attach($validated['note_id'], ['order' => $nextOrder, 'type' => 'DECAD']);

        $lockout->notesDeconsignation()->where('order', '>=', $nextOrder)->where('lockouts_notes.note_id', '!=', $validated['note_id'])->orderBy('order')->get()->each(function ($note) {
            $note->pivot->update(['order' => $note->pivot->order + 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.add_note_success'))
            ->withFragment('release');

    }

    public function moveUpNotesDeconsignation($locale, Lockouts $lockout, $note_id)
    {
        $note = $lockout->notesDeconsignation()->get()->where('id', $note_id)->first();

        $lockout->notesDeconsignation()->updateExistingPivot($note_id, ['order' => $note->pivot->order - 1]);

        $lockout->notesDeconsignation()->where('lockouts_notes.order', '=', $note->pivot->order-1)->where('lockouts_notes.note_id', '!=', $note_id)->get()->each(function ($note) {
            $note->pivot->update(['order' => $note->pivot->order + 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.move_note_success'))
            ->withFragment('release');
    }

    public function moveDownNotesDeconsignation($locale, Lockouts $lockout, $note_id)
    {
        $note = $lockout->notesDeconsignation()->get()->where('id', $note_id)->first();

        $lockout->notesDeconsignation()->updateExistingPivot($note_id, ['order' => $note->pivot->order + 1]);

        $lockout->notesDeconsignation()->where('lockouts_notes.order', '=', $note->pivot->order+1)->where('lockouts_notes.note_id', '!=', $note_id)->get()->each(function ($note) {
            $note->pivot->update(['order' => $note->pivot->order - 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.move_note_success'))
            ->withFragment('release');
    }

    public function storeItems($locale, Lockouts $lockout, Request $request)
    {
        $validated = $request->validate([
            'device_id' => 'nullable|exists:devices,id',
            'note_id' => 'nullable|exists:notes,id',
            'order' => 'nullable|integer|min:1'
        ]);

        if(isset($validated['order'])) {
            $nextOrder = intval($validated['order']);
        } else {
            $nextOrder = ($lockout->lockoutItems()->max('order') ?? 0) + 1;
        }

        if(isset($validated['device_id'])) {
            // Configuration pour un device
            $itemableType = Devices::class;
            $itemableId = $validated['device_id'];
            
            $successMessage = 'lockouts.message.add_device_success';
        }

        if(isset($validated['note_id'])) {
            // Configuration pour une note
            $itemableType = Notes::class;
            $itemableId = $validated['note_id'];
            $successMessage = 'lockouts.message.add_note_success';
        }

        // Créer l'item
        $lockout->lockoutItems()->create([
            'itemable_type' => $itemableType,
            'itemable_id' => $itemableId,
            'order' => $nextOrder,
        ]);        

        $lockout->lockoutItems()->where('order', '>=', $nextOrder)->where('lockout_items.itemable_id', '!=', $itemableId)->orderBy('order')->get()->each(function ($item) {
            $item->update(['order' => $item->order + 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans($successMessage))
            ->withFragment('lockout');
    }

    public function moveUpItems($locale, Lockouts $lockout, $item_id)
    {
        $item = $lockout->lockoutItems()->where('lockout_items.itemable_id', '=', $item_id)->first();

        $item->update(['order' => $item->order-1]);

        $lockout->lockoutItems()->where('lockout_items.order', '=', $item->order)->where('lockout_items.itemable_id', '!=', $item_id)->get()->each(function ($_item) {
            $_item->update(['order' => $_item->order+1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.move_item_success'))
            ->withFragment('lockout');
    }

    public function moveDownItems($locale, Lockouts $lockout, $item_id)
    {
        $item = $lockout->lockoutItems()->where('lockout_items.itemable_id', '=', $item_id)->first();

        $item->update(['order' => $item->order+1]);

        $lockout->lockoutItems()->where('lockout_items.order', '=', $item->order)->where('lockout_items.itemable_id', '!=', $item_id)->get()->each(function ($_item) {
            $_item->update(['order' => $_item->order-1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.move_item_success'))
            ->withFragment('lockout');
    }

    public function destroyItems($locale, Lockouts $lockout, LockoutItems $item)
    {
        $item->delete();

        $lockout->lockoutItems()->orderBy('order')->get()->each(function ($item, $index) {
            $item->update(['order' => $index + 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.delete_item_success'))
            ->withFragment('lockout');
    }

    public function storeNotesPreparation($locale, Lockouts $lockout, Request $request)
    {
        $validated = $request->validate([
            'note_id' => 'required|exists:notes,id',
            'order' => 'nullable|integer|min:1'
        ]);

        if(isset($validated['order'])) {
            $nextOrder = intval($validated['order']);   
        } else {
            $nextOrder = ($lockout->notesPreparation()->max('order') ?? 0) + 1;
        }

        $lockout->notesPreparation()->attach($validated['note_id'], ['order' => $nextOrder]);

        $lockout->notesPreparation()->where('order', '>=', $nextOrder)->where('lockouts_notes.note_id', '!=', $validated['note_id'])->orderBy('order')->get()->each(function ($note) {
            $note->pivot->update(['order' => $note->pivot->order + 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.add_note_success'))
            ->withFragment('preparation');
    }

    public function moveUpNotesPreparation($locale, Lockouts $lockout, $note_id)
    {
        $note = $lockout->notesPreparation()->get()->where('id', $note_id)->first();    

        $lockout->notesPreparation()->updateExistingPivot($note_id, ['order' => $note->pivot->order - 1]);

        $lockout->notesPreparation()->where('lockouts_notes.order', '=', $note->pivot->order-1)->where('lockouts_notes.note_id', '!=', $note_id)->get()->each(function ($note) {
            $note->pivot->update(['order' => $note->pivot->order + 1]);
        });
        
        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.move_note_success'))
            ->withFragment('preparation');
    }

    public function moveDownNotesPreparation($locale, Lockouts $lockout, $note_id)
    {
        $note = $lockout->notesPreparation()->get()->where('id', $note_id)->first();

        $lockout->notesPreparation()->updateExistingPivot($note_id, ['order' => $note->pivot->order + 1]);

        $lockout->notesPreparation()->where('lockouts_notes.order', '=', $note->pivot->order+1)->where('lockouts_notes.note_id', '!=', $note_id)->get()->each(function ($note) {
            $note->pivot->update(['order' => $note->pivot->order - 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.move_note_success'))
            ->withFragment('preparation');
    }

    public function destroyNotesDeconsignation($locale, Lockouts $lockout, Notes $note)
    {
        $lockout->notes()->detach($note->id);

        $lockout->notesDeconsignation()->orderBy('order')->get()->each(function ($note, $index) {
            $note->pivot->update(['order' => $index + 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.delete_note_success'))
            ->withFragment('release');
    }

    public function destroyNotesPreparation($locale, Lockouts $lockout, Notes $note)
    {
        $lockout->notesPreparation()->detach($note->id);

        $lockout->notesPreparation()->orderBy('order')->get()->each(function ($note, $index) {
            $note->pivot->update(['order' => $index + 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.delete_note_success'))
            ->withFragment('preparation');
    }

    public function storeTrades($locale, Lockouts $lockout, Request $request)
    {
        $validated = $request->validate([
            'trade_id' => 'required|exists:trades,id',
            'order' => 'nullable|integer|min:1'
        ]);

        
        if(isset($validated['order'])) {
            $nextOrder = intval($validated['order']);
        } else {
            $nextOrder = ($lockout->trades()->max('order') ?? 0) + 1;
        }

        $lockout->trades()->attach($validated['trade_id'], ['order' => $nextOrder]);

        $lockout->trades()->where('order', '>=', $nextOrder)->where('lockouts_trades.trade_id', '!=', $validated['trade_id'])->orderBy('order')->get()->each(function ($trade) {
            $trade->pivot->update(['order' => $trade->pivot->order + 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.add_trade_success'))
            ->withFragment('business');
    }

    public function destroyTrade($locale, Lockouts $lockout, Trades $trade)
    {
        $lockout->trades()->detach($trade->id);
        $lockout->trades()->orderBy('order')->get()->each(function ($trade, $index) {
            $trade->pivot->update(['order' => $index + 1]);
        });
        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.delete_trade_success'))
            ->withFragment('business');
    }

    public function moveUpTrade($locale, Lockouts $lockout, $trade_id)
    {
        $trade = $lockout->trades()->get()->where('id', $trade_id)->first();
        $lockout->trades()->updateExistingPivot($trade_id, ['order' => $trade->pivot->order - 1]);

        $lockout->trades()->where('lockouts_trades.order', '=', $trade->pivot->order-1)->where('lockouts_trades.trade_id', '!=', $trade_id)->orderBy('order')->get()->each(function ($trade) {
            $trade->pivot->update(['order' => $trade->pivot->order + 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.move_trade_success'))
            ->withFragment('business');
    }

    public function moveDownTrade($locale, Lockouts $lockout, $trade_id)
    {
        $trade = $lockout->trades()->get()->where('id', $trade_id)->first();

        $lockout->trades()->updateExistingPivot($trade_id, ['order' => $trade->pivot->order + 1]);

        $lockout->trades()->where('lockouts_trades.order', '=', $trade->pivot->order+1)->where('lockouts_trades.trade_id', '!=', $trade_id)->get()->each(function ($trade) {
            $trade->pivot->update(['order' => $trade->pivot->order - 1]);
        });

        return redirect()->route('lockouts.configure', [$locale, $lockout])
            ->with('success', trans('lockouts.message.move_trade_success'))
            ->withFragment('business');
    }

    public function destroy($locale, Lockouts $lockout)
    {
        $lockout->delete();
        return redirect()->route('lockouts.index', app()->getLocale())
            ->with('success', trans('lockouts.message.delete_lockout_success'));
    }

    private function getDependencies()
    {
        return [
            'departments' => Department::where('company_id', auth()->user()->company_id)->get(),
            'subDepartments' => Subdepartment::where('company_id', auth()->user()->company_id)->get(),
            'locations' => Location::where('company_id', auth()->user()->company_id)->get()
        ];
    }
}
