<?php

namespace App\Http\Controllers\Lockouts;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Notes;
use Illuminate\Support\Facades\Storage;

class NotesController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $notes = Notes::latest('id')->paginate(env('PAGINATION_DEFAULT_LIMIT'));
        return view('notes.index', compact('notes'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('notes.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'label_fr' => 'required|string',
            'label_en' => 'required|string',
            'image' => 'required|image|mimes:jpeg,png,jpg,gif|max:2048'
        ]);

        if ($request->hasFile('image')) {
            $imageName = time() . '_' . $request->file('image')->getClientOriginalName();
            $request->file('image')->storeAs(Notes::STORAGE, $imageName);
            $validatedData['image'] = $imageName;
        }

        if($validatedData) {
            $validatedData['company_id'] = auth()->user()->company->id;
            Notes::create($validatedData);
        }

        return redirect()->route('notes.index', app()->getLocale())->with('success', 'Note créée avec succès.');
    }

    /**
     * Display the specified resource.
     */
    public function show($locale, Notes $note)
    {
        return view('notes.show', compact('note'));
    }

    /**
     * Show the form for editing the specified resource.
     * @param Notes $note
     *
     */
    public function edit($locale, Notes $note)
    {
        return view('notes.edit', compact('note'));
    }

    /**
     * Update the specified resource in storage.
     * @param Request $request
     * @param Notes $note
     * @return \Illuminate\Http\RedirectResponse
     */
    public function update(Request $request, $locale, Notes $note)
    {
        $validatedData = $request->validate([
            'label_fr' => 'required|string',
            'label_en' => 'required|string',
            'image' => 'sometimes|image|mimes:jpeg,png,jpg,gif|max:2048'
        ]);

        if ($request->hasFile('image')) {
            if ($note->image && $note->image !== Notes::DEFAULT_IMAGES) {
                Storage::delete(Notes::STORAGE . '/' . $note->image);
            }

            $imageName = time() . '_' . $request->file('image')->getClientOriginalName();
            $request->file('image')->storeAs(Notes::STORAGE, $imageName);
            $validatedData['image'] = $imageName;
        }

        $note->update($validatedData);

        return redirect()->route('notes.index', app()->getLocale())->with('success', 'Note mise à jour avec succès.');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy($locale, Notes $note)
    {
        Storage::disk('public')->delete($note->image);
        $note->delete();

        return redirect()->route('notes.index', app()->getLocale())->with('success', 'Note supprimée avec succès.');
    }
}
