<?php

namespace App\Http\Controllers\Lockouts;

use App\Http\Controllers\Controller;
use App\Models\Positions;
use Illuminate\Http\Request;

class PositionsController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $positions = Positions::latest('id')->paginate(env('PAGINATION_DEFAULT_LIMIT'));
        return view('lockouts.positions.index', compact('positions'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('lockouts.positions.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'label_fr' => 'required|string',
            'label_en' => 'required|string',
        ]);

        if($validatedData) {
            $validatedData['company_id'] = auth()->user()->company->id;

            Positions::create($validatedData);
        }

        return redirect()->route('positions.index', app()->getLocale())->with('success', 'Position créée avec succès.');
    }

    /**
     * Display the specified resource.
     */
    public function show($locale, Positions $position)
    {
        return view('lockouts.positions.show', compact('position'));
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($locale, Positions $position)
    {
        return view('lockouts.positions.edit', compact('position'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $locale, Positions $position)
    {
        $validatedData = $request->validate([
            'label_fr' => 'required|string',
            'label_en' => 'required|string',
        ]);

        if($validatedData) {
            $position->update($validatedData);
        }

        return redirect()->route('positions.index', app()->getLocale())->with('success', 'Position modifiée avec succès.');
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy($locale, Positions $position)
    {
        $position->delete();
        return redirect()->route('positions.index', app()->getLocale())->with('success', 'Position supprimée avec succès.');
    }
}
