<?php

namespace App\Http\Controllers\Lockouts;

use App\Http\Controllers\Controller;
use App\Models\Trades;
use Illuminate\Http\Request;

class TradeController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $trades = Trades::latest('id')->paginate(env('PAGINATION_DEFAULT_LIMIT'));
        return view('trades.index', compact('trades'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        return view('trades.create');
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $validatedData = $request->validate([
            'name' => 'required|string|max:255',
            'details' => 'required|string',
        ]);

        if($validatedData) {
            $trade = new Trades();
            $trade->name = $validatedData['name'];
            $trade->details = $validatedData['details'];
            $trade->company_id = auth()->user()->company_id;
            $trade->save();
        }

        return redirect()->route('trades.index', app()->getLocale())->with('success', __('trades.create_success'));
    }

    /**
     * Display the specified resource.
     */
    public function show($locale, Trades $trade)
    {
        return view('trades.show', compact('trade'));
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($locale, Trades $trade)
    {
        return view('trades.edit', compact('trade'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $locale, Trades $trade)
    {
        $validatedData = $request->validate([
            'name' => 'required|string|max:255',
            'details' => 'required|string',
        ]);

        if($validatedData) {
            $trade->name = $validatedData['name'];
            $trade->details = $validatedData['details'];
            $trade->save();
        }

        return redirect()->route('trades.index', app()->getLocale())->with('success', __('trades.update_success'));
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy($locale, Trades $trade)
    {
        $trade->delete();
        return redirect()->route('trades.index', app()->getLocale())->with('success', __('trades.delete_success'));
    }
}
