<?php

namespace App\Http\Controllers;

use App\Models\UserTraining;
use Barryvdh\DomPDF\Facade\Pdf;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Imagick;

class PDFController extends Controller
{
    /**
     * user_training_attestation
     */
    public function user_training_attestation(Request $request)
    {
        $data = [];

        $id = $request->id;
        $user_training = UserTraining::find($id);

        if ($user_training) 
        {
            $this->check_company($user_training);

            $title = $this->user->fname . " " . $this->user->lname . " - " . $user_training->training->name;
            $title = preg_replace('/[^A-Za-z0-9 -]/', '', $title);

            $data['user_training'] = $user_training;
            $data['training'] = $user_training->training;
            $data['title'] = $title;

            $pdf = Pdf::loadView('pdf.user-training-attestation', $data);
            $pdf->setPaper('letter', 'landscape');
            
            return $pdf->stream("$title.pdf");
        } 
        else 
        {
            $this->not_found();
        }
    }

    /**
     * pdf_to_images
     */
    public static function pdf_to_images($pdf_path, $files_path)
    {
        $pdf_file = Storage::path($pdf_path);

        $imagick = new Imagick();
        
        // Définir une haute résolution avant de charger le fichier PDF
        $imagick->setResolution(600, 600);
        $imagick->readImage($pdf_file);

        $images = [];
        foreach ($imagick as $i => $page) 
        {
            // Définir le format et la qualité de l'image sans redimensionner
            $page->setImageFormat('jpg');
            $page->setImageCompression(Imagick::COMPRESSION_JPEG);
            $page->setImageCompressionQuality(100);

            $imageName = 'page-' . ($i + 1) . '.jpg';
            $imagePath = $files_path . $imageName;

            // Sauvegarder l'image dans le chemin spécifié
            $page->writeImage(Storage::path($imagePath));
            // Ajouter le chemin de l'image au tableau
            $images[$imageName] = $imagePath;

            // Définir les permissions du fichier créé (note : le chemin doit être absolu)
            chmod(Storage::path($imagePath), 0775);
        }

        $imagick->clear();
        $imagick->destroy();

        return $images;
    }
}
