<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Activity;
use App\Models\AssignedTraining;
use App\Models\Trainer;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;

class ActivityController extends Controller
{
    /**
     * index
     */
    public function index(Request $request, $locale)
    {
        $assigned_training_id = $request->assigned_training_id;

        $assigned_training = AssignedTraining::find($assigned_training_id);
        
        if ($assigned_training) 
        {
            $data = [];

            $parameters = Session::get('assigned_training_filter', []);
            $parameters['locale'] = $locale;
            $data['parameters'] = $parameters;
            $data['trainers'] = Trainer::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
            $data['assigned_training'] = $assigned_training;
            $data['activities_status'] = ['Not Completed' => __('app.not_completed'), 'Completed' => __('app.completed')];
            $data['activities_type'] = ['Theory' => __('app.Theory'), 'Practice' => __('app.Practice')];

            return view('training.activity.list', $data);
        }
        else
        {
            $this->not_found();
        }
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $id = $request->input('id');
        $presentiel_id = $request->input('presentiel_id');
        $assigned_training_id = $request->input('assigned_training_id');
        $trainer_id = $request->input('trainer_id');
        $date = $request->input('date');
        $status = $request->input('status');
        $type = $request->input('type');
        $duration = $request->input('duration');
        $notes = $request->input('notes');

        $data = array(
            'trainer_id' => $trainer_id,
            'duration' => $this->time_to_second($duration),
            'status' => $status,
            'notes' => $notes
        );
        $data = $this->get_not_null($data);
        
        $activity = Activity::find($id);
        if ($activity) 
        {
            $activity->update($data);

            AssignedTrainingController::complete($assigned_training_id);
        }
        else
        {
            if ($id)
            {
                $this->not_found();
            }
            else
            {
                $data['assigned_training_id'] = $assigned_training_id;
                $data['date'] = $date;
                $data['type'] = $type;
                $activity = Activity::create($data);

                AssignedTrainingController::complete($assigned_training_id);
            }
        }
        
        if ($presentiel_id)
        {
            return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('success', __('message.successfully_registered'));
        }
        else
        {
            $assigned_training_id = $activity->assigned_training_id;
            return redirect()->route('activity.list', [$locale, $assigned_training_id])->with('success', __('message.successfully_registered'));
        }
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $presentiel_id = $request->presentiel_id;

        $activity = Activity::find($id);
        if ($activity)
        {
            // $assigned_training = $activity->assigned_training;
            // $assigned_training->delete();
            $assigned_training_id = $activity->assigned_training_id;
            $activity->delete();

            AssignedTrainingController::complete($assigned_training_id);

            if ($presentiel_id)
            {
                return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('success', __('message.successfully_deleted'));
            }
            else
            {
                return redirect()->route('activity.list', [$locale, $assigned_training_id])->with('success', __('message.successfully_registered'));
            }
        } 
        else
        {
            if ($presentiel_id)
            {
                return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('error', __('message.error_deleting'));
            }
            else
            {
                $this->not_found();
            }
        }
    }

    /**
     * upload
     */
    public function upload(Request $request, $locale)
    {  
        $data = [];

        $id = $request->id;

        $activity = Activity::find($id);

        if (!$activity) 
        {
            $this->not_found();
        }

        $data["activity"] = $activity;

        $parameters = Session::get('assigned_training_filter', []);
        $parameters['locale'] = $locale;
        $data['parameters'] = $parameters;

        return view('training.activity.upload', $data);
    }

    /**
     * store_upload
     */
    public function store_upload(Request $request, $locale)
    {
        $activity_id = $request->input('activity_id');
        $activity = Activity::find($activity_id);

        if ($activity)
        {
            if ($request->hasFile('documents')) 
            {
                $file_paths = $this->store_files($request->file('documents'), 'documents');
                $activity->documents = serialize($file_paths);
                $activity->save();
            }

            $parameters = Session::get('assigned_training_filter', []);
            $parameters['locale'] = $locale;

            return redirect()->route('assigned-training.result', $parameters)->with('success', __('message.successfully_registered'));
        }
        else
        {
            $this->not_found();
        }
    }

    /**
     * get
     */
    public function get(Request $request)
    {
        $id = $request->id ? : 0;
        
        $activity = Activity::find($id);

        if ($activity) 
        {
            $activity->assigned_training = $activity->assigned_training;
            $activity->assigned_training->training = $activity->assigned_training->training;
            $activity->duration = $this->sec_to_time_full($activity->duration);
        }

        return $activity;
    }

    /**
     * complete
     */
    public function complete(Request $request, $locale)
    {
        $id = $request->id;
        
        $activity = Activity::find($id);
        if (!$activity)
        {
            $this->not_found();
        }

        $activity->status = 'Completed';
        $activity->completed = 1;
        $activity->save();
        
        AssignedTrainingController::complete($activity->assigned_training_id);

        return redirect()->route('activity.list', [$locale, $activity->assigned_training_id])->with('success', __('message.successfully_registered'));
    }
}
