<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Answer;
use App\Models\Module;
use App\Models\Task;
use App\Models\Training;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class AnswerController extends Controller
{
    /**
     * index
     */
    public function index(Request $request, $locale, $training_id, $module_id, $id)
    {
        $data = array();

        $training = Training::find($training_id);
        $this->check_company($training);
        $data['training'] = $training;

        $module = Module::find($module_id);
        $this->check_company($module);
        $data['module'] = $module;

        $task = Task::find($id);
        if (!$task) 
        {
            $this->not_found();
        }
        $data['task'] = $task;
        $data['answers'] = $task->answers;

        return view('training.module.answer', $data);
    }

    /**
     * complete
     */
    public function complete(Request $request, $locale)
    {
        $id = $request->input('id');
        $module_id = $request->input('module_id');
        $training_id = $request->input('training_id');

        $task = Task::find($id);
        if (!$task) 
        {
            $this->not_found();
        }
        
        if (in_array($task->answer_type, array(4, 5)) && !count($task->answers))
        {
            return redirect()->route('answer.index', [$locale, $training_id, $module_id, $id])->with('error_stay', __('message.you_must_set_1_answer_correct'));
        }
        else if (count($task->answers) < 2)
        {
            return redirect()->route('answer.index', [$locale, $training_id, $module_id, $id])->with('error_stay', __('message.not_enough_answers'));
        }
        
        return redirect()->route('module.view', [$locale, $module_id, $training_id])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale, $training_id, $module_id, $task_id)
    {
        $id = $request->id;
        $answer = Answer::find($id);

        if ($answer) 
        {
            if ($answer->filepath)
            {
                $result_delete = $this->delete_paths($answer->filepath);
            }

            $answer->delete();

            return redirect()->route('answer.index', [$locale, $training_id, $module_id, $task_id])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('answer.index', [$locale, $training_id, $module_id, $task_id])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * create
     */
    public function create(Request $request, $locale, $training_id, $module_id, $task_id)
    {
        $data = array();
        
        $training = Training::find($training_id);
        $this->check_company($training);
        $module = Module::find($module_id);
        $this->check_company($module);
        $task = Task::find($task_id);
        if (!$task) 
        {
            $this->not_found();
        }
        $data['training'] = $training;
        $data['module'] = $module;
        $data['task'] = $task;

        return view('training.module.edit-answer', $data);
    }

    /**
     * edit
     */
    public function edit(Request $request, $locale, $training_id, $module_id, $task_id, $id)
    {
        $data = array();
        
        $training = Training::find($training_id);
        $this->check_company($training);
        $module = Module::find($module_id);
        $this->check_company($module);
        $task = Task::find($task_id);
        if (!$task) 
        {
            $this->not_found();
        }
        $answer = Answer::find($id);
        if (!$answer)
        {
            $this->not_found();
        }
        $answer->filepath = Storage::url($answer->filepath);

        $data['training'] = $training;
        $data['module'] = $module;
        $data['task'] = $task;
        $data['answer'] = $answer;

        return view('training.module.edit-answer', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $id = $request->input('id');
        $task_id = $request->input('task_id');
        $module_id = $request->input('module_id');
        $training_id = $request->input('training_id');

        $module = Module::find($module_id);
        $this->check_company($module);
        $task = Task::find($task_id);
        if (!$task) 
        {
            $this->not_found();
        }
        
        $data = array(
            "task_id" => $task_id,
            "answer" => $request->answer,
            "correct" => $request->correct ? : 0
        );
        $answer = Answer::find($id);

        if ($request->hasFile('file'))
        {
            if ($answer && $answer->filepath)
            {
                $result_delete = $this->delete_paths($answer->filepath);
            }
            $file_paths = $this->store_files($request->file('file'), 'documents');
            reset($file_paths);
            $filename = key($file_paths);
            $filepath = current($file_paths);
            $data["filename"] = $filename;
            $data["filepath"] = $filepath;
        }

        if ($answer)
        {
            $answer->update($data);
        }
        else
        {
            $answer = Answer::create($data);
        }
        return redirect()->route('answer.index', [$locale, $training_id, $module_id, $task_id])->with('success', __('message.successfully_registered'));
    }
}
