<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\AssignedTraining;
use App\Models\Category;
use App\Models\Department;
use App\Models\EmployeCategory;
use App\Models\Employer;
use App\Models\Langs;
use App\Models\Location;
use App\Models\PlaceOfBirth;
use App\Models\Platform;
use App\Models\Role;
use App\Models\Subdepartment;
use App\Models\Team;
use App\Models\Trainer;
use App\Models\Training;
use App\Models\User;
use Barryvdh\DomPDF\Facade\Pdf;
use DateTime;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Session;

class AssignedTrainingController extends Controller
{
    protected $request_data;

    /**
     * contruct
     */
    public function __construct(Request $request)
    {
        parent::__construct($request);
    }

    /**
     * attestation_update
     */
    public function attestation_update(Request $request, $locale)
    {
        $id = $request->id;
        $presentiel_id = $request->presentiel_id;
        $attest_date = $request->input('attest_date');
        $trainer_id = $request->input('trainer_id');

        $assigned_training = AssignedTraining::find($id);
        if ($assigned_training)
        {
            $this->check_company($assigned_training);

            $assigned_training->attest_date = $attest_date;
            $assigned_training->trainer_id = $trainer_id;
            $assigned_training->save();
        }
        else
        {
            $this->not_found();
        }

        if ($presentiel_id)
        {
            return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('success', __('message.successfully_registered'));
        }
        else 
        {
            $parameters = Session::get('assigned_training_filter', []);
            $parameters['locale'] = $locale;

            return redirect()->route('assigned-training.result', $parameters)->with('success', __('message.successfully_registered'));
        }
    }

    /**
     * attestation_remove
     */
    public function attestation_remove(Request $request, $locale)
    {
        $presentiel_id = $request->presentiel_id;
        $assigned_training_id = $request->assigned_training_id;

        $assigned_training = AssignedTraining::find($assigned_training_id);
        if ($assigned_training)
        {
            $this->check_company($assigned_training);
            
            $assigned_training->attest_date = null;
            $assigned_training->save();
        }
        else
        {
            $this->not_found();
        }
        return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('success', __('message.successfully_registered'));
    }

    /**
     * filter
     */
    public function filter(Request $request, $locale)
    {   
        $data = $this->set_data();
        
        $request_data = $request->all();
        $parameters = [];
        if (count($request_data))
        {
            $parameters = Session::get('assigned_training_filter', []);
        }
        $data['parameters'] = $parameters;

        return view('training.assigned.filter', $data);
    }

    /**
     * result
     */
    public function result(Request $request, $locale)
    {
        $request_data = $request->all();

        Session::put('assigned_training_filter', $request_data);
        $parameters = $request_data;
        $parameters['locale'] = $locale;

        foreach ($request_data as $key => $val)
        {
            if (!$val)
            {
                if ($val !== "0")
                {
                    unset($request_data[$key]);
                }
            }
            else if (strpos($key, '_date') !== false)
            {
                $format = "d/m/Y";
                list($date_start, $date_end) = explode(' - ', $val, 2);
                $date_start = DateTime::createFromFormat($format, $date_start);
                $date_start = $date_start->format('Y-m-d');
                $date_end = DateTime::createFromFormat($format, $date_end);
                $date_end = $date_end->format('Y-m-d');
                $request_data[$key] = ['start' => $date_start, 'end' => $date_end];
            }
        }
        $this->request_data = $request_data;
        
        $data = $this->set_data();
        
        $data['parameters'] = $parameters;
        $data['parameters_all_data'] = $parameters;
        if (isset($data['parameters_all_data']['page']))
        {
            unset($data['parameters_all_data']['page']);
        }
        
        $assigned_trainings_qb = $this->get_qb();
        
        $paginate = $this->setting->paginate;
        if (isset($request_data['all']) && $request_data['all']) 
        {
            $paginate = 1000;
        }
        $assigned_trainings = count($request_data) ? $assigned_trainings_qb->distinct('assigned_training.id')->paginate($paginate) : $assigned_trainings_qb->where('assigned_training.id', 0)->paginate($paginate);

        $data['assigned_trainings'] = $assigned_trainings;
        
        $mapping = $this->get_mapping($request);
        $maps = $mapping['maps'];
        $data['maps'] = $maps;

        return view('training.assigned.result', $data);
    }

    /**
     * get_qb
     */
    public function get_qb()
    {
        $assigned_trainings_qb = AssignedTraining::select('assigned_training.*');
        $assigned_trainings_qb = $assigned_trainings_qb->leftJoin('users', 'users.id', '=', 'assigned_training.user_id');
        $assigned_trainings_qb = $assigned_trainings_qb->leftJoin('users_roles', 'users_roles.user_id', '=', 'assigned_training.user_id');
        $assigned_trainings_qb = $assigned_trainings_qb->leftJoin('training', 'training.id', '=', 'assigned_training.training_id');
        $assigned_trainings_qb = $assigned_trainings_qb->leftJoin('activities', 'activities.assigned_training_id', '=', 'assigned_training.id');
        $assigned_trainings_qb = $assigned_trainings_qb->leftJoin('training_demand', 'training_demand.id', '=', 'assigned_training.training_demand_id');
        $assigned_trainings_qb = $assigned_trainings_qb->where('assigned_training.company_id', $this->company->id);

        $this->set_qb('matricule_id', 'users.ref', 'like', $assigned_trainings_qb);
        $this->set_qb('user_id', 'users.id', '=', $assigned_trainings_qb);
        $this->set_qb('username', 'users.username', 'like', $assigned_trainings_qb);
        $this->set_qb('role_id', 'users_roles.role_id', '=', $assigned_trainings_qb);
        $this->set_qb('department_id', 'users.department_id', '=', $assigned_trainings_qb);
        $this->set_qb('subdepartment_id', 'users.subdepartment_id', '=', $assigned_trainings_qb);
        $this->set_qb('employer_id', 'users.employer_id', '=', $assigned_trainings_qb);
        $this->set_qb('location_id', 'users.location_id', '=', $assigned_trainings_qb);
        $this->set_qb('team_id', 'users.team_id', '=', $assigned_trainings_qb);
        $this->set_qb('employe_category_id', 'users.employe_category_id', '=', $assigned_trainings_qb);
        $this->set_qb('place_of_birth_id', 'users.place_of_birth_id', '=', $assigned_trainings_qb);
        $this->set_qb('minority', 'users.minority', '=', $assigned_trainings_qb);
        $this->set_qb('sex', 'users.sex', '=', $assigned_trainings_qb);
        $this->set_qb('job_status', 'users.job_status', '=', $assigned_trainings_qb);
        $this->set_qb('procedure', 'users.is_procedure_resp', '=', $assigned_trainings_qb);
        $this->set_qb('lockout', 'users.is_lockout_resp', '=', $assigned_trainings_qb);
        $this->set_qb('lang', 'users.lang', '=', $assigned_trainings_qb);
        $this->set_qb('user_status', 'users.status', '=', $assigned_trainings_qb);

        $this->set_qb('platform_id', 'training.platform_id', '=', $assigned_trainings_qb);
        $this->set_qb('category_id', 'training.category_id', '=', $assigned_trainings_qb);
        $this->set_qb('training_id', 'training.id', '=', $assigned_trainings_qb);
        $this->set_qb('training_prefix', 'training.prefix', 'like', $assigned_trainings_qb);
        $this->set_qb('assigned_date', 'assigned_training.start_date', 'between', $assigned_trainings_qb);
        $this->set_qb('due_date', 'assigned_training.due_date', 'between', $assigned_trainings_qb);
        $this->set_qb('completion_date', 'assigned_training.completion_date', 'between', $assigned_trainings_qb);
        $this->set_qb('refresh_date', 'assigned_training.refresh_date', 'between', $assigned_trainings_qb);
        $this->set_qb('activity_date', 'activities.date', 'between', $assigned_trainings_qb);
        $this->set_qb('training_status', 'assigned_training.status', '=', $assigned_trainings_qb);
        $this->set_qb('training_department_id', 'training.department_id', '=', $assigned_trainings_qb);
        $this->set_qb('training_subdepartment_id', 'training.subdepartment_id', '=', $assigned_trainings_qb);
        $this->set_qb('training_location_id', 'training.location_id', '=', $assigned_trainings_qb);
        $this->set_qb('trainer_id', 'assigned_training.trainer_id', '=', $assigned_trainings_qb);
        $this->set_qb('training_demand_team_id', 'training_demand.team_id', '=', $assigned_trainings_qb);
        $this->set_qb('training_demand_id', 'assigned_training.training_demand_id', '=', $assigned_trainings_qb);
        $this->set_qb('comments', 'assigned_training.comments', '=', $assigned_trainings_qb);
        $this->set_qb('result', 'training.result', '=', $assigned_trainings_qb);

        return $assigned_trainings_qb;
    }

    /**
     * set_data
     */
    private function set_data() 
    {
        $data = [];

        $data['users'] = User::where('company_id', $this->company->id)->where('status', 1)->select('id', 'lname', 'fname', 'ref')->get();
        $data['roles'] = Role::where('status', 1)->get();
        $data['departments'] = Department::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
        $data['subdepartments'] = Subdepartment::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
        $data['employers'] = Employer::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
        $data['locations'] = Location::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
        $data['teams'] = Team::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
        $data['employe_categories'] = EmployeCategory::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
        $data['place_of_births'] = PlaceOfBirth::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
        $data['minorities'] = [0 => __('app.no'), 1 => __('app.yes')];
        $data['sexes'] = [-1 => __('app.not_specified'), 0 => __('users.male'), 1 => __('users.female')];
        $data['jobs_status'] = [1 => __('users.permanent'), 2 => __('users.permanent_part_time'), 3 => __('users.temporary_full_time'), 4 => __('users.temporary_part_time')];
        $data['procedures'] = [0 => __('app.no'), 1 => __('app.yes')];
        $data['lockouts'] = [0 => __('app.no'), 1 => __('app.yes')];
        $data['langs'] = Langs::all();
        $data['user_status'] = [0 => __('users.inactive'), 1 => __('users.active')];
        $data['training_valid'] = [0 => __('app.no'), 1 => __('app.yes')];
        
        $data['platforms'] = Platform::whereIn("id", [1, 2, 4])->get();
        $data['categories'] = Category::whereIn("id", [1, 2, 3, 6])->get();
        $data['trainings'] = Training::where('company_id', $this->company->id)->where('active', 1)->select('id', 'name', 'prefix')->get();
        $data['training_status'] = ['In Progress' => __('app.in_progress'), 'Need for Practice' => __('app.need_for_practice'), 'Completed' => __('app.completed')];
        $data['trainers'] = Trainer::where('company_id', $this->company->id)->orWhereNull('company_id')->get();

        $data['format']  = ['csv' => __('app.CSV'), 'html' => __('app.HTML'), 'txt' => __('app.Text'), 'pdf' => __('app.PDF'), 'xls' => __('app.Excel_95'), 'xlsx' => __('app.Excel_2007')];

        Session::put('ref_data', $data);

        return $data;
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $assigned_training = AssignedTraining::find($id);

        $parameters = Session::get('assigned_training_filter', []);
        $parameters['locale'] = $locale;

        if ($assigned_training) 
        {
            $this->check_company($assigned_training);

            $documents = $this->unserialize($assigned_training->documents);
            if ($documents)
            {
                $result_delete = $this->delete_paths($documents);
            }

            $assigned_training->delete();

            return redirect()->route('assigned-training.result', $parameters)->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('assigned-training.result', $parameters)->with('error', __('message.error_deleting'));
        }
    }

    /**
     * get
     */
    public function get(Request $request, $locale)
    {
        $id = $request->id ? : 0;

        $assigned_training = AssignedTraining::find($id);

        if ($assigned_training) 
        {
            $this->check_company($assigned_training);
            $documents = $this->unserialize($assigned_training->documents);
            Session::put('downloads', $documents);
            $documents_html = '';
            foreach ($documents as $document_name => $document_path)
            {
                $link_path = route('other.download', [$locale, $document_name]);
                $documents_html .= '
                    <div class="mt-2">
                        <a href="' . $link_path . '">
                            <h5>' . $document_name . ' <i class="ti ti-download fs-5"></i></h5>
                        </a>
                    </div>
                ';
            }
            if (!$documents_html)
            {
                $documents_html = '<h5>' . __("app.nothing_uploaded_yet") . '</h5>';
            }
            $assigned_training->documents_html = $documents_html;
            $assigned_training->training = $assigned_training->training;
        }

        return $assigned_training;
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $id = $request->id;
        $trainer_id = $request->input('trainer_id');
        $attest_date = $request->input('attest_date');
        $cert_date = $request->input('cert_date');
        $comments = $request->input('comments');
        $assigned_training = AssignedTraining::find($id);

        $parameters = Session::get('assigned_training_filter', []);
        $parameters['locale'] = $locale;

        if ($assigned_training) 
        {
            $this->check_company($assigned_training);

            $assigned_training->trainer_id = $trainer_id;
            if ($assigned_training->training->result == "2")
            {
                $assigned_training->cert_date = $cert_date;
            }
            else
            {
                $assigned_training->attest_date = $attest_date;
            }
            if ($comments)
            {
                $assigned_training->comments = $comments;
            }

            $assigned_training->save();

            return redirect()->route('assigned-training.result', $parameters)->with('success', __('message.successfully_registered'));
        } 
        else 
        {
            return redirect()->route('assigned-training.result', $parameters)->with('error', __('message.not_found'));
        }
    }

    /**
     * upload
     */
    public function upload(Request $request, $locale)
    {  
        $data = [];

        $id = $request->id;

        $assigned_training = AssignedTraining::find($id);

        if (!$assigned_training) 
        {
            $this->not_found();
        }

        $data["assigned_training"] = $assigned_training;

        $parameters = Session::get('assigned_training_filter', []);
        $parameters['locale'] = $locale;
        $data['parameters'] = $parameters;

        return view('training.assigned.upload', $data);
    }

    /**
     * store_upload
     */
    public function store_upload(Request $request, $locale)
    {
        $assigned_training_id = $request->input('assigned_training_id');
        $assigned_training = AssignedTraining::find($assigned_training_id);

        if ($assigned_training)
        {
            if ($request->hasFile('documents')) 
            {
                $file_paths = $this->store_files($request->file('documents'), 'documents');
                $assigned_training->documents = serialize($file_paths);
                $assigned_training->save();
            }

            $parameters = Session::get('assigned_training_filter', []);
            $parameters['locale'] = $locale;

            return redirect()->route('assigned-training.result', $parameters)->with('success', __('message.successfully_registered'));
        }
        else
        {
            $this->not_found();
        }
    }

    /**
     * complete
     */
    public static function complete($assigned_training_id)
    {
        if (!$assigned_training_id)
        {
            return false;
        }
        $assigned_training = AssignedTraining::find($assigned_training_id);
        if (!$assigned_training)
        {
            return false;
        }

        $is_completed = true;
        foreach ($assigned_training->activities as $activity)
        {
            if ($activity->status != 'Completed')
            {
                $is_completed = false;
            }
        }

        if ($is_completed && $assigned_training->status != 'Completed' && $assigned_training->completed != 1)
        {
            $assigned_training->status = 'Completed';
            $assigned_training->completed = 1;
        }
        else
        {
            $assigned_training->status = 'In Progress';
            $assigned_training->completed = 0;

            $assigned_training->save();
        }
        
        return $assigned_training;
    }

    /**
     * export_csv
     */
    public function export_csv(Request $request)
    {
        $assigned_trainings = $this->get_qb()->distinct('assigned_training.id')->get();
        $ref_data = Session::get('ref_data', []);

        $data = [
            [
                __("app.matricule_id"), 
                __("app.full_name"),
                __("users.username"),
                __('users.corporate_e_mail'),
                __('users.personal_e_mail'),
                __("users.role"),
                __('app.user_department'),
                __('app.user_subdepartment'),
                __('users.employer'),
                __('users.location_division'),
                __('users.team'),
                __('app.employee_category'),
                __('users.job_status'),
                __('users.language'),
                __('users.place_of_birth'),
                __('users.date_of_birth'),
                __('app.visible_minority'),
                __('users.sex'),
                __('users.start_date'),
                __('users.stop_date'),
                __('app.lockout_responsible'),
                __('app.procedure_responsible'),
                __('app.user_status'),
                __("app.platform"),
                __("app.category"),
                __("app.prefix"),
                __("app.training_name"),
                __('app.assigned_date'),
                __('app.due_date'),
                __('app.trainer_name'),
                __("app.attestation"),
                __("app.certification"),
                __("app.duration_of_theory"),
                __("app.duration_of_practice"),
                __("app.training_demand_id"),
                __("app.training_demand_team"),
                __('app.result'),
                __('app.status'),
                __('app.refresh'),
                __('app.training_valid')
            ]
        ];
        foreach ($assigned_trainings as $assigned_training)
        {
            $data[$assigned_training->id] = [
                $assigned_training->user->ref,
                $assigned_training->user->lname . ', ' .$assigned_training->user->fname,
                $assigned_training->user->username,
                $assigned_training->user->email,
                $assigned_training->user->email2,
                $assigned_training->user->role_user->role->name,
                $assigned_training->user->department->name,
                $assigned_training->user->subdepartment->name,
                $assigned_training->user->employer->name,
                $assigned_training->user->location->name,
                $assigned_training->user->team->name,
                $assigned_training->user->employe_category->name,
                $ref_data["jobs_status"][$assigned_training->user->job_status] ?? '',
                $assigned_training->user->lang,
                $assigned_training->user->place_of_birth->name,
                $assigned_training->user->dateofbirth,
                $ref_data["minorities"][$assigned_training->user->minority] ?? '',
                $ref_data['sexes'][$assigned_training->user->sex] ?? '',
                $assigned_training->user->datestart,
                $assigned_training->user->datestop,
                $ref_data['lockouts'][$assigned_training->user->is_lockout_resp] ?? '',
                $ref_data['procedures'][$assigned_training->user->is_procedure_resp] ?? '',
                $ref_data['user_status'][$assigned_training->user->status] ?? '',
                $assigned_training->training->platform->type,
                $assigned_training->training->category->new_name,
                $assigned_training->training->prefix,
                $assigned_training->training->name,
                $assigned_training->start_date,
                $assigned_training->due_date,
                $assigned_training->trainer->lname . ' ' . $assigned_training->trainer->fname,
                $assigned_training->attest_date,
                $assigned_training->cert_date,
                $assigned_training->training->duration_th,
                $assigned_training->training->duration_pr,
                $assigned_training->training_demand->id,
                $assigned_training->training_demand->team->name,
                $assigned_training->result,
                $ref_data['training_status'][$assigned_training->status] ?? '',
                $assigned_training->refresh_date,
                $ref_data['training_valid'][$assigned_training->completed] ?? '',
            ];
        }

        $filename = "export_assigned_training_" . date('Ymd_His') . ".csv";

        $handle = fopen($filename, 'w+');
        foreach ($data as $row) 
        {
            fputcsv($handle, $row);
        }
        fclose($handle);

        return Response::download($filename)->deleteFileAfterSend();
    }

    /**
     * export_activity
     */
    public function export_activity(Request $request, $locale)
    {
        $format = $request->input('format');

        $assigned_trainings = $this->get_qb()->distinct('assigned_training.id')->get();

        $mapping = $this->get_mapping($request);
        $mapping_keys = $mapping['keys'];
        $mapping_vals = $mapping['vals'];

        $data = [];
        $data['th'] = [__("app.total_duration_th")];
        $data['pr'] = [__("app.total_duration_pr")];
        $data['head'] = $mapping_keys;

        $total_duration_th = 0;
        $total_duration_pr = 0;
        foreach ($assigned_trainings as $assigned_training)
        {
            foreach ($assigned_training->activities as $activity)
            {
                if ($activity->type == 'Theory')
                {
                    $total_duration_th += $activity->duration;
                }
                if ($activity->type == 'Practice')
                {
                    $total_duration_pr += $activity->duration;
                }
                foreach ($mapping_vals as $i => $val)
                {
                    $str = '$val = ' . $val . ';';
                    eval($str);
                    $data[$activity->id][$i] = $val;
                }
            }
        }
        $data['th'] = [__("app.total_duration_th", ['total_duration_th' => $this->sec_to_time_full($total_duration_th)])];
        $data['pr'] = [__("app.total_duration_pr", ['total_duration_pr' => $this->sec_to_time_full($total_duration_pr)])];

        $filename = "export_activities_" . date('Ymd_His') . "." . $format;

        if ($format != 'pdf')
        {
            $handle = fopen($filename, 'w+');
        }

        if ($format == 'csv')
        {
            foreach ($data as $row) 
            {
                fputcsv($handle, $row);
            }
        }
        if ($format == 'html' || $format == 'pdf')
        {
            $html = '
                <!DOCTYPE html>
                <html lang="'.$locale.'">
                <head>
                    <meta charset="UTF-8">
                    <meta name="viewport" content="width=device-width, initial-scale=1.0">
                    <meta name="description" content="' . __('app.export_activities') . '" />
                    <meta name="company" content="Paelo" />
                    <meta name="lastModifiedBy" content="Paelo" />
                    <meta name="created" content="' . date('Y-m-d H:i:s') . '" />
                    <meta name="modified" content="' . date('Y-m-d H:i:s') . '" />
                    <title>' . __('app.export_activities') . '</title>
                    <style>th{background-color:#E5E5E5;}' . ($format == 'pdf' ? 'body {font-size: 5px;}' : '') . '</style>
                </head>
                <body>
            ';
            $html .= "<p>" . $data['th'][0] . "</p>";
            $html .= "<p>" . $data['pr'][0] . "</p>";
            $html .= "<table border='1' cellspacing='0' width='100%'>";
            $html .= "<thead><tr>";
            foreach ($mapping_keys as $key)
            {
                $html .= "<th>" . htmlspecialchars($key) . "</th>";
            }
            $html .= "</tr></thead>";
            $html .= "<tbody>";
            foreach ($data as $k => $row) 
            {
                if (in_array($k, array('th', 'pr', 'head')))
                {
                    continue;
                }
                $html .= "<tr>";
                foreach ($row as $cell) 
                {
                    $html .= "<td>" . htmlspecialchars($cell) . "</td>";
                }
                $html .= "</tr>";
            }
            $html .= "</tbody>";
            $html .= "</table>";
            $html .= "</body></html>";
            if ($format == 'html')
            {
                fwrite($handle, $html);
            }
        }
        if ($format == 'txt')
        {
            foreach ($data as $row) 
            {
                fputcsv($handle, $row, "\t");
            }
        }
        if ($format == 'pdf')
        {
            $pdf = Pdf::loadHTML($html);
            $pdf->setPaper('A4', 'landscape');
            return $pdf->download($filename);
        }
        if ($format == 'xls')
        {
            foreach ($data as $row) 
            {
                fputcsv($handle, $row, "\t");
            }
        }
        if ($format == 'xlsx')
        {
            foreach ($data as $row) 
            {
                fputcsv($handle, $row, "\t");
            }
        }

        if ($format != 'pdf')
        {
            fclose($handle);
        }

        return Response::download($filename)->deleteFileAfterSend();
    }

    /**
     * get_mapping
     */
    private function get_mapping($request)
    {
        $alls = [
            '#' => '$activity->id',
            'app.last_name' => '$assigned_training->user->lname',
            'app.first_name' => '$assigned_training->user->fname',
            'app.matricule_id' => '$assigned_training->user->matricule_id',
            'users.person_id' => '$assigned_training->user->person_id',
            'app.nap' => '$assigned_training->user->nas',
            'users.department' => '$assigned_training->user->department->name',
            'users.sub_department' => '$assigned_training->user->subdepartment->name',
            'users.employer' => '$assigned_training->user->employer->name',
            'users.location_division' => '$assigned_training->user->location->name',
            'users.team' => '$assigned_training->user->team->name',
            'users.language' => '$assigned_training->user->lang',
            'users.place_of_birth' => '$assigned_training->user->place_of_birth->name',
            'users.date_of_birth' => '$assigned_training->user->dateofbirth',
            'app.employee_category' => '$assigned_training->user->employe_category->name',
            'app.training_name' => '$assigned_training->training->name',
            'app.prefix' => '$assigned_training->training->prefix',
            'app.type' => '$activity->type',
            'app.trainer_id' => '$assigned_training->trainer->lname . " " . $assigned_training->trainer->fname',
            'app.date' => '$activity->date',
            'app.status' => '$activity->status',
            'app.duration_hms' => '$this->sec_to_time_full($activity->duration)',
            'app.notes' => '$activity->notes',
            'app.platform_name' => '$assigned_training->training->platform->type',
            'app.attest_date' => '$assigned_training->attest_date',
            'app.cert_date' => '$assigned_training->cert_date',
            'app.cat_name' => '$assigned_training->training->category->new_name',
            'app.training_passed' => '$assigned_training->completed'
        ];
        $request_data = $request->all();
        $format = $request_data['format'] ?? '';
        unset($request_data['_token']);
        unset($request_data['format']);
        $alls_new = [];
        if (count($request_data) && $format)
        {
            foreach ($alls as $key => $val)
            {
                if (in_array($key, $request_data))
                {
                    $alls_new[$key] = $val;
                }
            }
            $alls = $alls_new;
        }
        
        $keys = [];
        $vals = [];
        $maps = [];
        foreach ($alls as $key => $val)
        {
            $keys[] = __($key);
            $vals[] = $val;
            $maps[$key] = __($key);
        }
        return [
            'alls' => $alls,
            'keys' => $keys,
            'vals' => $vals,
            'maps' => $maps
        ];
    }
}

