<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Form;
use App\Models\FormField;
use App\Models\FormFieldTypes;
use App\Models\Training;
use App\Models\User;
use Illuminate\Http\Request;

class FormController extends Controller
{
    protected $data;

    /**
     * contruct
     */
    public function __construct(Request $request)
    {
        parent::__construct($request);

        $data = array();
        
        $data['category'] = Category::find($request->category_id);
        $data['trainings'] = Training::select('id', 'name', 'prefix')->where('company_id', $this->company->id)->where('active', 1)->where('disable', 0)->get();

        $this->data = $data;
    }

    /**
     * create
     */
    public function create(Request $request)
    {
        $data = $this->data;

        $data['has_assign_to_user'] = false;
        $data['form_fields'] = [];

        return view('training.form-edit', $data);
    }

    /**
     * edit
     */
    public function edit(Request $request)
    {
        $data = $this->data;

        $form = Form::find($request->id);
        $data['form'] = $form;
        if ($form) 
        {
            $this->check_company($form);
            
            $has_assign_to_user = false;
            foreach ($form->form_fields as $form_field)
            {
                if ($form_field->type == 19)
                {
                    $has_assign_to_user = true;
                }
            }
            $data['has_assign_to_user'] = $has_assign_to_user;

            $form_field_types = FormFieldTypes::orderBy('id', 'ASC')->get();
            $data['form_field_types'] = [];
            foreach ($form_field_types as $form_field_type)
            {
                $data['form_field_types'][$form_field_type->id] = $form_field_type->name;
            }

            $data['category'] = $form->category;
            $data['training'] = $form->training;
            $data['form_fields'] = $form->form_fields;
        }
        else
        {
            $this->not_found();
        }

        return view('training.form-edit', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $id = $request->input('id');
        $form = Form::find($id);

        $validation = [
            'name' => 'required'
        ];

        $validated = $request->validate($validation);

        $data = $request->all();
        unset($data['_token']);
        unset($data['id']);

        if (!$form)
        {
            $data['company_id'] = $this->company->id;
        }

        $data = $this->get_not_null($data);
        if ($form) 
        {
            $this->check_company($form);
            $data['training_id'] = intval($data['training_id']);
            $form->update($data);
        }
        else
        {
            $form = Form::create($data);
        }
        
        return redirect()->route('training.index', $locale)->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $form = Form::find($id);

        if ($form) 
        {
            $this->check_company($form);

            $form->delete();

            return redirect()->route('training.index', $locale)->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('training.index', $locale)->with('error', __('message.error_deleting'));
        }
    }

    /**
     * view
     */
    public function view(Request $request)
    {
        $data = $this->data;

        $data['is_view'] = false;

        $form = Form::find($request->id);
        $data['form'] = $form;
        if ($form) 
        {
            $this->check_company($form);
            
            $data['category'] = $form->category;
            $data['form_fields'] = [];
            foreach ($form->form_fields as $i => $form_field)
            {
                if ($form_field->type == 5)
                {
                    $content = explode("|", $form_field->content);
                    $form_field->content = $content[0];
                    unset($content[0]);
                    $form_field->list = $content;
                }
                $data['form_fields'][$i] = $form_field;
            }

            $assign_users = User::where("company_id", $this->company->id)->get();
            $data['assign_users'] = $assign_users;
        }
        else
        {
            $this->not_found();
        }

        return view('training.form-view', $data);
    }
}
