<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Form;
use App\Models\FormField;
use App\Models\FormFieldTypes;
use App\Models\Training;
use Illuminate\Http\Request;

class FormFieldController extends Controller
{
    /**
     * create
     */
    public function create(Request $request)
    {
        $data = [];

        $form_id = $request->form_id;
        $form = Form::find($form_id);
        if ($form) 
        {
            $this->check_company($form);
            
            $has_assign_to_user = false;
            foreach ($form->form_fields as $form_field)
            {
                if ($form_field->type == 19)
                {
                    $has_assign_to_user = true;
                }
            }
            $data['has_assign_to_user'] = $has_assign_to_user;

            $data['form'] = $form;
            $data['form_field_types'] = FormFieldTypes::orderBy('id', 'ASC')->get();
        }
        else
        {
            $this->not_found();
        }

        return view('training.form-field-edit', $data);
    }

    /**
     * edit
     */
    public function edit(Request $request)
    {
        $data = [];

        $id = $request->id;
        $form_id = $request->form_id;
        $form = Form::find($form_id);
        if ($form) 
        {
            $this->check_company($form);
            
            $has_assign_to_user = false;
            foreach ($form->form_fields as $form_field)
            {
                if ($form_field->type == 19)
                {
                    $has_assign_to_user = true;
                }
            }
            $data['has_assign_to_user'] = $has_assign_to_user;
            
            $form_field = FormField::find($id);

            $data['form'] = $form;
            $data['form_field'] = $form_field;
            $data['form_field_types'] = FormFieldTypes::orderBy('id', 'ASC')->get();
        }
        else
        {
            $this->not_found();
        }

        return view('training.form-field-edit', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $id = $request->input('id');
        $form_id = $request->input('form_id');

        $validation = [
            'field' => 'required',
            'type' => 'required'
        ];
        $validated = $request->validate($validation);

        $form = Form::find($form_id);
        if ($form) 
        {
            $this->check_company($form);

            $form_field = FormField::find($id);

            $data = $request->all();
            unset($data['_token']);
            unset($data['id']);

            $last_form_field = FormField::where('form_id', $form_id)->orderBy('order', 'DESC')->limit(1)->first();
            $order = 0;
            if ($last_form_field)
            {
                $order = $last_form_field->order;
            }
            $data['order'] = $order + 1;

            $data = $this->get_not_null($data);
            if ($form_field)
            {
                $form_field->update($data);
            }
            else
            {
                $form_field = FormField::create($data);
            }
        }
        else
        {
            $this->not_found();
        }
        
        return redirect()->route('form.edit', [$locale, $form_id])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $form_id = $request->form_id;
        $form_field = FormField::find($id);

        if ($form_field) 
        {
            $form_field->delete();

            return redirect()->route('form.edit', [$locale, $form_id])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('form.edit', [$locale, $form_id])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * update_order
     */
    public function update_order(Request $request)
    {
        $form_id = $request->form_id;
        $ids = $request->ids;
        foreach ($ids as $i => $form_field_id)
        {
            $form_field = FormField::where('form_id', $form_id)->where('id', $form_field_id)->first();
            $form_field->update(['order' => $i + 1]);
        }
        return $ids;
    }
}
