<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\JobTrainingGroup;
use App\Models\Training;
use Illuminate\Http\Request;

class JobTrainingController extends Controller
{
    /**
     * index
     */
    public function index()
    {
        $data = array();
        
        $job_training_groups = JobTrainingGroup::where('company_id', $this->company->id)->get();
        $data['job_training_groups'] = $job_training_groups;

        $categories = Category::where('id', '<=', 6)->get();
        $data['categories'] = $categories;

        return view('training.job-training', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $validated = $request->validate([
            'name' => 'required'
        ]);

        $id = $request->input('id');
        $job_training_group = JobTrainingGroup::find($id);
        $data = array(
            'prefix' => $request->prefix,
            'name' => $request->name,
            'description' => $request->description,
            'company_id' => $this->company->id
        );

        if ($job_training_group) 
        {
            $this->check_company($job_training_group);
            $job_training_group->update($data);
        }
        else
        {
            $job_training_group = JobTrainingGroup::create($data);
        }
        
        return redirect()->route('job-training.list', [$locale])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $job_training_group = JobTrainingGroup::find($id);

        if ($job_training_group) 
        {
            $this->check_company($job_training_group);
            $job_training_group->delete();

            return redirect()->route('job-training.list', [$locale])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('job-training.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * training_detach
     */
    public function training_detach(Request $request, $locale, $job_training_group_id)
    {
        $training_id = $request->id;
        $job_training_group = JobTrainingGroup::find($job_training_group_id);

        if ($job_training_group) 
        {
            $this->check_company($job_training_group);
            $job_training_group->trainings()->detach($training_id);

            return redirect()->route('job-training.view', [$locale, $job_training_group->id])->with('success', __('message.successfully_registered'));
        } 
        else 
        {
            return redirect()->route('job-training.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * view
     */
    public function view(Request $request)
    {
        $job_training_group = JobTrainingGroup::find($request->id);
        $data['job_training_group'] = $job_training_group;
        $training_categories = [];
        if ($job_training_group)
        {
            $this->check_company($job_training_group);
            foreach ($job_training_group->trainings as $training)
            {
                $training_categories[$training->category_id.'-'.$training->platform_id][] = $training;
            }
        }
        else
        {
            $this->not_found();
        }
        $data['training_categories'] = $training_categories;

        $trainings = [];
        $training_ids = [];
        foreach ($training_categories as $training_category)
        {
            foreach ($training_category as $training)
            {
                $trainings[$training->id] = $training;
                $training_ids[] = $training->id;
            }
        }
        $data['trainings'] = $trainings;

        $categories = Category::where('id', '<=', 6)->get();
        $category_1_2 = clone $categories[0];
        $category_1_2->code = 'external_presentiel';
        $categories[] = $category_1_2;
        $data['categories'] = $categories;

        $new_trainings_tmp = Training::where('company_id', $this->company->id)
                                ->whereNotIn('id', $training_ids)
                                ->where('category_id', '<=', 6)
                                ->where('platform_id', '<=', 2)
                                ->get();
        $new_trainings = [];
        foreach ($new_trainings_tmp as $training)
        {
            if ($training->category_id == 1 && $training->platform_id == 2)
            {
                $training->category->code = 'external_presentiel';
            }
            $new_trainings[] = $training;
        }
        $data['new_trainings'] = $new_trainings;
        
        return view('training.job-training-view', $data);
    }

    /**
     * training_select
     */
    public function training_select(Request $request, $locale)
    {
        $id = $request->id;
        $training_ids = $request->training_id;
        $job_training_group = JobTrainingGroup::find($id);
        if ($job_training_group)
        {
            $job_training_group->trainings()->attach($training_ids);
            return redirect()->route('job-training.view', [$locale, $id])->with('success', __('message.successfully_registered'));
        }
        else
        {
            return redirect()->route('job-training.list', $locale)->with('error', __('message.error_training_not_found'));
        }
    }
}
