<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Livereading;
use App\Models\Usertraining;
use App\Models\Category;

//use App\Http\Requests\LivereadingRequest;
use Illuminate\Http\Request;

class LiveReadingController extends Controller
{
    /**
     * Show the list of LiveReading models.
     *
     * @return \Illuminate\View\View
     */
    public function index(Request $request)
    {

        // Définir les catégories avec les IDs spécifiés
        $category_ids = [1, 2]; // Utilisation d'un tableau directement
        $categories = Category::whereIn('id', $category_ids)->get();

        // Initialiser la requête avec les relations nécessaires
        $query = Livereading::with(['training', 'trainer']);

        // Appliquer les filtres si des paramètres sont fournis dans la requête
        if ($request->filled('training_id')) {
            $query->where('training_id', $request->training_id);
        }

        if ($request->filled('trainer_id')) {
            $query->where('trainer_id', $request->trainer_id);
        }

        if ($request->filled('user_name')) {
            $query->whereHas('user', function ($q) use ($request) {
                $q->where('name', 'like', '%' . $request->user_name . '%');
            });
        }

        if ($request->filled('date')) {
            $query->where('date', $request->date);
        }

        // Exécuter la requête avec tri et pagination
        $history = $query->orderBy('id', 'desc')->paginate(5);

        // Retourner la vue avec les données nécessaires
        return view('training.livereading.livereading', compact('history', 'categories'));
    }

    /**
     * Display a single LiveReading model.
     *
     * @param int $id
     * @return \Illuminate\View\View
     */
    public function show($id)
    {
        $model = $this->findModel($id);

        return view('livereading.view', [
            'model' => $model,
        ]);
    }

    /**
     * Create a new LiveReading model.
     *
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\View\View
     */
    public function create(Request $request)
    {
        $model = new Livereading();

        if ($request->isMethod('post')) {
            $data = $request->validate([
                // Ajoute ici les règles de validation
                'name' => 'required|string|max:255',
            ]);

            $model->fill($data);

            if ($model->save()) {
                return redirect()->route('livereading.show', $model->id);
            }
        }

        return view('livereading.create', [
            'model' => $model,
        ]);
    }

    /**
     * Update an existing LiveReading model.
     *
     * @param int $id
     * @return \Illuminate\Http\RedirectResponse|\Illuminate\View\View
     */
    public function update(Request $request, $id)
    {
        $model = $this->findModel($id);

        if ($request->isMethod('post')) {
            $data = $request->validate([
                // Ajoute ici les règles de validation
                'name' => 'required|string|max:255',
            ]);

            $model->fill($data);

            if ($model->save()) {
                return redirect()->route('livereading.show', $model->id);
            }
        }

        return view('livereading.update', [
            'model' => $model,
        ]);
    }

    /**
     * Delete an existing LiveReading model.
     *
     * @param int $id
     * @return \Illuminate\Http\RedirectResponse
     */
    public function destroy($id)
    {
        $model = $this->findModel($id);

        if (!$model->completed) {
            $trainees = unserialize($model->trainees);

            foreach ($trainees as $trainee) {
                $utr = Usertraining::where('training_id', $model->training_id)
                    ->where('attestation', 0)
                    ->where('certification', 0)
                    ->where('user_id', $trainee)
                    ->first();

                if ($utr) {
                    $utr->delete();
                }
            }
        }

        $model->delete();

        return redirect()->route('usertraining.live_reading');
    }

    /**
     * Find the LiveReading model based on its primary key value.
     *
     * @param int $id
     * @return Livereading
     */
    protected function findModel($id)
    {
        $model = Livereading::find($id);

        if (!$model) {
            abort(404, 'The requested page does not exist.');
        }

        return $model;
    }
}
