<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Module;
use App\Models\Training;
use App\Models\TrainingModule;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Session;

class ModuleController extends Controller
{
    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $validated = $request->validate([
            'name' => 'required',
            'type' => 'required',
        ]);
        Log::info('Validated : ', $validated);

        $data = $request->all();
        unset($data['_token']);

        $id = $request->input('id');
        unset($data['id']);
        $module = Module::find($id);

        $data['company_id'] = $this->company->id;
        $training_id = $data['training_id'];

        $data = $this->get_not_null($data);
        if ($module) 
        {
            $this->check_company($module);
            $module->update($data);
        }
        else
        {
            $module = Module::create($data);
            $module->trainings()->attach([$training_id]);
        }
        
        return redirect()->route('training.view', [$locale, $training_id])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale, $training_id)
    {
        $id = $request->id;
        $module = Module::find($id);

        if ($module) 
        {
            $this->check_company($module);

            $module->trainings()->detach();
            $module->delete();

            return redirect()->route('training.view', [$locale, $training_id])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('training.view', [$locale, $training_id])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * update_order
     */
    public function update_order(Request $request, $locale)
    {
        $training_id = $request->training_id;
        $ids = $request->ids;
        foreach ($ids as $i => $module_id)
        {
            $training_module = TrainingModule::where('training_id', $training_id)->where('module_id', $module_id)->first();
            $training_module->update(['order' => $i + 1]);
        }
        return $ids;
    }

    /**
     * view
     */
    public function view(Request $request)
    {
        $training_id = $request->training_id;

        $module = Module::find($request->id);
        if ($module) 
        {
            $this->check_company($module);
            
            $training = Training::find($training_id);

            $data['module'] = $module;
            $data['training_id'] = $training_id;
            $data['training'] = $training;
            $data['tasks'] = $module->tasks;

            $documents = $this->unserialize($module->documents);   
            $data['documents'] = $documents;
            Session::put('downloads', $documents);
        }
        else
        {
            $this->not_found();
        }

        return view('training.module.view', $data);
    }
}
