<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Activity;
use App\Models\AssignedTraining;
use App\Models\Presentiel;
use App\Models\Trainer;
use App\Models\Training;
use App\Models\TrainingLocation;
use App\Models\User;
use App\Models\UserForm;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Storage;

class PresentielController extends Controller
{
    /**
     * index
     */
    public function index()
    {
        $data = array();
        
        $presentiels = Presentiel::where('company_id', $this->company->id)->get();
        $data['presentiels'] = [];
        foreach ($presentiels as $i => $presentiel)
        {
            $trainees = $this->unserialize($presentiel->trainees);
            $trainees = User::whereIn('id', $trainees)->get();
            $presentiel->trainees = $trainees;
            $data['presentiels'][$i] = $presentiel;
        }

        return view('training.presentiel.index', $data);
    }

    /**
     * create
     */
    public function create(Request $request, $locale)
    {
        $data = array();

        $data['trainings'] = Training::select('id', 'name', 'prefix')
                                    ->where('company_id', $this->company->id)
                                    ->where('active', 1)
                                    ->where('disable', 0)
                                    ->where('platform_id', 2)
                                    ->get();
        $data['trainers'] = Trainer::where('company_id', $this->company->id)->get();
        $data['training_locations'] = TrainingLocation::where('company_id', 0)->get();
        
        return view('training.presentiel.edit', $data);
    }

    /**
     * view
     */
    public function view(Request $request, $locale)
    {
        $data = array();

        $id = $request->id;
        $presentiel = Presentiel::find($id);
        if ($presentiel)
        {
            $this->check_company($presentiel);
        }
        else
        {
            $this->not_found();
        }
        $presentiel->duration = $this->minute_to_time($presentiel->duration);
        $documents = $this->unserialize($presentiel->documents);
        $data['documents'] = $documents;
        Session::put('downloads', $documents);
        $trainees = $this->unserialize($presentiel->trainees);
        $presentiel->trainees = User::whereIn('id', $trainees)->get();
        
        $data['presentiel'] = $presentiel;
        $data['activities_presentiel'] = [];
        $data['assigned_training'] = [];
        foreach ($presentiel->activities_presentiel as $i => $activity_presentiel)
        {
            $activity_presentiel->duration = $this->minute_to_time($activity_presentiel->duration);
            $data['activities_presentiel'][$i] = $activity_presentiel;
            foreach ($activity_presentiel->activities as $activity)
            {
                if ($activity->assigned_training)
                {
                    $data['assigned_training'][$activity->assigned_training->id] = $activity->assigned_training;
                }
            }
        }
        foreach ($presentiel->assigned_training as $assigned_training)
        {
            $data['assigned_training'][$assigned_training->id] = $assigned_training;
        }
        
        $data['trainers'] = Trainer::where('company_id', $this->company->id)->get();
        $data['training_locations'] = TrainingLocation::where('company_id', 0)->get();
        $data['trainees'] = DB::table('users')
                            ->leftJoin('users_roles', 'users_roles.user_id', 'users.id')
                            ->whereIn('users_roles.role_id', [4])
                            ->where('users.status', '=', 1)
                            ->where('users.company_id', $this->company->id)
                            ->select(['users.id', 'users.fname', 'users.lname', 'users.ref'])
                            ->orderBy('users.fname', 'asc')
                            ->orderBy('users.lname', 'asc')
                            ->orderBy('users.id', 'asc')
                            ->get();
        $data['selected_trainees'] = [];
        foreach ($data['assigned_training'] as $assign_training)
        {
            $data['selected_trainees'][$assign_training->user_id] = $assign_training->user_id;
        }
        
        return view('training.presentiel.view', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $id = $request->input('id');
        $presentiel = Presentiel::find($id);

        $validation = [
            'start_date' => 'required',
            'end_date' => 'required',
        ];
        $request->validate($validation);

        $data = $request->all();
        unset($data['_token']);
        unset($data['id']);

        if (!$presentiel)
        {
            $data['company_id'] = $this->company->id;
        }
        unset($data['do_not_send_notification_to_trainees']);
        unset($data['do_not_send_notification_to_trainer']);
        $data['trainees'] = serialize([]);

        $data = $this->get_not_null($data);
        if ($presentiel) 
        {
            $this->check_company($presentiel);
            $presentiel->update($data);
        }
        else
        {
            $presentiel = Presentiel::create($data);
        }
        
        return redirect()->route('presentiel.list', $locale)->with('success', __('message.successfully_registered'));
    }

    /**
     * store_trainee
     */
    public function store_trainee(Request $request, $locale)
    {
        $trainee_id = $request->input('trainee_id');
        $presentiel_id = $request->input('presentiel_id');
        $presentiel = Presentiel::find($presentiel_id);

        if ($presentiel)
        {
            $this->check_company($presentiel);
            foreach ($presentiel->assigned_training as $assign_training)
            {
                $user_id = $assign_training->user_id;
                if (!in_array($user_id, $trainee_id))
                {
                    Activity::where('assigned_training_id', $assign_training->id)->delete();
                    $assign_training->user_form->delete();
                    $assign_training->delete();
                }
                else 
                {
                    $key = array_search($user_id, $trainee_id);
                    unset($trainee_id[$key]);
                }
            }
            
            foreach ($trainee_id as $user_id)
            {
                $assigned_training = new AssignedTraining();
                $assigned_training->platform_id = $presentiel->training->platform_id;
                $assigned_training->category_id = $presentiel->training->category_id;
                $assigned_training->training_id = $presentiel->training_id;
                $assigned_training->user_id = $user_id;
                $assigned_training->trainer_id = $presentiel->trainer_id;
                $assigned_training->start_date = $presentiel->start_date;
                $assigned_training->comments = '';
                $assigned_training->status = 'In Progress';
                $assigned_training->training_passed = 0;
                $assigned_training->presentiel_id = $presentiel->id;
                $assigned_training->company_id = $presentiel->company_id;
                $assigned_training->save();
            }
        }
        else
        {
            $this->not_found();
        }
        
        return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('success', __('message.successfully_registered'));
    }

    /**
     * store_document
     */
    public function store_document(Request $request, $locale)
    {
        $presentiel_id = $request->input('presentiel_id');
        $presentiel = Presentiel::find($presentiel_id);

        if ($presentiel)
        {
            $this->check_company($presentiel);
            
            if ($request->hasFile('documents')) 
            {
                $file_paths = $this->store_files($request->file('documents'), 'documents');

                $data = [];
                $documents = unserialize($presentiel->documents);
                foreach ($file_paths as $key => $val)
                {
                    $documents[$key] = $val;
                }
                $data['documents'] = serialize($documents);
                
                $presentiel->update($data);
            }
            
        }
        else
        {
            $this->not_found();
        }
        
        return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $presentiel = Presentiel::find($id);

        if ($presentiel) 
        {
            $this->check_company($presentiel);
            $presentiel->delete();

            return redirect()->route('presentiel.list', [$locale])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('presentiel.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * destroy_trainee
     */
    public function destroy_trainee(Request $request, $locale)
    {
        $id = $request->id;
        $presentiel_id = $request->presentiel_id;
        $presentiel = Presentiel::find($presentiel_id);

        if ($presentiel) 
        {
            $this->check_company($presentiel);

            $assigned_training = AssignedTraining::find($id);
            if ($assigned_training)
            {
                $this->check_company($assigned_training);

                Activity::where('assigned_training_id', $assigned_training->id)->delete();
                $assigned_training->user_training->delete();
                $assigned_training->user_form->delete();
                $assigned_training->delete();
                return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('success', __('message.successfully_deleted'));
            }
            /*
            $trainees = $this->unserialize($presentiel->trainees);
            if (in_array($id, $trainees))
            {
                $key = array_search($id, $trainees);
                unset($trainees[$key]);
                $presentiel->trainees = serialize($trainees);
                $presentiel->save();
                return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('success', __('message.successfully_deleted'));
            }
            */
        } 
        return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('error', __('message.error_deleting'));
    }

    /**
     * destroy_user_form
     */
    public function destroy_user_form(Request $request, $locale)
    {
        $id = $request->id;
        $presentiel_id = $request->presentiel_id;
        $presentiel = Presentiel::find($presentiel_id);

        if ($presentiel) 
        {
            $this->check_company($presentiel);

            $user_form = UserForm::find($id);
            if ($user_form)
            {
                $assigned_training = $user_form->assigned_training;
                $assigned_training->user_form_id = 0;
                $assigned_training->save();

                $user_form->delete();

                return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('success', __('message.successfully_deleted'));
            }
        } 
        return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('error', __('message.error_deleting'));
    }

    /**
     * delete_file
     */
    public function delete_file(Request $request, $locale, $filename)
    {
        $presentiel_id = $request->id;
        $presentiel = Presentiel::find($presentiel_id);

        if ($presentiel) 
        {
            $this->check_company($presentiel);

            $documents = $this->unserialize($presentiel->documents);
            
            if (array_key_exists($filename, $documents))
            {
                if (Storage::exists($documents[$filename]))
                {
                    Storage::delete($documents[$filename]);
                }
                unset($documents[$filename]);
                $presentiel->update(["documents" => serialize($documents)]);
                
                return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('success', __('message.successfully_deleted'));
            }
            return redirect()->route('presentiel.view', [$locale, $presentiel_id])->with('warning', __('message.not_found'));
        } 
        else 
        {
            return redirect()->route('presentiel.list', $locale)->with('error', __('message.error_deleting'));
        }
    }

    /**
     * update_cost
     */
    public static function update_cost($presentiel)
    {
        $cost = 0;
        foreach ($presentiel->training_costs as $training_cost)
        {
            $cost += $training_cost->cost;
        }
        
        $presentiel->cost = $cost;
        $presentiel->save();

        return $cost;
    }
}
