<?php

namespace App\Http\Controllers\Training;

use App\Helper\Times;
use App\Http\Controllers\Controller;
use App\Models\AssignedTraining;
use App\Models\Category;
use App\Models\Training;
use App\Models\UserTraining;
use App\Models\UserTrainingGroup;
use App\Models\UserTrainingJob;
use DateTime;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Response;

class StatisticController extends Controller
{
    /**
     * counter
     */
    public function counter(Request $request)
    {
        $data = array();

        $data['filters'] = [
            1 => __('app.assigned_date'),
            2 => __('app.completion_date'),
            3 => __('app.renewal_date'),
            4 => __('app.due_date')
        ];

        return view('training.report.counter', $data);
    }

    /**
     * counter_result
     */
    public function counter_result(Request $request)
    {
        $data = array();

        $this->counter_query($request, $data);

        return view('training.report.counter-result', $data);
    }

    /**
     * counter_export
     */
    public function counter_export(Request $request)
    {
        $data = array();
        $assigned_trainings = $this->counter_query($request, $data);

        $data = [
            [
                __("#"), 
                __("app.category"),
                __("app.training_name"),
                __("app.cost"),
                __("app.duration_of_theory"),
                __("app.duration_of_practice"),
                __("app.employer"),
                __("app.training_level"),
                __("app.department"),
                __("app.sub_department"),
                __("users.last_name"),
                __("users.first_name"),
                __("users.id_matricule_num"),
                __("app.assigned_date"),
                __("app.due_date"),
                __("app.attestation_certification_date"),
                __("app.refresh_date")
            ]
        ];
        foreach ($assigned_trainings as $assigned_training)
        {
            $training = $assigned_training->training;
            $user = $assigned_training->user;
            if (!$training->name) continue;
            
            $data[$assigned_training->id] = [
                $assigned_training->id,
                __('menus.' . $training->category->code),
                $training->name,
                '$ ' . $training->cost,
                Times::sec_to_time($training->duration_th),
                Times::sec_to_time($training->duration_pr),
                $user->employer->name,
                $training->training_level->name,
                $user->department->name,
                $user->subdepartment->name,
                $user->lname,
                $user->fname,
                $user->ref,
                $assigned_training->start_date,
                $assigned_training->due_date,
                $assigned_training->cert_date ?? $assigned_training->attest_date,
                $assigned_training->refresh_date,
            ];
        }

        $filename = "export_counter_" . date('Ymd_His') . ".csv";

        $handle = fopen($filename, 'w+');
        foreach ($data as $row) 
        {
            fputcsv($handle, $row);
        }
        fclose($handle);

        return Response::download($filename)->deleteFileAfterSend();
    }

    /**
     * counter_query
     */
    protected function counter_query(Request $request, &$data)
    {
        $filters = [
            1 => __('app.assigned_date'),
            2 => __('app.completion_date'),
            3 => __('app.renewal_date'),
            4 => __('app.due_date')
        ];

        $start_date = $request->input('start_date') ?? $request->start_date;
        $end_date = $request->input('end_date') ?? $request->end_date;
        $filter = $request->input('filter') ?? $request->filter;
        $sort_by = $request->sort_by ?? 'assigned_training.id';
        $sort_order = $request->sort_order ?? 'asc';
        $start = $start_date ?? date('Y-m-d');
        $end = $end_date ?? date('Y-m-d');

        $assigned_trainings_qb = AssignedTraining::select('assigned_training.*');
        $assigned_trainings_qb = $assigned_trainings_qb->where('assigned_training.company_id', $this->company->id);
        switch ($filter)
        {
            case 1:
                $assigned_trainings_qb = $assigned_trainings_qb->whereBetween('assigned_training.start_date', [$start, $end]);
                break;
            case 2:
                $assigned_trainings_qb = $assigned_trainings_qb->where(function ($query) use ($start, $end) {
                    $query->whereBetween('assigned_training.cert_date', [$start, $end])
                        ->orwhereBetween('assigned_training.attest_date', [$start, $end]);
                });
                break;
            case 3:
                $assigned_trainings_qb = $assigned_trainings_qb->whereBetween('assigned_training.due_date', [$start, $end]);
                break;
            case 4:
                $assigned_trainings_qb = $assigned_trainings_qb->whereBetween('assigned_training.refresh_date', [$start, $end]);
                break;
            default:
                $assigned_trainings_qb = $assigned_trainings_qb->where('assigned_training.id', 0);
        }
        $assigned_trainings_qb = $assigned_trainings_qb->orderBy($sort_by, $sort_order);
        $assigned_trainings = $assigned_trainings_qb->get();

        $data['start_date'] = $start_date;
        $data['end_date'] = $end_date;
        $data['filter'] = $filters[$filter];
        $data['assigned_trainings'] = $assigned_trainings;
        $data['sort_by'] = $sort_by;
        $data['sort_order'] = $sort_order;

        return $assigned_trainings;
    }

    /**
     * stats
     */
    public function stats(Request $request)
    {
        $data = array();
        
        $period = $request->period ?? $request->input('period');
        $start_date = $request->start_date ?? $request->input('start_date');
        $end_date = $request->end_date ?? $request->input('end_date');
        $period_calcul = str_replace('last_', '-', $period);
        $period_calcul = str_replace('_months', ' month', $period_calcul);
        $period_calcul = str_replace('_month', ' month', $period_calcul);
        $date = date('Y-m-d', strtotime($period_calcul));
        
        $start_date = $start_date ?? $date;
        $end_date = $end_date ?? date('Y-m-d');
        $start = new DateTime($start_date);
        $end = new DateTime($end_date);
        $diff = $start->diff($end);
        $days = $diff->days;

        $user_trainings = UserTraining::select('user_training.assigned_date', 'user_training.active AS ', 'user_training.reassigned AS temporary', DB::raw('COUNT(*) AS count'))
                            ->where('user_training.company_id', $this->company->id)
                            ->where('user_training.assigned_date' , '>=', $date)
                            ->groupBy('user_training.assigned_date', 'user_training.active', 'temporary')
                            ->orderBy('user_training.assigned_date', 'asc')
                            ->get();
        
        $list_date = [];
        $count_total = [];
        $count_active = [];
        $count_inactive = [];
        $count_temporary = [];
        foreach ($user_trainings as $user_training)
        {
            $assigned_date = $user_training->assigned_date;
            $active = $user_training->active;
            $temporary = $user_training->temporary;
            $count = $user_training->count;

            if (!array_key_exists($assigned_date, $count_total)) $count_total[$assigned_date] = 0;
            if (!array_key_exists($assigned_date, $count_temporary)) $count_temporary[$assigned_date] = 0;
            if (!array_key_exists($assigned_date, $count_active)) $count_active[$assigned_date] = 0;
            if (!array_key_exists($assigned_date, $count_inactive)) $count_inactive[$assigned_date] = 0;

            $list_date[$assigned_date] = $assigned_date;
            $count_total[$assigned_date] += $count;
            if ($temporary) $count_temporary[$assigned_date] += $count;
            else if ($active) $count_active[$assigned_date] += $count;
            else $count_inactive[$assigned_date] += $count;
        }

        $data['period'] = $period ?? 'last_1_month';
        $data['start_date'] = $start_date;
        $data['end_date'] = $end_date;

        $data['list_date'] = json_encode(array_values($list_date));
        $data['count_total'] = json_encode(array_values($count_total));
        $data['count_active'] = json_encode(array_values($count_active));
        $data['count_inactive'] = json_encode(array_values($count_inactive));
        $data['count_temporary'] = json_encode(array_values($count_temporary));

        $data['count_min'] = $count_total ? min($count_total) : 0;
        $data['count_max'] = $count_total ? max($count_total) : 1;
        $data['days'] = $days;
        $data['average'] = round(($data['count_min'] + $data['count_max']) / 2);

        return view('training.report.stats', $data);
    }

    /**
     * user_training_result
     */
    public function user_training_result(Request $request)
    {
        $data = $request->all();
        
        $data = $this->get_not_null($data);

        $this->user_training_query($request, $data);

        return view('training.report.user-training-result', $data);
    }

    /**
     * user_training_export
     */
    public function user_training_export(Request $request)
    {
        $data = array();
        $user_trainings = $this->user_training_query($request, $data);

        $data = [
            [
                __("app.trainee"),
                __("app.status"),
                __("app.start"),
                __("app.progress"),
                __("app.attestation"),
                __("app.training"),
                __("app.time_taken")
            ]
        ];
        foreach ($user_trainings as $user_training)
        {
            $training = $user_training->training;
            $user = $user_training->user;
            if (!$training->name) continue;
            
            $data[$user_training->id] = [
                $user->fname . ' ' . $user->lname,
                $user_training->active ? __('app.yes') : __('app.no'),
                $user_training->assigned_date,
                $user_training->progress . '%',
                $user_training->attest_date ?? $user_training->cert_date,
                $training->name,
                Times::sec_to_time_full($user_training->th_duration_taken + $user_training->pr_duration_taken)
            ];
        }

        $filename = "export_user_training_" . date('Ymd_His') . ".csv";

        $handle = fopen($filename, 'w+');
        foreach ($data as $row) 
        {
            fputcsv($handle, $row);
        }
        fclose($handle);

        return Response::download($filename)->deleteFileAfterSend();
    }

    /**
     * user_training_query
     */
    protected function user_training_query(Request $request, &$data)
    {
        $start_date = $request->input('start_date') ?? $request->start_date;
        $end_date = $request->input('end_date') ?? $request->end_date;
        $type = $request->input('type') ?? $request->type;
        $start = $start_date ?? date('Y-m-d');
        $end = $end_date ?? date('Y-m-d');

        $category = Category::where('code', $type)->first();

        $user_training_qb = UserTraining::select('user_training.*');
        $user_training_qb = $user_training_qb->leftJoin('training', 'training.id', 'user_training.training_id');
        $user_training_qb = $user_training_qb->where('user_training.company_id', $this->company->id);
        $user_training_qb = $user_training_qb->where('training.category_id', $category->id);
        $user_training_qb = $user_training_qb->whereBetween('user_training.assigned_date', [$start, $end]);
        $user_trainings = $user_training_qb->get();

        $data['start_date'] = $start_date;
        $data['end_date'] = $end_date;
        $data['type'] = $type;
        $data['user_trainings'] = $user_trainings;

        return $user_trainings;
    }

    /**
     * training_group_export
     */
    public function training_group_export(Request $request)
    {
        $data = array();
        $user_training_groups = $this->training_group_query($request, $data);

        $data = [
            [
                __("app.id"),
                __("app.fname"),
                __("app.lname"),
                __("app.username"),
                __("app.tr_group_name"),
                __("app.date")
            ]
        ];
        foreach ($user_training_groups as $user_training_group)
        {
            $training_group = $user_training_group->training_group;
            $user = $user_training_group->user;
            if (!$training_group->name) continue;
            
            $data[$user_training_group->id] = [
                $user->id,
                $user->fname,
                $user->lname,
                $user->username,
                $training_group->name,
                $user_training_group->date
            ];
        }

        $filename = "export_training_group_" . date('Ymd_His') . ".csv";

        $handle = fopen($filename, 'w+');
        foreach ($data as $row) 
        {
            fputcsv($handle, $row);
        }
        fclose($handle);

        return Response::download($filename)->deleteFileAfterSend();
    }

    /**
     * training_group_query
     */
    protected function training_group_query(Request $request, &$data)
    {
        $start_date = $request->input('start_date') ?? $request->start_date;
        $end_date = $request->input('end_date') ?? $request->end_date;
        $training_group_id = $request->input('training_group_id') ?? $request->training_group_id;
        $start = $start_date ?? date('Y-m-d');
        $end = $end_date ?? date('Y-m-d');

        $user_training_group_qb = UserTrainingGroup::select('user_training_group.*');
        $user_training_group_qb = $user_training_group_qb->leftJoin('training_group', 'training_group.id', 'user_training_group.training_group_id');
        $user_training_group_qb = $user_training_group_qb->where('training_group.company_id', $this->company->id);
        $user_training_group_qb = $training_group_id ? $user_training_group_qb->where('training_group.id', $training_group_id) : $user_training_group_qb;
        $user_training_group_qb = $user_training_group_qb->whereBetween('user_training_group.date', [$start, $end]);
        $user_training_groups = $user_training_group_qb->get();

        $data['user_training_groups'] = $user_training_groups;

        return $user_training_groups;
    }

    /**
     * job_training_export
     */
    public function job_training_export(Request $request)
    {
        $data = array();
        $user_training_jobs = $this->job_training_query($request, $data);

        $data = [
            [
                __("app.id"),
                __("app.fname"),
                __("app.lname"),
                __("app.username"),
                __("app.job_group_name"),
                __("app.date")
            ]
        ];
        foreach ($user_training_jobs as $user_training_job)
        {
            $job_training_group = $user_training_job->job_training_group;
            $user = $user_training_job->user;
            if (!$job_training_group->name) continue;
            
            $data[$user_training_job->id] = [
                $user->id,
                $user->fname,
                $user->lname,
                $user->username,
                $job_training_group->name,
                $user_training_job->date
            ];
        }

        $filename = "export_job_training_" . date('Ymd_His') . ".csv";

        $handle = fopen($filename, 'w+');
        foreach ($data as $row) 
        {
            fputcsv($handle, $row);
        }
        fclose($handle);

        return Response::download($filename)->deleteFileAfterSend();
    }

    /**
     * job_training_query
     */
    protected function job_training_query(Request $request, &$data)
    {
        $start_date = $request->input('start_date') ?? $request->start_date;
        $end_date = $request->input('end_date') ?? $request->end_date;
        $job_training_group_id = $request->input('job_training_group_id') ?? $request->job_training_group_id;
        $start = $start_date ?? date('Y-m-d');
        $end = $end_date ?? date('Y-m-d');

        $user_training_job_qb = UserTrainingJob::select('user_training_job.*');
        $user_training_job_qb = $user_training_job_qb->leftJoin('job_training_group', 'job_training_group.id', 'user_training_job.job_training_group_id');
        $user_training_job_qb = $user_training_job_qb->where('job_training_group.company_id', $this->company->id);
        $user_training_job_qb = $job_training_group_id ? $user_training_job_qb->where('job_training_group.id', $job_training_group_id) : $user_training_job_qb;
        $user_training_job_qb = $user_training_job_qb->whereBetween('user_training_job.date', [$start, $end]);
        $user_training_jobs = $user_training_job_qb->get();

        $data['user_training_jobs'] = $user_training_jobs;

        return $user_training_jobs;
    }
}
