<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Http\Controllers\PDFController;
use App\Models\Module;
use App\Models\Task;
use App\Models\Training;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;


class TaskController extends Controller
{
    /**
     * create
     */
    public function create(Request $request, $locale, $training_id, $module_id)
    {
        $data = array();
        
        $training = Training::find($training_id);
        $this->check_company($training);
        $module = Module::find($module_id);
        $this->check_company($module);
        $data['module'] = $module;
        $data['training'] = $training;

        $documents = $this->unserialize($training->documents);
        $data['training_documents'] = $documents;

        return view('training.module.create-task', $data);
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale, $module_id, $training_id)
    {
        $id = $request->id;
        $ids = explode(",", $id);
        foreach ($ids as $id) 
        {
            $task = Task::find($id);
            if ($task) 
            {
                $data_path = $this->unserialize($task->data_path);
                $result_delete = $this->delete_paths($data_path);

                foreach ($task->answers as $answer)
                {
                    $answer->delete();
                }
                $task->delete();
            }
        }

        if ($task) 
        {
            return redirect()->route('module.view', [$locale, $module_id, $training_id])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('module.view', [$locale, $module_id, $training_id])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $id = $request->input('id');
        $module_id = $request->input('module_id');
        $training_id = $request->input('training_id');
        
        $module = Module::find($module_id);
        if ($module)
        {
            $this->check_company($module);

            $training = Training::find($training_id);
            if ($training)
            {
                $this->check_company($module);
                try 
                {
                    $training_documents = unserialize($training->documents);
                }
                catch (\Exception $e) 
                {
                    $training_documents = [];
                }
            }
            else
            {
                $this->not_found();
            }
            
            $task = false;
            if ($id)
            {
                $task = Task::find($id);
            }
            $data_type = $task ? $task->data_type : $request->input('data_type');
            $validation = [
                'title' => 'required',
            ];
            if (!$task)
            {
                $validation['data_type'] = 'required';
            }
            if (!$task || $request->hasFile('upload'))
            {
                if ($data_type == 'scorm')
                {
                    $validation['upload'] = 'required|file|mimes:zip';
                }
                else if ($data_type == 'media')
                {
                    $validation['upload'] = 'required|file|mimes:jpg,jpeg,png,bmp,gif,svg,webp,mp4,mov,avi,wmv,flv,m3u8,ts,3gp';
                    if ($training->category->id == 2)
                    {
                        $validation['upload'] .= ',pdf';
                    }
                }
                else
                {
                    $validation['data_path'] = 'required';
                }
            }
            $request->validate($validation);
            
            $data_type = $task ? $task->data_type : $request->input('data_type');
            $title = $request->input('title');
            $extra_link = $request->input('extra_link');
            $documents = [];
            
            foreach ($request->input('documents', []) as $document)
            {
                if (array_key_exists($document, $training_documents))
                {
                    $documents[$document] = $training_documents[$document];
                }
            }
            $data_path = $request->input('data_path');
            $scorm_version = $request->input('scorm_version') ? : 0;
            $question = $request->input('question');

            $data = [];
            $data['title'] = $title;
            $data['extra_link'] = $extra_link;
            $data['documents'] = serialize($documents);
            if (in_array($data_type, array('media', 'scorm')) && $request->hasFile('upload'))
            {
                if ($task)
                {
                    $data_path_old = $this->unserialize($task->data_path);
                    $result_delete = $this->delete_paths($data_path_old);
                }

                $file = $request->file('upload');
                $mime_type = $file->getClientMimeType();
                $data_extension = $file->getClientOriginalExtension();
                if ($data_type == 'media')
                {
                    if ($data_extension != 'pdf')
                    {
                        $data_type = explode('/', $mime_type)[0];
                    }
                }
                if ($data_type == 'media' && $data_extension == 'pdf')
                {
                    $file_paths = $this->store_files($request->file('upload'), 'documents');
                    reset($file_paths);
                    $filename = key($file_paths);
                    $filepath = current($file_paths);
                    $file_name = pathinfo($filepath, PATHINFO_FILENAME);
                    $path_directories = $this->get_path('directories');
                    $path_directories .= $file_name . '/';
                    if (Storage::makeDirectory($path_directories))
                    {
                        $images_path = PDFController::pdf_to_images($filepath, $path_directories); 
                        $file_paths = $images_path; 
                    }
                }
                else
                {
                    $file_paths = $this->store_files($request->file('upload'), $data_type . 's');
                }
                $data_path = $file_paths;
                if ($data_type == 'scorm')
                {
                    $path_directories = $this->get_path('directories');
                    reset($file_paths);
                    $filename = key($file_paths);
                    $filepath = current($file_paths);
                    $file_name = pathinfo($filepath, PATHINFO_FILENAME);
                    $zip = new \ZipArchive;
                    $res_zip = $zip->open(Storage::path($filepath));
                    if ($res_zip === true)
                    {
                        $firstDirectory = "";
                        for ($i = 0; $i < $zip->numFiles; $i++) 
                        {
                            $stat = $zip->statIndex($i);
                            $file_Name = $stat['name'];
                            if (substr($file_Name, -1) === '/')
                            {
                                $firstDirectory = $file_Name;
                                break;
                            }
                        }
                        if (!$firstDirectory)
                        {
                            $path_directories_extract = $path_directories . $file_name . "/";
                        }
                        else
                        {
                            $path_directories_extract = $path_directories . $firstDirectory;
                        }

                        $zip->extractTo(Storage::path($path_directories_extract));
                        $zip->close();
                        $scorm_data_old = $firstDirectory;
                        $scorm_data_new = $path_directories . $file_name  . '/';
                        Storage::move($scorm_data_old, $scorm_data_new);
                        // $data['scorm_data'] = serialize([$firstDirectory => $scorm_data_new]);
                    }
                }
            }
            $data['data_type'] = $data_type;
            if ($request->hasFile('upload'))
            {
                if ($data_type == 'scorm')
                {
                    $data['data_path'] = serialize(isset($scorm_data_new) ? array_merge($data_path, [$scorm_data_new]) : $data_path);
                }
                else
                {
                    $data['data_path'] = serialize($data_path);
                }
            }
            if ($data_type == 'url')
            {
                $data['data_path'] = $request->input('data_path');
            }
            
            $data['question'] = $question;
            $data['scorm_version'] = $scorm_version;
            if ($task) 
            {
                $data = $this->get_not_null($data);
                $task->update($data);
            }
            else
            {
                $training = Training::find($training_id);
                if ($training)
                {
                    $this->check_company($module);
                }
                else
                {
                    $this->not_found();
                }  
                
                $data['module_id'] = $module_id;
                $last_task = Task::where('module_id', $module_id)->orderBy('order', 'DESC')->limit(1)->first();
                $order = 0;
                if ($last_task)
                {
                    $order = $last_task->order;
                }
                $data['order'] = $order + 1;
                $data['answer_type'] = 1;

                $task = Task::create($data);

                if ($task->question == 'yes')
                {
                    return redirect()->route('task.question.create', [$locale, $training_id, $module_id, $task->id])->with('success', __('message.successfully_registered'));
                }
            }
            return redirect()->route('task.view', [$locale, $training_id, $module_id, $task->id])->with('success', __('message.successfully_registered'));
        }
        else
        {
            $this->not_found();
        }
    }

    /**
     * update_order
     */
    public function update_order(Request $request, $locale)
    {
        $module_id = $request->module_id;
        $ids = $request->ids;
        foreach ($ids as $i => $id)
        {
            $task = Task::where('module_id', $module_id)->where('id', $id)->first();
            $task->update(['order' => $i + 1]);
        }
        return $ids;
    }

    /**
     * edit
     */
    public function edit(Request $request, $locale, $training_id, $module_id, $id)
    {
        $data = array();
        
        $training = Training::find($training_id);
        $this->check_company($training);
        $data['training'] = $training;

        $module = Module::find($module_id);
        $this->check_company($module);
        $data['module'] = $module;

        $task = Task::find($id);
        if (!$task) 
        {
            $this->not_found();
        }
        $data['task'] = $task;
        $data['answers'] = $task->answers;
        $data_path = $task->data_path;
        $data_path_name = "";
        if (in_array($task->data_type, array('image', 'video')))
        {
            $data_path = $this->unserialize($task->data_path);
            $data_path_name = array_keys($data_path);
            $data_path_name = end($data_path_name);
            $data_path = Storage::url(end($data_path));
        }
        if (in_array($task->data_type, array('media')))
        {
            $data_path_name = $task->title;
            $data_path = $this->unserialize($task->data_path);
            $new_data_path = [];
            foreach ($data_path as $key_data_path => $val_data_path)
            {
                $new_data_path[$key_data_path] = Storage::url($val_data_path);
            }
            $data_path = $new_data_path;
        }
        $data['data_path'] = $data_path;
        $data['data_path_name'] = $data_path_name;

        $documents = $this->unserialize($training->documents);
        $data['training_documents'] = $documents;
        $documents = $this->unserialize($task->documents);
        $data['documents'] = $documents;

        return view('training.module.edit-task', $data);
    }

    /**
     * view
     */
    public function view(Request $request, $locale, $training_id, $module_id, $id)
    {
        $data = array();
        
        $training = Training::find($training_id);
        $this->check_company($training);
        $data['training'] = $training;

        $module = Module::find($module_id);
        $this->check_company($module);
        $data['module'] = $module;

        $task = Task::find($id);
        if (!$task) 
        {
            $this->not_found();
        }
        $data['task'] = $task;
        $data['answers'] = $task->answers;
        $data_path = $task->data_path;
        $data_path_name = "";
        if (in_array($task->data_type, array('image', 'video')))
        {
            $data_path = $this->unserialize($task->data_path);
            $data_path_name = array_keys($data_path);
            $data_path_name = end($data_path_name);
            $data_path = Storage::url(end($data_path));
        }
        if (in_array($task->data_type, array('media')))
        {
            $data_path_name = $task->title;
            $data_path = $this->unserialize($task->data_path);
            $new_data_path = [];
            foreach ($data_path as $key_data_path => $val_data_path)
            {
                $new_data_path[$key_data_path] = Storage::url($val_data_path);
            }
            $data_path = $new_data_path;
        }
        $data['data_path'] = $data_path;
        $data['data_path_name'] = $data_path_name;

        $documents = $this->unserialize($task->documents);
        $data['documents'] = $documents;
        Session::put('downloads', $documents);
        
        return view('training.module.view-task', $data);
    }

    /**
     * question
     */
    public function question(Request $request, $locale, $training_id, $module_id, $id)
    {
        $data = array();
        
        $training = Training::find($training_id);
        $this->check_company($training);
        $data['training'] = $training;

        $module = Module::find($module_id);
        $this->check_company($module);
        $data['module'] = $module;

        $task = Task::find($id);
        if (!$task) 
        {
            $this->not_found();
        }
        $data['task'] = $task;
        $data_path = $task->data_path;
        $data_path_name = "";
        if (in_array($task->data_type, array('image', 'video')))
        {
            $data_path = $this->unserialize($task->data_path);
            $data_path_name = array_keys($data_path);
            $data_path_name = end($data_path_name);
            $data_path = Storage::url(end($data_path));
        }
        if (in_array($task->data_type, array('media')))
        {
            $data_path_name = $task->title;
            $data_path = $this->unserialize($task->data_path);
            $new_data_path = [];
            foreach ($data_path as $key_data_path => $val_data_path)
            {
                $new_data_path[$key_data_path] = Storage::url($val_data_path);
            }
            $data_path = $new_data_path;
        }
        $data['data_path'] = $data_path;
        $data['data_path_name'] = $data_path_name;

        return view('training.module.question', $data);
    }

    /**
     * question_store
     */
    public function question_store(Request $request, $locale)
    {
        $data = array();
        
        $id = $request->input('id');
        $module_id = $request->input('module_id');
        $training_id = $request->input('training_id');
        
        $validated = $request->validate([
            'title' => 'required',
            'answer_type' => 'required'
        ]);

        $training = Training::find($training_id);
        $this->check_company($training);
        $data['training'] = $training;

        $module = Module::find($module_id);
        $this->check_company($module);
        $data['module'] = $module;

        $task = Task::find($id);
        if (!$task) 
        {
            $this->not_found();
        }
        $task->title = $request->input('title');
        $task->question = $request->input('question');
        $task->answer_type = $request->input('answer_type');
        $saved = $task->save();
        $data['task'] = $task;

        if ($saved)
        {
            return redirect()->route('answer.index', [$locale, $training_id, $module_id, $task->id])->with('success', __('message.successfully_registered'));
        }
        else
        {
            return redirect()->route('answer.index', [$locale, $training_id, $module_id, $task->id])->with('error', __('message.error_registered'));
        }
    }
}
