<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Trainer;
use App\Models\User;
use Illuminate\Http\Request;

class TrainerController extends Controller
{
    /**
     * index
     */
    public function index()
    {
        $data = array();
        
        $trainers = Trainer::where('company_id', $this->company->id)->get();
        $trainer_ids = [];
        foreach ($trainers as $trainer) 
        {
            $trainer_ids[] = $trainer->id;
        }
        $data['trainers'] = $trainers;

        $trainer_users = User::where('company_id', $this->company->id)->whereNotIn('id', $trainer_ids)->get();
        $data['trainer_users'] = $trainer_users;

        return view('training.trainer', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $validated = $request->validate([
            'fname' => 'nullable',
            'lname' => 'nullable',
            'training_institution' => 'nullable'
        ]);

        $id = $request->input('id');
        $trainer = Trainer::find($id);

        if ($trainer) 
        {
            $this->check_company($trainer);
            $data = array(
                'fname' => $request->fname,
                'lname' => $request->lname,
                'training_institution' => $request->training_institution,
            );
            $trainer->update($data);
        }
        else
        {
            $user_ids = $request->user_ids;
            if (is_array($user_ids) && count($user_ids))
            {
                foreach ($user_ids as $user_id)
                {
                    $user = User::find($user_id);
                    if ($user)
                    {
                        $data = array(
                            'user_id' => $user->id,
                            'fname' => $user->fname,
                            'lname' => $user->lname,
                            'training_institution' => $request->training_institution,
                            'company_id' => $this->company->id
                        );
                        $trainer = Trainer::create($data);
                    }
                }
            }
            else
            {
                $data = array(
                    'user_id' => 0,
                    'fname' => $request->fname,
                    'lname' => $request->lname,
                    'training_institution' => $request->training_institution,
                    'company_id' => $this->company->id
                );
                $trainer = Trainer::create($data);
            }
        }
        
        return redirect()->route('trainer.list', [$locale])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $trainer = Trainer::find($id);

        if ($trainer) 
        {
            $this->check_company($trainer);
            $trainer->delete();

            return redirect()->route('trainer.list', [$locale])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('trainer.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }
}
