<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\TrainingCategory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;

class TrainingCategoryController extends Controller
{
    /**
     * index
     */
    public function index()
    {
        $data = array();
        
        $training_categories = TrainingCategory::where('company_id', $this->company->id)->get();
        $data['training_categories'] = $training_categories;

        return view('training.training-category', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $validated = $request->validate([
            'name' => 'required'
        ]);
        Log::info('Validated : ', $validated);

        $id = $request->input('id');
        $training_category = TrainingCategory::find($id);

        if ($training_category) 
        {
            $this->check_company($training_category);
            $data = array(
                'name' => $request->name,
            );
            $training_category->update($data);
        }
        else
        {
            $data = array(
                'name' => $request->name,
                'company_id' => $this->company->id
            );
            $training_category = TrainingCategory::create($data);
        }
        
        return redirect()->route('training-category.list', [$locale])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $training_category = TrainingCategory::find($id);

        if ($training_category) 
        {
            $this->check_company($training_category);
            $training_category->delete();

            return redirect()->route('training-category.list', [$locale])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('training-category.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }
}
