<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Department;
use App\Models\Equipement;
use App\Models\Form;
use App\Models\Location;
use App\Models\Module;
use App\Models\Platform;
use App\Models\Report;
use App\Models\RList;
use App\Models\Signer;
use App\Models\Subdepartment;
use App\Models\TrackRecord;
use App\Models\TrackRecordAnswer;
use App\Models\Training;
use App\Models\TrainingLevel;
use App\Models\TrainingResult;
use App\Models\UserTraining;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Storage;
use App\Models\LiveReading;
use App\Models\User;
use App\Models\Groups;

class TrainingController extends Controller
{
    protected $data;

    /**
     * contruct
     */
    public function __construct(Request $request)
    {
        parent::__construct($request);

        $data = array();
        
        $data['category'] = Category::find($request->category_id);
        $data['platforms'] = $request->category_id == 1 ? Platform::get() : Platform::where("id", 1)->get();
        $data['training_result'] = TrainingResult::get();
        $data['training_levels'] = TrainingLevel::where('company_id', $this->company->id)->get();
        $data['departments'] = Department::where('company_id', $this->company->id)->get();
        $data['subdepartments'] = Subdepartment::where('company_id', $this->company->id)->get();
        $data['locations'] = Location::where('company_id', $this->company->id)->get();
        $data['equipements'] = Equipement::where('company_id', $this->company->id)->get();
        $data['forms'] = Form::where('company_id', $this->company->id)->get();
        $data['surveys'] = Training::where('company_id', $this->company->id)->where('category_id', 5)->get();
        $data['signers'] = Signer::where('company_id', $this->company->id)->get();
        $data['yes_or_no_lists'] = RList::where('type', 'yes_or_no')->get();
        $data['following_trainings'] = Training::select('id', 'name', 'prefix')->where('company_id', $this->company->id)->where('active', 1)->where('disable', 0)->get();
        $data['procedure'] = $request->procedure;

        $this->data = $data;
    }

    /**
     * index
     */
    public function index()
    {
        $data = array();
        
        $categories = Category::where('id', '<=', 6)->get();    
        foreach ($categories as $category)
        {
            $data['training_' . $category->id] = array();
        }

        $count_all_training = 0;
        $count_disable_training = 0;
        $count_paid_training = 0;
        $count_immutable_training = 0;
        $trainings = Training::where('company_id', $this->company->id)->get();
        foreach ($trainings as $i => $training)
        {
            $training->duration_th = $this->sec_to_time($training->duration_th);
            $training->duration_pr = $this->sec_to_time($training->duration_pr);
            $data['training_' . $training->category_id][$i] = $training;
            $count_all_training++;
            if ($training->disable) $count_disable_training++;
            if ($training->paid) $count_paid_training++;
            if ($training->is_immutable) $count_immutable_training++;
        }

        $data['count_all_training'] = $count_all_training;
        $data['count_disable_training'] = $count_disable_training;
        $data['count_paid_training'] = $count_paid_training;
        $data['count_immutable_training'] = $count_immutable_training;

        $data['forms_7'] = Form::where('company_id', $this->company->id)->where('category_id', 7)->get();

        $data['forms_9'] = Form::where('company_id', $this->company->id)->where('category_id', 9)->get();

        $data['reports'] = Report::where('company_id', $this->company->id)->get();
        
        return view('training.index', $data);
    }

    /**
     * create
     */
    public function create(Request $request)
    {
        $data = $this->data;
        $category_id = $request->category_id;
        $data['procedure'] = $request->procedure;
        if ($category_id == 1)
        {
            return view('training.edit', $data);
        }
        else
        {
            return view('training.edit-simple', $data);
        }
    }

    /**
     * edit
     */
    public function edit(Request $request)
    {
        $data = $this->data;

        $training = Training::find($request->id);
        $data['training'] = $training;
        if ($training) 
        {
            $this->check_company($training);
            
            $data['category'] = $training->category;
            $category_id = $training->category_id;
        }
        else
        {
            $this->not_found();
        }

        if ($category_id == 1)
        {
            return view('training.edit', $data);
        }
        else
        {
            return view('training.edit-simple', $data);
        }
    }

    /**
     * view
     */
    public function view(Request $request)
    {
        $data = $this->data;

        $training = Training::find($request->id);
        $data['training'] = $training;
        if ($training) 
        {
            $this->check_company($training);
            
            $data['category'] = $training->category;

            $data['modules'] = $training->modules;

            $documents = $this->unserialize($training->documents);
            $data['documents'] = $documents;
            Session::put('downloads', $documents);
        }
        else
        {
            $this->not_found();
        }
        $data['all_modules'] = Module::where('company_id', $this->company->id)->get();

        return view('training.view', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $id = $request->input('id');
        $training = Training::find($id);
        $procedure = $request->input('procedure');

        $validation = [
            'platform_id' => 'required',
            'name' => 'required',
            'prefix' => 'required',
            'result' => 'required'
        ];
        if ($training)
        {
            unset($validation['platform_id']);
        }
        if ($procedure)
        {
            unset($validation['platform_id']);
            unset($validation['result']);
        }

        $validated = $request->validate($validation);

        $data = $request->all();
        unset($data['_token']);
        unset($data['id']);

        if (!$training)
        {
            $data['creator_id'] = $this->user->id;
            $data['company_id'] = $this->company->id;
        }
        if ($procedure)
        {
            $data['platform_id'] = 1;
            $data['result'] = 1;
        }

        if ($request->hasFile('image')) 
        {
            $file_paths = $this->store_files($request->file('image'), 'images');
            $data['image'] = serialize($file_paths);
        }
        if ($request->hasFile('file')) 
        {
            $file_paths = $this->store_files($request->file('file'), 'documents');
            $data['image'] = serialize($file_paths);
        }
        if ($request->hasFile('documents')) 
        {
            $file_paths = $this->store_files($request->file('documents'), 'documents');
            $data['documents'] = serialize($file_paths);
        }
        
        $data = $this->get_not_null($data);
        if ($training) 
        {
            $this->check_company($training);
            $training->update($data);
        }
        else
        {
            $training = Training::create($data);
            $category_id = $request->category_id;
            if ($category_id == 2 && $training)
            {
                $data = [
                    "name" => $training->name,
                    "type" => "Field does not apply",
                    "company_id" => $this->company->id,
                    "category_id" => $category_id
                ];
                $module = Module::create($data);
                $module->trainings()->attach([$training->id]);
            }
        }
        
        return redirect()->route('training.index', $locale)->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $training = Training::find($id);

        if ($training) 
        {
            $this->check_company($training);

            if ($training->image) 
            {
                Storage::delete('public/' . $training->image);
            }
            if ($training->documents) 
            {
                Storage::delete('public/' . $training->documents);
            }

            $training->delete();

            return redirect()->route('training.index', $locale)->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('training.index', $locale)->with('error', __('message.error_deleting'));
        }
    }

    /**
     * module_select
     */
    public function module_select(Request $request, $locale)
    {
        $module_ids = $request->module_id;
        $training_id = $request->training_id;
        $training = Training::find($training_id);
        if ($training) 
        {
            $this->check_company($training);

            $training->modules()->sync($module_ids);

            return redirect()->route('training.view', [$locale, $training_id])->with('success', __('message.successfully_registered'));
        }
        else
        {
            return redirect()->route('training.index', $locale)->with('error', __('message.error_training_not_found'));
        }
    }

    /**
     * module_duplicate
     */
    public function module_duplicate(Request $request, $locale)
    {
        $module_ids = $request->module_id;
        $training_id = $request->training_id;
        $training = Training::find($training_id);
        if ($training) 
        {
            $this->check_company($training);

            if ($module_ids)
            {
                $modules = Module::whereIn('id', $module_ids)->get();
                $new_module_ids = [];
                foreach ($modules as $module)
                {
                    $new_module = $module->replicate();
                    $new_module->save();
                    $new_module_id = $new_module->id;
                    $new_module_ids[] = $new_module_id;
                    foreach ($module->tasks as $task)
                    {
                        $new_task = $task->replicate();
                        $new_task->module_id = $new_module_id;
                        if (in_array($task->data_type, array('image', 'video')))
                        {
                            $new_task->data_path = serialize([]);
                            $data_path = $this->unserialize($task->data_path);
                            if (count($data_path))
                            {
                                $new_data_path = [];
                                foreach ($data_path as $key_dp => $val_dp)
                                {
                                    $file_info = pathinfo($val_dp);
                                    $new_path = $file_info['dirname'] . '/' . $file_info['filename'] . '_' . $new_module_id . '.' . $file_info['extension'];
                                    Storage::copy($val_dp, $new_path);
                                    $new_data_path[$key_dp] = $new_path;
                                }
                                $new_task->data_path = serialize($new_data_path);
                            }
                        }
                        $documents = $this->unserialize($task->documents);
                        $new_task->documents = serialize([]);
                        if (count($documents))
                        {
                            $new_documents = [];
                            foreach ($documents as $key_doc => $val_doc)
                            {
                                $file_info = pathinfo($val_doc);
                                $new_path = $file_info['dirname'] . '/' . $file_info['filename'] . '_' . $new_module_id . '.' . $file_info['extension'];
                                Storage::copy($val_doc, $new_path);
                                $new_documents[$key_doc] = $new_path;
                            }
                            $new_task->documents = serialize($new_documents);
                        }
                        $new_task->save();
                    }
                    $new_module->trainings()->attach([$training_id]);
                }

                return redirect()->route('training.view', [$locale, $training_id])->with('success', __('message.successfully_registered'));
            }
            else
            {
                return redirect()->route('training.view', [$locale, $training_id])->with('warning', __('message.no_module_selected'));
            }
        }
        else
        {
            return redirect()->route('training.index', $locale)->with('error', __('message.error_training_not_found'));
        }
    }

    /**
     * delete_file
     */
    public function delete_file(Request $request, $locale, $filename)
    {
        $training_id = $request->id;
        $training = Training::find($training_id);

        if ($training) 
        {
            $this->check_company($training);

            $documents = $this->unserialize($training->documents);
            
            if (array_key_exists($filename, $documents))
            {
                if (Storage::exists($documents[$filename]))
                {
                    Storage::delete($documents[$filename]);
                }
                unset($documents[$filename]);
                $training->update(["documents" => serialize($documents)]);
                
                return redirect()->route('training.view', [$locale, $training_id])->with('success', __('message.successfully_deleted'));
            }
            return redirect()->route('training.view', [$locale, $training_id])->with('warning', __('message.not_found'));
        } 
        else 
        {
            return redirect()->route('training.index', $locale)->with('error', __('message.error_deleting'));
        }
    }

    /**
     * disable
     */
    public function disable(Request $request, $locale)
    {
        $training_id = $request->input('training_id');
        $training = Training::find($training_id);
        
        if ($training)
        {
            $training->disable = 1;
            $training->save();

            return redirect()->route('training.view', [$locale, $training_id])->with('success', __('message.successfully_registered'));
        }
        else
        {
            $this->not_found();
        }
    }

    /**
     * enable
     */
    public function enable(Request $request, $locale)
    {
        $training_id = $request->input('training_id');
        $training = Training::find($training_id);
        
        if ($training)
        {
            $training->disable = 0;
            $training->save();

            $tasks = [];
            foreach ($training->modules as $module)
            {
                foreach ($module->tasks as $task)
                {
                    $tasks[$task->id] = ['task' => $task, 'module' => $module];
                }
            }

            $tasks_new = $tasks;
            $user_trainings = UserTraining::where('training_id', $training->id)->get();
            foreach ($user_trainings as $user_training)
            {
                foreach ($user_training->track_records as $track_record)
                {
                    $task_id = $track_record->task->id;
                    unset($tasks_new[$task_id]);
                }

                foreach ($tasks_new as $task_id => $task)
                {
                    $module = $task['module'];
                    $task = $task['task'];
                    $this->create_track_record($user_training, $module, $task);
                }
            }
            
            return redirect()->route('training.index', [$locale])->with('success', __('message.successfully_registered'));
        }
        else
        {
            $this->not_found();
        }
    }

    /**
     * create_track_record
     */
    protected function create_track_record($user_training, $module, $task)
    {
        $track_record = new TrackRecord();
        $track_record->user_training_id = $user_training->id;
        $track_record->module_id = $module->id;
        $track_record->module_order = $module->pivot->order;
        $track_record->module_name = $module->name;
        $track_record->task_id = $task->id;
        $track_record->task_order = $task->order;
        $track_record->task_title = $task->title;
        $track_record->task_data_path = $task->data_path;
        $track_record->data_type = $task->data_type;
        $track_record->trainee = 0;
        $track_record->instructor = 0;
        $track_record->supervisor = 0;
        $track_record->question = $task->question;
        $track_record->answer_type = $task->answer_type;
        $track_record->answer_points = -1;
        $track_record->task_duration = $task->duration;
        $track_record->save();

        foreach ($task->answers as $answer)
        {
            $track_record_answer = new TrackRecordAnswer();
            $track_record_answer->track_record_id = $track_record->id;
            $track_record_answer->answer = $answer->answer;
            $track_record_answer->correct = $answer->correct;
            $track_record_answer->feedback = $answer->feedback;
            $track_record_answer->filename = $answer->filename;
            $track_record_answer->filepath = $answer->filepath;
            $track_record_answer->save();
        }

        return $track_record;
    }

    public function live_reading($locale)
    {
        $data = [];
        $request = request();
        $data['category'] = Category::whereIn('id', [1, 2])->get();

        if($request->isMethod('post')) {
            $category_id = $request->input('category_id');
            $training_id = $request->input('training_id');
            return redirect()->route('trainings.select-trainee', [
                'locale' => $locale,
                'category_id' => $category_id,
                'training_id' => $training_id
            ]);
        }
        return view('training.live_reading', compact('data'));
    }

    public function get_by_category($locale, $category_id)
    {
        $trainings = Training::where('category_id', $category_id)->get();
        return response()->json($trainings);
    }

    public function select_trainee($locale, $category_id, $training_id)
    {
        $request = request();
        $search = null;
        $depId = null;
        $subdepId = null;
        $locId = null;
        $liveMembers = [];

        if ($request->isMethod('post')) {
            $search = $request->input('search');
            $depId = $request->input('departments');
            $subdepId = $request->input('subdepartments'); 
            $locId = $request->input('locations');
            $liveMembers = $request->input('live_members', []);
            if (is_string($liveMembers)) {
                $liveMembers = explode(',', $liveMembers);
            }
            $liveMembers = array_filter($liveMembers); // Remove empty values
            
            if (!empty($liveMembers)) {
                // $history = new LiveReading();
                // $history->training_id = $training_id;
                // $history->user_id = auth()->id();
                // $history->start_time = now();
                // $history->status = 'in_progress';
                // $history->save();

                foreach ($liveMembers as $member_id) {
                    $participant = new Livereading();
                    $participant->trainingID = $training_id;
                    $participant->companyID = auth()->user()->company_id;
                    $participant->trainees = $member_id;
                    $participant->date = now();
                    $participant->completed = false;
                    $participant->trainerID = auth()->id();
                    $participant->duration = 0;
                    $participant->updated_at = now();
                    $participant->save();
                }

                return redirect()->route('trainings.live-reading', [
                    'locale' => $locale,
                    'training_id' => $training_id,
                    'members' => implode(',', $liveMembers)
                ]);
            }
        }

        $users = User::where('company_id', $this->company->id)
            ->where('username', '!=', 'admin')
            ->where('status', '>', 0)
            ->whereNotIn('id', $liveMembers);

        if ($search) {
            $users->where(function($query) use ($search) {
                $query->where('lname', 'like', "%$search%")
                    ->orWhere('fname', 'like', "%$search%")
                    ->orWhere('id', $search);
            });
        }

        if ($depId) {
            $users->where('department_id', $depId);
        }
        if ($subdepId) {
            $users->where('subdepartment_id', $subdepId);
        }
        if ($locId) {
            $users->where('location_id', $locId);
        }

        $availUsers = $users->orderBy('lname')
            ->orderBy('fname')
            ->get()
            ->mapWithKeys(function($user) {
                $name = $user->lname . ' ' . $user->fname . ' (' . $user->id . ')';
                if ($user->employee_id) {
                    $name .= ' [' . str_pad($user->employee_id, 3, "0", STR_PAD_LEFT) . ']';
                }
                return [$user->id => $name];
            })->toArray();

        $members = User::whereIn('id', isset($model->live_members) ? $model->live_members : [])
            ->where('company_id', $this->company->id)
            ->get()
            ->mapWithKeys(function($user) {
                $name = $user->lname . ' ' . $user->fname . ' (' . $user->id . ')';
                if ($user->employee_id) {
                    $name .= ' [' . $user->employee_id . ']';
                }
                return [$user->id => $name];
            })->toArray();

        $userGroups = Groups::where('company_id', $this->company->id)
            ->get()
            ->mapWithKeys(function($group) {
                return [$group->id => $group->name];
            })->toArray();

        $departments = Department::where('company_id', $this->company->id)
            ->orderBy('name')
            ->get();
            
        $subdepartments = Subdepartment::where('company_id', $this->company->id)
            ->orderBy('name')
            ->get();

        $locations = Location::where('company_id', $this->company->id)
            ->orderBy('name')
            ->get();

        return view('training.select_trainee', [
            'availUsers' => $availUsers,
            'userGroups' => $userGroups,
            'members' => $members,
            'departments' => $departments,
            'subdepartments' => $subdepartments,
            'locations' => $locations,
            'search' => $search,
            'category_id' => $category_id,
            'training_id' => $training_id,
            'members' => $members
        ]);
    }
}

