<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Presentiel;
use App\Models\TrainingCost;
use App\Models\TrainingCostType;
use Illuminate\Http\Request;
use Psy\VarDumper\Presenter;

class TrainingCostController extends Controller
{
    /**
     * index
     */
    public function index(Request $request)
    {
        $data = array();

        $presentiel_id = $request->presentiel_id;
        $presentiel = Presentiel::find($presentiel_id);
        if ($presentiel)
        {
            $this->check_company($presentiel);

            $training_cost_types = TrainingCostType::where('company_id', $this->company->id)
                                        ->orWhereNull('company_id')
                                        ->get();

            $data['presentiel'] = $presentiel;
            $data['training_costs'] = $presentiel->training_costs;

            return view('training.presentiel.cost-list', $data);
        }
        else
        {
            $this->not_found();
        }
    }

    /**
     * add
     */
    public function add(Request $request)
    {
        $data = array();

        $presentiel_id = $request->presentiel_id;
        $presentiel = Presentiel::find($presentiel_id);
        if ($presentiel)
        {
            $this->check_company($presentiel);

            $training_cost_types = TrainingCostType::where('company_id', $this->company->id)
                                        ->orWhereNull('company_id')
                                        ->get();

            $data['presentiel'] = $presentiel;
            $data['training_cost_types'] = $training_cost_types;

            return view('training.presentiel.cost-add', $data);
        }
        else
        {
            $this->not_found();
        }
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $presentiel_id = $request->input('presentiel_id');
        $type = $request->input('type');
        $cost = $request->input('cost');
        $date = $request->input('date');
        $notes = $request->input('notes');

        $presentiel = Presentiel::find($presentiel_id);
        if ($presentiel) 
        {
            $this->check_company($presentiel);
            $data = array(
                'presentiel_id' => $presentiel_id,
                'type' => $type,
                'cost' => $cost,
                'date' => $date,
                'notes' => $notes
            );
            $training_cost = TrainingCost::create($data);

            PresentielController::update_cost($training_cost->presentiel);
            
            return redirect()->route('presentiel.view', [$locale, $training_cost->presentiel_id])->with('success', __('message.successfully_registered'));
        }
        else
        {
            $this->not_found();
        }
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $training_cost = TrainingCost::find($id);

        if ($training_cost) 
        {
            $training_cost->delete();

            PresentielController::update_cost($training_cost->presentiel);

            return redirect()->route('presentiel.view', [$locale, $training_cost->presentiel_id])->with('success', __('message.successfully_deleted'));
        } 
        else
        {
            $this->not_found();
        }
    }
}
