<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Location;
use App\Models\Platform;
use App\Models\Team;
use App\Models\Training;
use App\Models\TrainingDemand;
use App\Models\TrainingDemandLevel;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Session;

class TrainingDemandController extends Controller
{
    protected $request_data;

    /**
     * contruct
     */
    public function __construct(Request $request)
    {
        parent::__construct($request);
    }

    /**
     * index
     */
    public function index(Request $request, $locale)
    {
        $data = array();
        
        $data['users'] = User::where('company_id', $this->company->id)->where('status', 1)->select('id', 'lname', 'fname', 'ref', 'nas')->get();
        $data['platforms'] = Platform::get();
        $data['categories'] = Category::whereIn("id", [1, 2, 3, 6])->get();
        $data['trainings'] = Training::where('company_id', $this->company->id)->where('active', 1)->select('id', 'name', 'prefix')->get();
        $training_demand_levels = TrainingDemandLevel::where('company_id', $this->company->id)->get();
        $data['users_level1'] = [];
        $data['users_level2'] = [];
        $data['users_level3'] = [];
        foreach ($training_demand_levels as $training_demand_level)
        {
            if ($training_demand_level->req_step1_notification)
            {
                $data['users_level1'][$training_demand_level->user_id] = $training_demand_level->user;
            }
            if ($training_demand_level->req_step2_notification)
            {
                $data['users_level2'][$training_demand_level->user_id] = $training_demand_level->user;
            }
            if ($training_demand_level->req_step3_notification)
            {
                $data['users_level3'][$training_demand_level->user_id] = $training_demand_level->user;
            }
        }
        $data['locations'] = Location::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
        $data['teams'] = Team::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
        $data['decision_status'] = [0 => __('app.Pending'), 1 => __('app.Approved'), -1 => __('app.Declined')];

        $request_data = $request->all();
        $this->request_data = $request_data;
        $parameters = $request_data;
        $parameters['locale'] = $locale;
        $data['parameters'] = $parameters;
        Session::put('training_demand_filter', $request_data);
        
        $training_demands = $this->get_qb()->paginate($this->setting->paginate);
        
        $count_approved = 0;
        $count_declined = 0;
        $count_pending = 0;
        foreach ($training_demands as $i => $training_demand)
        {
            switch ($training_demand->status)
            {
                case 'approved':
                    $count_approved++;
                    break;
                case 'declined':
                    $count_declined++;
                    break;
                case 'pending':
                    $count_pending++;
                    break;
            }
            $training_demand = $this->set_level($training_demand);
            $training_demands[$i] = $training_demand;
        }
        $data['decision_status'] = [0 => __('app.Pending'), 1 => __('app.Approved'), -1 => __('app.Declined')];
        $data['training_demands'] = $training_demands;
        $data['count_approved'] = $count_approved;
        $data['count_declined'] = $count_declined;
        $data['count_pending'] = $count_pending;
        $data['is_closed'] = false;

        $training_demand_levels = TrainingDemandLevel::where('user_id', $this->user->id)
                                    ->where('level', 4)
                                    ->get();
        $data['is_level4'] = count($training_demand_levels);

        return view('training.demand.index', $data);
    }

    /**
     * get_qb
     */
    public function get_qb()
    {
        $training_demands_qb = TrainingDemand::select('training_demand.*');

        $training_demands_qb = $training_demands_qb->leftJoin('training', 'training.id', '=', 'training_demand.training_id');
        $training_demands_qb = $training_demands_qb->where('training_demand.company_id', $this->company->id);

        $this->set_qb('id', 'training_demand.id', '=', $training_demands_qb);
        $this->set_qb('applicant_id', 'training_demand.applicant_id', '=', $training_demands_qb);
        $this->set_qb('level2', 'training_demand.level2', 'like', $training_demands_qb);
        $this->set_qb('level3', 'training_demand.level3', 'like', $training_demands_qb);
        $this->set_qb('status', 'training_demand.status', '=', $training_demands_qb);
        $this->set_qb('decision_made', 'training_demand.decision_made', '=', $training_demands_qb);
        $this->set_qb('demand_date', 'training_demand.demand_date', '=', $training_demands_qb);
        $this->set_qb('user_id', 'training_demand.user_id', '=', $training_demands_qb);
        $this->set_qb('platform_id', 'training.platform_id', '=', $training_demands_qb);
        $this->set_qb('training_name', 'training.name', 'like', $training_demands_qb);
        $this->set_qb('team_id', 'training_demand.team_id', '=', $training_demands_qb);

        return $training_demands_qb;
    }

    /**
     * closed
     */
    public function closed(Request $request)
    {
        $data = array();
        
        $training_demands = TrainingDemand::where('company_id', $this->company->id)
                            ->where('is_closed', 1)
                            ->paginate($this->setting->paginate);
        
        foreach ($training_demands as $i => $training_demand)
        {
            $training_demand = $this->set_level($training_demand);
            $training_demands[$i] = $training_demand;
        }
        $data['decision_status'] = [0 => __('app.Pending'), 1 => __('app.Approved'), -1 => __('app.Declined')];
        $data['training_demands'] = $training_demands;
        $data['is_closed'] = true;

        return view('training.demand.index', $data);
    }

    /**
     * create
     */
    public function create(Request $request)
    {
        $data = array();
        
        $data['users'] = User::where('company_id', $this->company->id)->where('status', 1)->select('id', 'lname', 'fname', 'ref', 'nas')->get();
        $data['platforms'] = Platform::get();
        $data['categories'] = Category::whereIn("id", [1, 2, 3, 6])->get();
        $data['trainings'] = Training::where('company_id', $this->company->id)->where('active', 1)->select('id', 'name', 'prefix')->get();
        $training_demand_levels = TrainingDemandLevel::where('company_id', $this->company->id)->get();
        $data['users_level1'] = [];
        $data['users_level2'] = [];
        $data['users_level3'] = [];
        foreach ($training_demand_levels as $training_demand_level)
        {
            if ($training_demand_level->req_step1_notification)
            {
                $data['users_level1'][$training_demand_level->user_id] = $training_demand_level->user;
            }
            if ($training_demand_level->req_step2_notification)
            {
                $data['users_level2'][$training_demand_level->user_id] = $training_demand_level->user;
            }
            if ($training_demand_level->req_step3_notification)
            {
                $data['users_level3'][$training_demand_level->user_id] = $training_demand_level->user;
            }
        }
        $data['locations'] = Location::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
        $data['teams'] = Team::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
        $data['decision_status'] = [0 => __('app.Pending'), 1 => __('app.Approved'), -1 => __('app.Declined')];

        return view('training.demand.add', $data);
    }

    /**
     * view
     */
    public function view(Request $request)
    {
        $data = array();
        $id = $request->id;

        $training_demand = TrainingDemand::find($id);
        if ($training_demand)
        {
            $this->check_company($training_demand);
            $training_demand = $this->set_level($training_demand);
            $data['decision_status'] = [0 => __('app.Pending'), 1 => __('app.Approved'), -1 => __('app.Declined')];
            $data['training_demand'] = $training_demand;
        }
        else
        {
            $this->not_found();
        }

        return view('training.demand.view', $data);
    }

    /**
     * edit
     */
    public function edit(Request $request)
    {
        $data = array();
        $id = $request->id;

        $training_demand = TrainingDemand::find($id);
        if ($training_demand)
        {
            $this->check_company($training_demand);
            $training_demand = $this->set_level($training_demand);
            $data['decision_status'] = [0 => __('app.Pending'), 1 => __('app.Approved'), -1 => __('app.Declined')];
            $data['training_demand'] = $training_demand;
        }
        else
        {
            $this->not_found();
        }

        return view('training.demand.edit', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $data = $request->all();
        $id = $request->input('id');
        if (!$id)
        {
            $user_ids = $data["user_ids"] ?? [];
            $training_ids = $data["training_ids"] ?? [];
            $level2 = $data["level2"] ?? [];
            $level3 = $data["level3"] ?? [];
            
            unset($data['_token']);
            unset($data['id']);
            unset($data['user_ids']);
            unset($data['training_ids']);
            unset($data['level2']);
            unset($data['level3']);
            if (array_search('select_all', $level2)) unset($level2[array_search('select_all', $level2)]);
            if (array_search('select_all', $level3)) unset($level3[array_search('select_all', $level3)]);
            if (array_search('select_all', $user_ids)) unset($user_ids[array_search('select_all', $user_ids)]);
            if (array_search('select_all', $training_ids)) unset($training_ids[array_search('select_all', $training_ids)]);
            
            $training_demand_levels = TrainingDemandLevel::where('user_id', $this->user->id)
                                        ->where('level', 3)
                                        ->get();
            $is_level3 = count($training_demand_levels);

            foreach ($training_ids as $training_id)
            {
                $data['training_id'] = $training_id;
                $data = $this->get_not_null($data);
                
                if ($level2)
                {
                    $data['level2'] = serialize($level2);
                }
                if ($level3)
                {
                    $data['level3'] = serialize($level3);
                }
                if (isset($data['location']) && $data['location'])
                {
                    $location = Location::find($data['location']);
                    $data['location'] = $location->name;
                }

                $training_demand = TrainingDemand::find($id);
                if ($training_demand)
                {
                    $this->check_company($training_demand);
                    $training_demand->update($data);

                    foreach ($training_demand->training_demand_users as $training_demand_user)
                    {
                        $training_demand_user->delete();
                    }
                }
                else
                {
                    $data['company_id'] = $this->company->id;
                    $data['applicant_id'] = $this->user->id;
                    $data['user_id'] = $this->user->id;
                    $data['status'] = 0;
                    $training_demand = TrainingDemand::create($data);
                }

                if ($training_demand && $user_ids)
                {
                    $training_demand->users()->attach($user_ids);
                    if ($is_level3)
                    {
                        $training_demand->decision_date = date('Y-m-d');
                        $training_demand->decision_made = 1;
                        $training_demand->completed_date = date('Y-m-d');
                        $training_demand->status = 1;
                        $training_demand->save();
                    }
                }
            }
        }
        else
        {
            $training_demand = TrainingDemand::find($id);
            $training_demand->decision_made = $data['decision_made'] ?? 0;
            $training_demand->status = $data['status'] ?? 0;
            $training_demand->start_date = $data['start_date'] ?? '';
            $training_demand->save();
        }
        
        return redirect()->route('training-demand.list', [$locale])->with('success', __('message.successfully_registered'));
    }

    /**
     * close
     */
    public function close(Request $request, $locale)
    {
        $id = $request->input('id');
        $training_demand = TrainingDemand::find($id);

        if ($training_demand) 
        {
            $this->check_company($training_demand);
            $training_demand->is_closed = 1;
            $training_demand->completed_date = date("Y-m-d");
            $training_demand->save();

            return redirect()->route('training-demand.list', [$locale])->with('success', __('message.successfully_registered'));
        } 
        else
        {
            return redirect()->route('training-demand.list', [$locale])->with('error', __('message.error_registered'));
        }
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $training_demand = TrainingDemand::find($id);

        if ($training_demand) 
        {
            $this->check_company($training_demand);
            $training_demand->delete();

            return redirect()->route('training-demand.list', [$locale])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('training-demand.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * set_level
     */
    protected function set_level($training_demand)
    {
        $level2 = $this->unserialize($training_demand->level2);
        $level3 = $this->unserialize($training_demand->level3);
        $training_demand->level2 = '';
        $training_demand->level3 = '';
        if (count($level2))
        {
            foreach (User::whereIn('id', $level2)->get() as $user)
            {
                if ($training_demand->level2 != '')
                {
                    $training_demand->level2 .= ' - ';
                }
                $training_demand->level2 .= $user->fname . ' ' . $user->lname;
            }
        }
        if (count($level3))
        {
            foreach (User::whereIn('id', $level3)->get() as $user)
            {
                if ($training_demand->level3 != '')
                {
                    $training_demand->level3 .= ' - ';
                }
                $training_demand->level3 .= $user->fname . ' ' . $user->lname;
            }
        }
        return $training_demand;
    }

    /**
     * training_list
     */
    public function training_list(Request $request)
    {
        $platform_id = $request->platform_id;
        $category_id = $request->category_id;

        $trainings_qb = Training::select('id', 'name', 'prefix')
                        ->where('company_id', $this->company->id)
                        ->where('active', 1);
        if ($platform_id)
        {
            $trainings_qb = $trainings_qb->where('platform_id', $platform_id);
        }
        if ($category_id)
        {
            $trainings_qb = $trainings_qb->where('category_id', $category_id);
        }
        $trainings = $trainings_qb->get();
        $data['trainings'] = $trainings;

        return view('training.demand.training-list', $data);
    }

    /**
     * export
     */
    public function export(Request $request)
    {
        $this->request_data = Session::get('training_demand_filter', []);
        
        $type = $request->type;
        $decision_status = [0 => __('app.Pending'), 1 => __('app.Approved'), -1 => __('app.Declined')];

        $training_demands = $this->get_qb()->get();

        $data = [
            [
                __("#"), 
                __("app.applicant_1st_level"),
                __("app.2nd_level_treatment"),
                __("app.3rd_level_treatment"),
                __("app.2nd_level_status"),
                __("app.final_status"),
                __("app.demand_date"),
                __("app.user"),
                __("app.platform"),
                __("app.training"),
                __("app.description"),
                __("app.team"),
                __("app.date_of_training"),
                __("app.training_location")
            ]
        ];
        
        foreach ($training_demands as $training_demand)
        {
            $training_demand = $this->set_level($training_demand);
            $data[$training_demand->id] = [
                $training_demand->id,
                $training_demand->applicant->fname . ' ' . $training_demand->applicant->lname,
                $training_demand->level2,
                $training_demand->level3,
                $training_demand->status == -1 ? $training_demand->refusal_raison_level2 : ($training_demand->status == 1 ? $training_demand->completed_date : ($training_demand->status == 0 ? $decision_status[$training_demand->status] : '')),
                $training_demand->decision_made == -1 ? $training_demand->refusal_raison_level3 : ($training_demand->decision_made == 1 ? $training_demand->decision_date : ($training_demand->decision_made == 0 ? $decision_status[$training_demand->decision_made] : '')),
                $training_demand->demand_date,
                $training_demand->user->fname . ' ' . $training_demand->user->lname,
                $training_demand->training->platform->type,
                $training_demand->training->name,
                $training_demand->description,
                $training_demand->team->name,
                $training_demand->start_date,
                $training_demand->location
            ];
        }

        $filename = "export_training_demande_" . date('Ymd_His') . "." . $type;

        $handle = fopen($filename, 'w+');
        foreach ($data as $row) 
        {
            if ($type == "csv")
            {
                fputcsv($handle, $row);
            }
            else
            {
                fputcsv($handle, $row, "\t");
            }
        }
        fclose($handle);

        return Response::download($filename)->deleteFileAfterSend();
    }
}
