<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\TrainingDemandLevel;
use App\Models\User;
use Illuminate\Http\Request;

class TrainingDemandLevelController extends Controller
{
    /**
     * index
     */
    public function index(Request $request)
    {
        $data = array();
        
        $training_demand_levels = TrainingDemandLevel::where('company_id', $this->company->id)
                            ->paginate($this->setting->paginate);
        
        $data['training_demand_levels'] = $training_demand_levels;
        $data['levels'] = [1 => __('app.supervisor'), 2 => __('app.general_foreman'), 3 => __('app.training_department')];
        
        return view('training.demand.level-list', $data);
    }

    /**
     * create
     */
    public function create(Request $request)
    {
        $data = array();
        $level = $request->level;
        
        $training_demand_levels = TrainingDemandLevel::where('company_id', $this->company->id)
                                    ->where('level', $level)
                                    ->get();
        $users_in_group_id = [];
        $users_in_group = [];
        foreach ($training_demand_levels as $training_demand_level)
        {
            $users_in_group_id[$training_demand_level->user_id] = $training_demand_level->user_id;
            $users_in_group[$training_demand_level->user_id] = $training_demand_level->user;
        }
        
        $available_users = User::where('company_id', $this->company->id)
                            ->where('status', 1)
                            ->whereNotIn('id', $users_in_group_id)
                            ->select('id', 'lname', 'fname', 'ref', 'nas')
                            ->get();
        
        $data['available_users'] = $available_users;
        $data['users_in_group'] = $users_in_group;
        $data['training_demand_levels'] = $training_demand_levels;
        $data['level'] = $level;
        $data['levels'] = [1 => __('app.supervisor'), 2 => __('app.general_foreman'), 3 => __('app.training_department')];

        return view('training.demand.level-edit', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $data = $request->all();
        $level = $data['level'];
        $action = $data['action'] ?? [];
        $available_users = $data['available_users'] ?? [];
        $users_in_group = $data['users_in_group'] ?? [];

        if ($action == 'remove')
        {
            foreach ($users_in_group as $user_id)
            {
                TrainingDemandLevel::where('user_id', $user_id)
                                ->where('company_id', $this->company->id)
                                ->where('level', $level)
                                ->delete();
            }
        }

        if ($action == 'add')
        {
            foreach ($available_users as $user_id)
            {
                $data_training_demand_level = [
                    'user_id' => $user_id,
                    'company_id' => $this->company->id,
                    'level' => $level,
                ];
                TrainingDemandLevel::create($data_training_demand_level);
            }
        }

        return redirect()->route('training-demand-level.create', [$locale, $level])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $training_demand_level = TrainingDemandLevel::find($id);

        if ($training_demand_level) 
        {
            $this->check_company($training_demand_level);
            $training_demand_level->delete();

            return redirect()->route('training-demand-level.list', [$locale])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('training-demand-level.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }
}
