<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Training;
use App\Models\TrainingGroup;
use Illuminate\Http\Request;

class TrainingGroupController extends Controller
{
    /**
     * index
     */
    public function index()
    {
        $data = array();
        
        $qb_training_groups = TrainingGroup::where('company_id', $this->company->id)
                                ->orWhereNull('company_id');

        $training_groups = $qb_training_groups->paginate($this->setting->paginate);

        $training_groups_existings = $qb_training_groups->select('id', 'prefix', 'name')->get();
        
        $categories = Category::where('id', '<=', 6)->get();

        $data['training_groups'] = $training_groups;
        $data['training_groups_existings'] = $training_groups_existings;
        $data['categories'] = $categories;

        return view('training.group.index', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $id = $request->input('id');
        $name = $request->input('name');
        $prefix = $request->input('prefix');
        $description = $request->input('description');
        $training_group_existing_id = $request->input('training_group_id');
        $training_group = TrainingGroup::find($id);

        if ($training_group) 
        {
            $this->check_company($training_group);
            
            $training_group->name = $name;
            $training_group->prefix = $prefix;
            $training_group->description = $description;
            $training_group->save();
        }
        else
        {
            $data = array(
                'name' => $name,
                'prefix' => $prefix,
                'description' => $description,
                'company_id' => $this->company->id
            );
            $training_group = TrainingGroup::create($data);
            if ($training_group_existing_id && $training_group)
            {
                $training_group_existing = TrainingGroup::find($training_group_existing_id);
                $training_ids = [];
                foreach ($training_group_existing->trainings as $training)
                {
                    $training_ids[$training->id] = $training->id;
                }

                if (count($training_ids))
                {
                    $training_group->trainings()->attach($training_ids);
                }
            }
        }
        
        return redirect()->route('training-group.list', [$locale])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $training_group = TrainingGroup::find($id);

        if ($training_group) 
        {
            $this->check_company($training_group);

            $training_group->trainings()->detach();
            $training_group->delete();

            return redirect()->route('training-group.list', [$locale])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('training-group.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * view
     */
    public function view(Request $request)
    {
        $training_group = TrainingGroup::find($request->id);
        $data['training_group'] = $training_group;
        $training_categories = [];
        if ($training_group)
        {
            $this->check_company($training_group);
            foreach ($training_group->trainings as $training)
            {
                $training_categories[$training->category_id.'-'.$training->platform_id][] = $training;
            }
        }
        else
        {
            $this->not_found();
        }
        $data['training_categories'] = $training_categories;

        $trainings = [];
        $training_ids = [];
        foreach ($training_categories as $training_category)
        {
            foreach ($training_category as $training)
            {
                $trainings[$training->id] = $training;
                $training_ids[] = $training->id;
            }
        }
        $data['trainings'] = $trainings;

        $categories = Category::where('id', '<=', 6)->get();
        $category_1_2 = clone $categories[0];
        $category_1_2->code = 'external_presentiel';
        $categories[] = $category_1_2;
        $data['categories'] = $categories;

        $new_trainings_tmp = Training::where('company_id', $this->company->id)
                                ->whereNotIn('id', $training_ids)
                                ->where('category_id', '<=', 6)
                                ->where('platform_id', '<=', 2)
                                ->get();
        $new_trainings = [];
        foreach ($new_trainings_tmp as $training)
        {
            if ($training->category_id == 1 && $training->platform_id == 2)
            {
                $training->category->code = 'external_presentiel';
            }
            $new_trainings[] = $training;
        }
        $data['new_trainings'] = $new_trainings;
        
        return view('training.group.view', $data);
    }

    /**
     * training_select
     */
    public function training_select(Request $request, $locale)
    {
        $id = $request->id;
        $training_ids = $request->training_id;
        $training_group = TrainingGroup::find($id);
        if ($training_group)
        {
            $training_group->trainings()->attach($training_ids);
            return redirect()->route('training-group.view', [$locale, $id])->with('success', __('message.successfully_registered'));
        }
        else
        {
            return redirect()->route('training-group.list', $locale)->with('error', __('message.error_training_not_found'));
        }
    }

    /**
     * training_detach
     */
    public function training_detach(Request $request, $locale, $training_group_id)
    {
        $training_id = $request->id;
        $training_group = TrainingGroup::find($training_group_id);

        if ($training_group) 
        {
            $this->check_company($training_group);
            $training_group->trainings()->detach($training_id);

            return redirect()->route('training-group.view', [$locale, $training_group->id])->with('success', __('message.successfully_registered'));
        } 
        else 
        {
            return redirect()->route('training-group.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }
}
