<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\JobTrainingGroup;
use App\Models\Training;
use App\Models\TrainingGroup;
use App\Models\TrainingReport;
use App\Models\TrainingReportQueue;
use App\Models\User;
use Illuminate\Http\Request;

class TrainingReportController extends Controller
{
    protected $data;

    /**
     * construct
     */
    public function __construct(Request $request)
    {
        parent::__construct($request);

        $this->data = [];
        
        $this->data['date_intervals'] = [
            1 => __('app.one_day'),
            2 => __('app.one_month'),
            3 => __('app.three_months'),
            4 => __('app.six_months'),
            5 => __('app.one_year'),
            6 => __('app.year_to_date_ytd'),
            7 => __('app.specific_date')
        ];

        $this->data['filter_types'] = [
            1 => __('app.trainings_assigned'),
            2 => __('app.trainings_assigned_and_completed'),
            3 => __('app.trainings_assigned_and_planned_to_be_completed'),
            4 => __('app.trainings_assigned_planned_to_be_completed_and_completed_within_the_due_date'),
            5 => __('app.trainings_assigned_planned_to_be_completed_and_completed_after_the_due_date'),
            6 => __('app.trainings_assigned_planned_to_be_completed_and_not_yet_completed'),
            7 => __('app.trainings_completed'),
            8 => __('app.trainings_planned_to_be_completed'),
            12 => __('app.trainings_planned_to_be_completed_and_completed'),
            9 => __('app.trainings_planned_to_be_completed_and_completed_within_the_due_date'),
            10 => __('app.trainings_planned_to_be_completed_and_completed_after_the_due_date'),
            11 => __('app.trainings_planned_to_be_completed_and_not_yet_completed')
        ];

        $this->data['filter_metrics'] = [
            1 => __('app.number_of_trainings'),
            2 => __('app.hours_of_trainings'),
            3 => __('app.scheduled_hours_of_trainings')
        ];
        
        $this->data['filter_outputs'] = [
            2 => __('app.table'),
            1 => __('app.pie_chart'),
            3 => __('app.column_chart')
        ];

        $this->data['multiple_filters'] = [
            1 => __('app.department'),
            2 => __('app.subdepartment'),
            3 => __('app.employer'),
            4 => __('users.job'),
            5 => __('app.location_division'),
            6 => __('app.minority'),
            7 => __('users.section'),
            8 => __('users.sex'),
            9 => __('app.age'),
            10 => __('app.team'),
            11 => __('app.employee_category'),
            12 => __('users.place_of_birth'),
        ];

        $this->data['date_filters'] = [
            0 => __('app.select') . '...',
            1 => __('app.assignment_date'),
            2 => __('app.due_date')
        ];

        $this->data['result_list'] = [
            1 => __('app.conformity'),
            2 => __('app.number_of_trainings'),
            3 => __('app.hours_of_completed_trainings'),
            4 => __('app.hours_of_assigned_trainings')
        ];

        $this->data['users'] = User::select('users.id', 'users.lname', 'users.fname', 'users.ref', 'users.nas')
                                ->leftJoin('users_roles', 'users_roles.user_id', '=', 'users.id')
                                ->where('users.company_id', $this->company->id)
                                ->where('users.status', 1)
                                ->whereIn('users_roles.role_id', [1])
                                ->get();
        
        $this->data['training_list'] = Training::select('id', 'name', 'prefix', 'image', 'disable', 'category_id', 'platform_id')
                                ->where('company_id', $this->company->id)
                                ->get();
    }

    /**
     * index
     */
    public function index(Request $request)
    {
        $data = $this->data;
        
        $qb_training_reports = TrainingReport::where('company_id', $this->company->id)
                                ->orWhereNull('company_id');

        $training_reports = $qb_training_reports->paginate($this->setting->paginate);

        $categories = Category::select('categories.id', 'categories.code', 'menus.icon')
                            ->leftJoin('menus', 'menus.menu_level3', 'categories.code')
                            ->whereIn('categories.id', [1, 2, 3, 6])
                            ->get();
        $new_categories = [];
        foreach($categories as $category)
        {
            $new_categories[$category->id] = $category;
        }

        $training_groups = TrainingGroup::where('company_id', $this->company->id)->get();
        $job_training_groups = JobTrainingGroup::where('company_id', $this->company->id)->get();
        
        $data['categories'] = $new_categories;
        $data['training_groups'] = $training_groups;
        $data['job_training_groups'] = $job_training_groups;
        $data['training_reports'] = $training_reports;

        return view('training.report.index', $data);
    }

    /**
     * view
     */
    public function view(Request $request)
    {
        $data = $this->data;
        $id = $request->id;

        $training_report = TrainingReport::find($id);
        if ($training_report) 
        {
            $this->check_company($training_report);

            $training_report->recipients = $this->unserialize($training_report->recipients);
            $training_report->recipients = User::select('users.id', 'users.lname', 'users.fname', 'users.ref', 'users.nas')
                                            ->whereIn('id', $training_report->recipients)
                                            ->get();

            $training_subreports = [];
            foreach ($training_report->training_subreports as $i => $training_subreport)
            {
                $filters = $this->unserialize($training_subreport->filters);
                $filters['metric'] = $this->data['filter_metrics'][$filters['metric']];
                $training_subreport->filters = $filters;
                $filters['output'] = $this->data['filter_outputs'][$filters['output']];
                $training_subreport->filters = $filters;
                $training_subreports[$i] = $training_subreport;
            }
            $trainings = Training::select('id', 'name', 'prefix', 'image', 'disable', 'category_id', 'platform_id')
                            ->whereIn('id', $this->unserialize($training_report->trainings))
                            ->get();
            $training_report->trainings = $trainings;
            $results = [];
            foreach ($this->unserialize($training_report->results) as $result)
            {
                $results[$result] = $this->data['result_list'][$result];
            }
            $training_report->results = $results;

            $data['training_report'] = $training_report;
            $data['training_subreports'] = $training_subreports;
        } 
        else 
        {
            $this->not_found();
        }
        
        return view('training.report.view', $data);
    }
    
    /**
     * create
     */
    public function create(Request $request)
    {
        $data = $this->data;

        $training_report = new TrainingReport();
        $training_report->recipients = [];
        $data['training_report'] = $training_report;

        return view('training.report.edit', $data);
    }

    /**
     * edit
     */
    public function edit(Request $request)
    {
        $data = $this->data;
        $id = $request->id;

        $training_report = TrainingReport::find($id);
        if ($training_report) 
        {
            $this->check_company($training_report);

            $training_report->recipients = $this->unserialize($training_report->recipients);
            $training_report->trainings = $this->unserialize($training_report->trainings);
            $training_report->results = $this->unserialize($training_report->results);

            $data['training_report'] = $training_report;
        } 
        else 
        {
            $this->not_found();
        }
        
        return view('training.report.edit', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $id = $request->input('id');
        $name = $request->input('name');
        $start_date = $request->input('start_date');
        $date_interval = $request->input('date_interval');
        $date = $request->input('date');
        $description = $request->input('description');
        $scheduled = $request->input('scheduled');
        $recipients = $request->input('recipients');
        $filter_by = $request->input('filter_by');
        $trainings = $request->input('trainings');
        $results = $request->input('results');
        if (array_search('select_all', $recipients))
        {
            unset($recipients[array_search('select_all', $recipients)]);
        }
        if (array_search('select_all', $trainings))
        {
            unset($trainings[array_search('select_all', $trainings)]);
        }
        if (array_search('select_all', $recipients))
        {
            unset($results[array_search('select_all', $results)]);
        }
        
        $training_report = TrainingReport::find($id);
        if ($training_report)
        {
            $this->check_company($training_report);
            
            $training_report->name = $name;
            $training_report->start_date = $start_date;
            $training_report->date_interval = $date_interval;
            $training_report->description = $description;
            if ($date_interval == "7")
            {
                $training_report->date = $date;
            }
            if ($scheduled)
            {
                $training_report->scheduled = $scheduled;
                $training_report->recipients = serialize($recipients);
            }
            if ($filter_by)
            {
                $training_report->filter_by = $filter_by;
                $training_report->trainings = serialize($trainings);
                $training_report->results = serialize($results);
            }
            $training_report->save();
        }
        else
        {
            $data = array(
                'name' => $name,
                'start_date' => $start_date,
                'date_interval' => $date_interval,
                'description' => $description,
                'filter_by' => 0,
                'company_id' => $this->company->id
            );
            if ($date_interval == "7")
            {
                $data["date"] = $date;
            }
            if ($scheduled)
            {
                $data["scheduled"] = $scheduled;
                $data["recipients"] = serialize($recipients);
            }
            if ($filter_by)
            {
                $training_report["filter_by"] = $filter_by;
                $training_report["trainings"] = serialize($trainings);
                $training_report["results"] = serialize($results);
            }
            $training_report = TrainingReport::create($data);
        }
        
        return redirect()->route('training-report.list', [$locale])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $training_report = TrainingReport::find($id);

        if ($training_report) 
        {
            $this->check_company($training_report);

            foreach ($training_report->training_subreports as $training_subreport)
            {
                $training_subreport->delete();
            }
            $training_report->delete();

            return redirect()->route('training-report.list', [$locale])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('training-report.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * generate
    */
    public function generate(Request $request, $locale)
    {
        $id = $request->id;

        $training_report = TrainingReport::find($id);
        if ($training_report) 
        {
            $this->check_company($training_report);

            $recipients = $this->unserialize($training_report->recipients);
            foreach ($recipients as $recipient_id)
            {
                $training_report_queue = new TrainingReportQueue();
                $training_report_queue->training_report_id = $training_report->id;
                $training_report_queue->recipient_id = $recipient_id;
                $training_report_queue->save();
            }
        }
        
        return redirect()->route('training-report.view', [$locale, $id])->with('success_stay', __('message.report_has_been_queued_for_generation'));
    }
}
