<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Training;
use App\Models\TrainingSubReport;
use DateInterval;
use DateTime;
use Illuminate\Http\Request;

class TrainingSubReportController extends Controller
{
    protected $data;

    /**
     * construct
     */
    public function __construct(Request $request)
    {
        parent::__construct($request);

        $this->data = [];

        $this->data['date_intervals'] = [
            1 => __('app.one_day'),
            2 => __('app.one_month'),
            3 => __('app.three_months'),
            7 => __('app.specific_date')
        ];

        $this->data['filter_types'] = [
            1 => __('app.trainings_assigned'),
            2 => __('app.trainings_assigned_and_completed'),
            3 => __('app.trainings_assigned_and_planned_to_be_completed'),
            4 => __('app.trainings_assigned_planned_to_be_completed_and_completed_within_the_due_date'),
            5 => __('app.trainings_assigned_planned_to_be_completed_and_completed_after_the_due_date'),
            6 => __('app.trainings_assigned_planned_to_be_completed_and_not_yet_completed'),
            7 => __('app.trainings_completed'),
            8 => __('app.trainings_planned_to_be_completed'),
            12 => __('app.trainings_planned_to_be_completed_and_completed'),
            9 => __('app.trainings_planned_to_be_completed_and_completed_within_the_due_date'),
            10 => __('app.trainings_planned_to_be_completed_and_completed_after_the_due_date'),
            11 => __('app.trainings_planned_to_be_completed_and_not_yet_completed')
        ];

        $this->data['filter_metrics'] = [
            1 => __('app.number_of_trainings'),
            2 => __('app.hours_of_trainings'),
            3 => __('app.scheduled_hours_of_trainings')
        ];
        
        $this->data['filter_outputs'] = [
            2 => __('app.table'),
            1 => __('app.pie_chart'),
            3 => __('app.column_chart')
        ];

        $this->data['multiple_filters'] = [
            1 => __('app.department'),
            2 => __('app.subdepartment'),
            3 => __('app.employer'),
            4 => __('users.job'),
            5 => __('app.location_division'),
            6 => __('app.minority'),
            7 => __('users.section'),
            8 => __('users.sex'),
            9 => __('app.age'),
            10 => __('app.team'),
            11 => __('app.employee_category'),
            12 => __('users.place_of_birth'),
        ];

        if ($request->training_report_id || $request->id)
        {
            $this->data['categories'] = Category::whereIn("id", [1, 2, 3, 4, 5, 6, 7, 8, 9, 100])
                                            ->get();

            $this->data['trainings'] = Training::select('id', 'name', 'prefix', 'image', 'disable', 'category_id', 'platform_id')
                                            ->where('company_id', $this->company->id)
                                            ->get();
        }
    }

    /**
     * create
     */
    public function create(Request $request)
    {
        $data = $this->data;
        $training_report_id = $request->training_report_id;

        $training_subreport = new TrainingSubReport();
        $training_subreport->training_report_id = $training_report_id;

        $data['training_subreport'] = $training_subreport;

        return view('training.report.sub-edit', $data);
    }

    /**
     * edit
     */
    public function edit(Request $request)
    {
        $data = $this->data;
        $id = $request->id;

        $training_subreport = TrainingSubReport::find($id);
        if ($training_subreport) 
        {
            $this->check_company($training_subreport);

            $training_subreport->recipients = $this->unserialize($training_subreport->recipients);

            $filters = $this->unserialize($training_subreport->filters);
            $training_subreport->filter_metric = $filters['metric'];
            $training_subreport->filter_output = $filters['output'];
            $training_subreport->multiple_filters = $filters['filters'];
            $training_subreport->category_id = $filters['category_id'];
            $training_subreport->training_id = $filters['training_id'];

            $data['training_subreport'] = $training_subreport;
        } 
        else 
        {
            $this->not_found();
        }
        
        return view('training.report.sub-edit', $data);
    }

    /**
     * store
    */
    public function store(Request $request, $locale)
    {
        $id = $request->input('id');
        $training_report_id = $request->input('training_report_id');
        $name = $request->input('name');
        $description = $request->input('description');
        $date_interval = $request->input('date_interval');
        $start_date = $request->input('start_date');
        $end_date = $request->input('end_date');
        $filter_type = $request->input('filter_type');
        $filter_metric = $request->input('filter_metric');
        $filter_output = $request->input('filter_output');
        $multiple_filters = $request->input('multiple_filters');
        $category_id = $request->input('category_id');
        $training_id = $request->input('training_id');
        $filters = [
            "metric" => $filter_metric,
            "output" => $filter_output,
            "filters" => $multiple_filters,
            "category_id" => $category_id,
            "training_id" => $training_id,
            "age" => "0"
        ];
        if (in_array($date_interval, [1, 2, 3]))
        {
            $date = new DateTime($start_date);
            switch ($date_interval)
            {
                case 1:
                    $date->add(new DateInterval('P1D'));
                    break;
                case 2:
                    $date->add(new DateInterval('P1M'));
                    break;
                case 3:
                    $date->add(new DateInterval('P3M'));
                    break;           
            }
            $end_date = $date->format('Y-m-d');
        }

        $training_subreport = TrainingSubReport::find($id);
        if ($training_subreport)
        {
            $this->check_company($training_subreport);
        }
        else
        {
            $training_subreport = new TrainingSubReport();
            $training_subreport->training_report_id = $training_report_id;
            $training_subreport->company_id = $this->company->id;
            $training_subreport_last = TrainingSubReport::select('report_order')
                                        ->where('training_report_id', $training_report_id)
                                        ->orderBy('report_order', 'desc')
                                        ->first();
            $training_subreport->report_order = $training_subreport_last ? $training_subreport_last->report_order + 1 : 1;
        }
        $training_subreport->name = $name;
        $training_subreport->description = $description;
        $training_subreport->start_date = $start_date;
        $training_subreport->end_date = $end_date;
        $training_subreport->filter_type = $filter_type;
        $training_subreport->filters = serialize($filters);
        $training_subreport->save();

        return redirect()->route('training-report.view', [$locale, $training_report_id])->with('success', __('message.successfully_registered'));
    }
    
    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $training_subreport = TrainingSubReport::find($id);

        if ($training_subreport) 
        {
            $training_subreport_id = $training_subreport->training_subreport_id;

            $this->check_company($training_subreport);

            $training_subreport->delete();

            return redirect()->route('training-report.view', [$locale, $training_subreport_id])->with('success', __('message.successfully_deleted'));
        }
        else 
        {
            return redirect()->route('training-report.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * update_order
     */
    public function update_order(Request $request, $locale)
    {
        $ids = $request->ids;
        foreach ($ids as $i => $training_subreport_id)
        {
            $training_subreport = TrainingSubReport::find($training_subreport_id);
            $training_subreport->update(['report_order' => $i + 1]);
        }
        return $ids;
    }
}
