<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\AssignedTraining;
use App\Models\Category;
use App\Models\Form;
use App\Models\Presentiel;
use App\Models\User;
use App\Models\UserForm;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;

class UserFormController extends Controller
{
    /**
     * create_presentiel
     */
    public function create_presentiel(Request $request)
    {
        $is_view = 'create';

        return $this->data($request, $is_view);
    }

    /**
     * create
     */
    public function create(Request $request)
    {
        $data = [];

        $user_id = $request->user_id;
        $category_id = $request->category_id;

        $user = User::find($user_id);
        $category = Category::find($category_id);
        if ($user && $category)
        {
            $this->check_company($user);
            $data["user"] = $user;
            $data["category"] = $category;
            $forms = Form::where('category_id', $category->id)
                            ->where('company_id', $this->company->id)
                            ->orderBy('name', 'ASC')
                            ->get();
            $data["forms"] = $forms;
        }
        else
        {
            $this->not_found();
        }

        return view('training.user-form.create', $data);
    }

    /**
     * view
     */
    public function view(Request $request)
    {
        $is_view = true;

        return $this->data($request, $is_view);
    }

    /**
     * edit
     */
    public function edit(Request $request)
    {
        $is_view = false;

        return $this->data($request, $is_view);
    }

    /**
     * data
     */
    public function data(Request $request, $is_view)
    {
        $data = [];

        $is_create = false;
        if ($is_view === "create")
        {
            $is_view = false;
            $is_create = true;
        }
        
        $data['is_view'] = $is_view;
        $assigned_training_id = $request->assigned_training_id ?? false;
        if ($assigned_training_id)
        {
            $assigned_training = AssignedTraining::find($assigned_training_id);
            if (!$assigned_training)
            {
                $this->not_found();
            }
            $presentiel = $assigned_training->presentiel;
        }
        else
        {
            $presentiel_id = $request->presentiel_id ?? false;
            if ($presentiel_id)
            {
                $presentiel = Presentiel::find($presentiel_id);
                if (!$presentiel)
                {
                    $this->not_found();
                }
            }
            else
            {
                $user_form_id = $request->user_form_id ?? false;
                if ($user_form_id)
                {
                    $user_form = UserForm::find($user_form_id);
                    if (!$user_form)
                    {
                        $this->not_found();
                    }
                }
            }
        }

        $data['trainees'] = [];
        if (isset($presentiel) && $presentiel)
        {
            foreach ($presentiel->activities_presentiel as $i => $activity_presentiel)
            {
                foreach ($activity_presentiel->activities as $activity)
                {
                    if ($activity->assigned_training)
                    {
                        $data['trainees'][$activity->assigned_training->user->id] = $activity->assigned_training->user;
                    }
                }
            }
            foreach ($presentiel->assigned_training as $assign_training)
            {
                $data['trainees'][$assign_training->user->id] = $assign_training->user;
            }
        }

        if ($is_create)
        {
            $user_form = new UserForm();
            $form = $assign_training->training->form;
        }
        else 
        {
            if (isset($assigned_training))
            {
                $user_form = $assigned_training->user_form;
            }
            else if (isset($presentiel))
            {
                $user_id = $request->user_id ?? false;
                foreach ($presentiel->assigned_training as $assign_training)
                {
                    if ($assign_training->user->id == $user_id)
                    {
                        $user_form = $assign_training->user_form;
                        $assigned_training = $assign_training;
                        break;
                    }
                }
            }
            else
            {
                $assigned_training = false;
            }
            $form = $user_form->form;
        }
        if ($form) 
        {   
            $this->check_company($form);

            $user_form->answers = $this->unserialize($user_form->answers);
            $data['assigned_training'] = isset($assigned_training) ? $assigned_training : false;
            $data['presentiel'] = isset($presentiel) ? $presentiel : false;
            $data['user_form'] = $user_form;
            $data['form'] = $form;
            $data['category'] = $form->category;
            $data['form_fields'] = [];
            foreach ($form->form_fields as $i => $form_field)
            {
                if ($form_field->type == 5)
                {
                    $content = explode("|", $form_field->content);
                    $form_field->content = $content[0];
                    unset($content[0]);
                    $form_field->list = $content;
                }
                $data['form_fields'][$i] = $form_field;
            }

            $assign_users = User::where("company_id", $this->company->id)->get();
            $data['assign_users'] = $assign_users;
        }
        else
        {
            $this->not_found();
        }

        return view('training.user-form', $data);
    }

    /**
     * store_presentiel
     */
    public function store_presentiel(Request $request, $locale)
    {
        $assigned_training_id = $request->assigned_training_id;
        $user_form_id = $request->user_form_id;

        if ($assigned_training_id && !$user_form_id)
        {
            $assigned_training = AssignedTraining::find($assigned_training_id);
            $data = array(
                'user_id' => $assigned_training->user_id,
                'form_id' => $assigned_training->training->form_id,
                'category_id' => $assigned_training->training->category_id,
                'assigned_training_id' => $assigned_training->id
            );
            $user_form = UserForm::create($data);
            $assigned_training->user_form_id = $user_form->id;
            $assigned_training->save();
        }

        $data = $request->all();
        unset($data['_token']);
        unset($data['assigned_training_id']);
        unset($data['user_form_id']);
        $answers = [];
        $data_user_form = [];
        foreach ($data as $key => $val)
        {
            if (in_array($key, array('signature', 'signature_i', 'signatyre_s')))
            {
                $data_user_form[$key] = $val;
            }
            else
            {
                $key = str_replace("_", " ", $key);
                $answers[$key] =$val;
            }
        }
        $data_user_form['answers'] = serialize($answers);

        if (!isset($user_form) || !$user_form)
        {
            $user_form = UserForm::find($user_form_id);
        }

        if ($user_form)
        {
            $user_form->update($data_user_form);
        }
        else
        {
            $this->not_found();
        }        
        
        if ($assigned_training_id)
        {
            return redirect()->route('user-form.edit-form', [$locale, $assigned_training_id])->with('success', __('message.successfully_registered'));
        }
        else
        {
            return redirect()->route('user-form.edit-form-id', [$locale, $user_form->id])->with('success', __('message.successfully_registered'));
        }
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $user_id = $request->user_id;
        $form_id = $request->form_id;
        
        $user = User::find($user_id);
        if ($user)
        {
            $this->check_company($user);

            $forms = Form::whereIn('id', $form_id)->get();
            foreach ($forms as $form)
            {
                $user_form = new UserForm();
                $user_form->user_id = $user->id;
                $user_form->form_id = $form->id;
                $user_form->category_id = $form->category->id;
                $user_form->save();
            }
        }
        else
        {
            $this->not_found();
        }
        return redirect()->route('assign-users.view', [$locale, $user_id])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $user_form = UserForm::find($id);

        if ($user_form) 
        {
            $this->check_company($user_form);

            $user_id = $user_form->user_id;

            $user_form->delete();
            
            return redirect()->route('assign-users.view', [$locale, $user_id])->with('success', __('message.successfully_registered'));
        } 
        else 
        {
            $this->not_found();
        }
    }

    /**
     * list
     */
    public function list(Request $request, $locale)
    {
        $data = [];

        $user_id = $request->user_id;
        $presentiel_id = $request->presentiel_id;

        $presentiel = Presentiel::find($presentiel_id);

        $assigned_training = AssignedTraining::where('company_id', $this->company->id)
                            ->where('user_id', $user_id)
                            ->where('presentiel_id', $presentiel_id)
                            ->first();

        $user_forms = [];
        if ($assigned_training)
        {
            if ($assigned_training->user_form->form->name)
            {
                $user_forms[$assigned_training->user_form_id] = $assigned_training->user_form;
            }

            foreach ($assigned_training->user_forms as $user_form)
            {
                if ($user_form->form->name)
                {
                    $user_forms[$user_form->id] = $user_form;
                }
            }
        }

        $parameters = Session::get('assigned_training_filter', []);
        $parameters['locale'] = $locale;
        $data['parameters'] = $parameters;

        $data['presentiel'] = $presentiel;
        $data['assigned_training'] = $assigned_training;
        $data['user_forms'] = $user_forms;

        return view('training.user-form.list', $data);
    }
}
