<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Department;
use App\Models\Group;
use App\Models\Location;
use App\Models\Subdepartment;
use App\Models\User;
use Illuminate\Http\Request;

class UserGroupController extends Controller
{
    /**
     * index
     */
    public function index()
    {
        $data = array();
        
        $qb_groups = Group::where('company_id', $this->company->id)
                                ->orWhereNull('company_id');

        $groups = $qb_groups->paginate($this->setting->paginate);

        $data['groups'] = $groups;

        return view('training.user-group.index', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $id = $request->input('id');
        $name = $request->input('name');
        $description = $request->input('description');
        $group = Group::find($id);

        if ($group) 
        {
            $this->check_company($group);
            
            $group->name = $name;
            $group->description = $description;
            $group->save();
        }
        else
        {
            $data = array(
                'name' => $name,
                'description' => $description,
                'company_id' => $this->company->id
            );
            $group = Group::create($data);
        }
        
        return redirect()->route('user-group.list', [$locale])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $group = Group::find($id);

        if ($group) 
        {
            $this->check_company($group);

            $group->users()->detach();
            $group->delete();

            return redirect()->route('user-group.list', [$locale])->with('success', __('message.successfully_deleted'));
        } 
        else 
        {
            return redirect()->route('user-group.list', [$locale])->with('error', __('message.error_deleting'));
        }
    }

    /**
     * view
     */
    public function view(Request $request)
    {
        $data = array();
        
        $id = $request->id;
        $department_id = $request->department_id;
        $subdepartment_id = $request->subdepartment_id;
        $location_id = $request->location_id;
        $group = Group::find($id);

        if ($group) 
        {
            $this->check_company($group);

            $users_in_group_id = [];
            $users_in_group = [];
            foreach ($group->users as $user)
            {
                $users_in_group_id[$user->id] = $user->id;
                $users_in_group[$user->id] = $user;
            }

            $qb_available_users = User::select('id', 'lname', 'fname', 'ref', 'nas');
            $qb_available_users = $qb_available_users->where('company_id', $this->company->id);
            $qb_available_users = $qb_available_users->where('status', 1);
            $qb_available_users = $qb_available_users->whereNotIn('id', $users_in_group_id);
            $qb_available_users = $department_id ? $qb_available_users->where('department_id', $department_id) : $qb_available_users;
            $qb_available_users = $subdepartment_id ? $qb_available_users->where('subdepartment_id', $subdepartment_id) : $qb_available_users;
            $qb_available_users = $location_id ? $qb_available_users->where('location_id', $location_id) : $qb_available_users;
            $available_users = $qb_available_users->get();

            $departments = Department::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
            $subdepartments = Subdepartment::where('company_id', $this->company->id)->orWhereNull('company_id')->get();
            $locations = Location::where('company_id', $this->company->id)->orWhereNull('company_id')->get();

            $data['group'] = $group;
            $data['users_in_group'] = $users_in_group;
            $data['available_users'] = $available_users;
            $data['departments'] = $departments;
            $data['subdepartments'] = $subdepartments;
            $data['locations'] = $locations;
            $data['department_id'] = $department_id;
            $data['subdepartment_id'] = $subdepartment_id;
            $data['location_id'] = $location_id;

            return view('training.user-group.view', $data);
        } 
        else 
        {
            $this->not_found();
        }
    }

    /**
     * store_users
     */
    public function store_users(Request $request, $locale)
    {
        $data = $request->all();
        $id = $data['id'];
        $action = $data['action'] ?? [];
        $available_users = $data['available_users'] ?? [];
        $users_in_group = $data['users_in_group'] ?? [];

        $group = Group::find($id);
        if ($group) 
        {
            $this->check_company($group);

            if ($action == 'remove')
            {
                $group->users()->detach($users_in_group);
            }

            if ($action == 'add')
            {
                $group->users()->attach($available_users);
            }

            return redirect()->route('user-group.view', [$locale, $group->id])->with('success', __('message.successfully_registered'));
        }
        else
        {
            $this->not_found();
        }
    }
}
