<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\Report;
use App\Models\User;
use App\Models\UserReport;
use Illuminate\Http\Request;

class UserReportController extends Controller
{
    /**
     * create
     */
    public function create(Request $request)
    {
        $data = [];

        $user_id = $request->user_id;
        $category_id = $request->category_id;

        $user = User::find($user_id);
        $category = Category::find($category_id);
        if ($user && $category)
        {
            $this->check_company($user);
            $data["user"] = $user;
            $data["category"] = $category;
            $reports = Report::where('company_id', $this->company->id)
                            ->orderBy('name', 'ASC')
                            ->get();
            $data["reports"] = $reports;
        }
        else
        {
            $this->not_found();
        }

        return view('training.user-report.create', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $user_id = $request->user_id;
        $report_id = $request->report_id;
        
        $user = User::find($user_id);
        if ($user)
        {
            $this->check_company($user);

            $reports = Report::whereIn('id', $report_id)->get();
            foreach ($reports as $report)
            {
                $user_report = new UserReport();
                $user_report->user_id = $user->id;
                $user_report->report_id = $report->id;
                $user_report->responsible = $user->id;
                $user_report->save();
            }
        }
        else
        {
            $this->not_found();
        }
        return redirect()->route('assign-users.view', [$locale, $user_id])->with('success', __('message.successfully_registered'));
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $user_report = UserReport::find($id);

        if ($user_report) 
        {
            $this->check_company($user_report);

            $user_id = $user_report->user_id;

            $user_report->delete();
            
            return redirect()->route('assign-users.view', [$locale, $user_id])->with('success', __('message.successfully_registered'));
        } 
        else 
        {
            $this->not_found();
        }
    }
}
