<?php

namespace App\Http\Controllers\Training;

use App\Http\Controllers\Controller;
use App\Models\Activity;
use App\Models\AssignedTraining;
use App\Models\Category;
use App\Models\Department;
use App\Models\JobTrainingGroup;
use App\Models\TrackRecord;
use App\Models\TrackRecordAnswer;
use App\Models\Trainer;
use App\Models\Training;
use App\Models\TrainingGroup;
use App\Models\User;
use App\Models\UserTraining;
use App\Models\UserTrainingModule;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Models\Instructor;
use App\Models\Supervisor;
use App\Models\LiveReading;
use App\Models\TrainingModule;


class UserTrainingController extends Controller
{
    /**
     * create
     */
    public function create(Request $request)
    {
        $data = array();

        $user_id = $request->user_id;
        $category_id = $request->category_id;
        $platform_id = $request->platform_id;

        $user = User::find($user_id);
        $category = Category::find($category_id);
        if ($user && $category)
        {
            $this->check_company($user);
            $data["user"] = $user;
            $data["category"] = $category;
            $trainings = Training::where('category_id', $category->id)
                            ->where('company_id', $this->company->id)
                            ->where('active', 1)
                            ->where('disable', 0)
                            ->where('platform_id', ($platform_id ? '=' : '!='), ($platform_id ? : -1))
                            ->orderBy('name', 'ASC')
                            ->get();
            $data["trainings"] = $trainings;
        }
        else
        {
            $this->not_found();
        }
        return view('training.user-training.create', $data);
    }

    /**
     * create_group
     */
    public function create_group(Request $request)
    {
        $data = array();

        $user_id = $request->user_id;
        $category_id = $request->category_id;

        $user = User::find($user_id);
        $category = Category::find($category_id);
        if ($user && $category)
        {
            $this->check_company($user);
            $data["user"] = $user;
            $data["category"] = $category;
            $training_groups = TrainingGroup::where('company_id', $this->company->id)
                            ->orderBy('name', 'ASC')
                            ->get();
            $data["type_list"] = $training_groups;
            $data["create_title"] = "assign_training_group_to_user";
            $data["create_add_title"] = "add_training_group";
            $data["create_select_title"] = "training_group";
            $data["type"] = "training_group";
        }
        else
        {
            $this->not_found();
        }
        return view('training.user-training.create-other', $data);
    }

    /**
     * create_job_itinerary
     */
    public function create_job_itinerary(Request $request)
    {
        $data = array();

        $user_id = $request->user_id;
        $category_id = $request->category_id;

        $user = User::find($user_id);
        $category = Category::find($category_id);
        if ($user && $category)
        {
            $this->check_company($user);
            $data["user"] = $user;
            $data["category"] = $category;
            $job_itineraries = JobTrainingGroup::where('company_id', $this->company->id)
                            ->orderBy('name', 'ASC')
                            ->get();
            $data["type_list"] = $job_itineraries;
            $data["create_title"] = "assign_job_itinerary_to_user";
            $data["create_add_title"] = "add_job_itinerary";
            $data["create_select_title"] = "job_itinerary";
            $data["type"] = "job_itinerary";
        }
        else
        {
            $this->not_found();
        }
        return view('training.user-training.create-other', $data);
    }

    /**
     * create_department
     */
    public function create_department(Request $request)
    {
        $data = array();

        $user_id = $request->user_id;
        $category_id = $request->category_id;

        $user = User::find($user_id);
        $category = Category::find($category_id);
        if ($user && $category)
        {
            $this->check_company($user);
            $data["user"] = $user;
            $data["category"] = $category;
            $departments = Department::where('company_id', $this->company->id)
                            ->orderBy('name', 'ASC')
                            ->get();
            $data["type_list"] = $departments;
            $data["create_title"] = "assign_department_trainings_to_user";
            $data["create_add_title"] = "add_department_trainings";
            $data["create_select_title"] = "department";
            $data["type"] = "department";
        }
        else
        {
            $this->not_found();
        }
        return view('training.user-training.create-other', $data);
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        $validation = [
            'start_date' => 'required'
        ];
        $request->validate($validation);

        $user_id = $request->user_id;
        $training_id = $request->training_id;
        $start_date = $request->start_date;
        $end_date = $request->end_date;
        
        $user = User::find($user_id);
        if ($user)
        {
            $this->check_company($user);

            $trainings = Training::whereIn('id', $training_id)->get();
            
            $trainer = Trainer::where('company_id', $this->company->id)
                            ->where('user_id', $user->id)
                            ->first();
            $survey = Training::where('company_id', $this->company->id)
                            ->where('category_id', 5)
                            ->where('active', 1)
                            ->first();

            $user_training_ids = [];
            foreach ($trainings as $training)
            {
                list($user_training, $assigned_training) = $this->store_template($training, $user, $start_date, $end_date, $trainer);
                if ($survey)
                {
                    /*
                    list($user_survey, $assigned_survey) = $this->store_template($survey, $user, $start_date, $end_date, $trainer);
                    $user_training->survey_id = $user_survey->id;
                    $user_training->save();
                    $assigned_training->survey_id = $user_survey->id;
                    $assigned_training->save();
                    */
                }
                $user_training_ids[] = $user_training->id;
            }
        }
        else
        {
            $this->not_found();
        }
        if($trainings[0]->category_id == 3){
            return redirect()->route('user-training.trainee', [$locale, $user_training_ids[0]])->with('success', __('message.successfully_registered'));
        }
        return redirect()->route('assign-users.view', [$locale, $user_id])->with('success', __('message.successfully_registered'));
    }

    /**
     * store_other
     */
    public function store_other(Request $request, $locale)
    {
        $validation = [
            'start_date' => 'required'
        ];
        $request->validate($validation);

        $type = $request->type;
        $user_id = $request->user_id;
        $line_id = $request->line_id;
        $start_date = $request->start_date;
        $end_date = $request->end_date;
        
        $user = User::find($user_id);
        if ($user)
        {
            $this->check_company($user);

            $trainings = [];
            $data_user_training = [];
            if ($type == 'training_group')
            {
                $training_group = TrainingGroup::find($line_id);
                $trainings = $training_group->trainings;
                $data_user_training['training_group_id'] = $training_group->id;
            }
            if ($type == 'job_itinerary')
            {
                $job_itinerary = JobTrainingGroup::find($line_id);
                $trainings = $job_itinerary->trainings;
                $data_user_training['job_training_group_id'] = $job_itinerary->id;
            }
            if ($type == 'department')
            {
                $department = Department::find($line_id);
                $trainings = $department->trainings;
            }
            
            $trainer = Trainer::where('company_id', $this->company->id)
                            ->where('user_id', $user->id)
                            ->first();
            $survey = Training::where('company_id', $this->company->id)
                            ->where('category_id', 5)
                            ->where('active', 1)
                            ->first();
            foreach ($trainings as $training)
            {
                list($user_training, $assigned_training) = $this->store_template($training, $user, $start_date, $end_date, $trainer);
                if ($survey)
                {
                    list($user_survey, $assigned_survey) = $this->store_template($survey, $user, $start_date, $end_date, $trainer);
                    $user_training->survey_id = $user_survey->id;
                    $user_training->save();
                    $assigned_training->survey_id = $user_survey->id;
                    $assigned_training->save();
                }
            }
        }
        else
        {
            $this->not_found();
        }
        return redirect()->route('assign-users.view', [$locale, $user_id])->with('success', __('message.successfully_registered'));
    }

    /**
     * store_template
     */
    public function store_template($training, $user, $start_date, $end_date, $trainer)
    {
        if (!$trainer)
        {
            $trainer = Trainer::where('company_id', $this->company->id)
                            ->where('lname', $training->platform->type)
                            ->first();
        }
        $trainer_id = $trainer ? $trainer->id : 0;

        try
        {
            DB::beginTransaction();
            $user_training = new UserTraining();
            $user_training->cat_name = $training->category->cat_name;
            $user_training->cat_num = $training->category->cat_num;
            $user_training->category_id = $training->category->id;
            $user_training->training_id = $training->id;
            $user_training->name = $training->name;
            $user_training->user_id = $user->id;
            $user_training->fname = $user->fname;
            $user_training->lname = $user->lname;
            $user_training->platform_id = $training->platform->id;
            $user_training->activate_on = $start_date;
            if ($end_date) $user_training->due_date = $end_date;
            $user_training->company_id = $this->company->id;
            $user_training->survey_id = 0;
            $user_training->save();

            foreach ($training->modules as $module)
            {
                $user_training_module = new UserTrainingModule();
                $user_training_module->user_training_id = $user_training->id;
                $user_training_module->module_id = $module->id;
                $user_training_module->type = $module->type;
                $user_training_module->score = -1;
                $user_training_module->save();

                foreach ($module->tasks as $task)
                {
                    $track_record = new TrackRecord();
                    $track_record->user_training_id = $user_training->id;
                    $track_record->module_id = $module->id;
                    $track_record->module_order = $module->pivot->order;
                    $track_record->module_name = $module->name;
                    $track_record->task_id = $task->id;
                    $track_record->task_order = $task->order;
                    $track_record->task_title = $task->title;
                    $track_record->task_data_path = $task->data_path;
                    $track_record->data_type = $task->data_type;
                    $track_record->trainee = 0;
                    $track_record->instructor = 0;
                    $track_record->supervisor = 0;
                    $track_record->question = $task->question;
                    $track_record->answer_type = $task->answer_type;
                    $track_record->answer_points = -1;
                    $track_record->task_duration = $task->duration;
                    $track_record->save();

                    foreach ($task->answers as $answer)
                    {
                        $track_record_answer = new TrackRecordAnswer();
                        $track_record_answer->track_record_id = $track_record->id;
                        $track_record_answer->answer = $answer->answer;
                        $track_record_answer->correct = $answer->correct;
                        $track_record_answer->feedback = $answer->feedback;
                        $track_record_answer->filename = $answer->filename;
                        $track_record_answer->filepath = $answer->filepath;
                        $track_record_answer->save();
                    }
                }
            }

            $assigned_training = new AssignedTraining();
            $assigned_training->platform_id = $training->platform->id;
            $assigned_training->category_id = $training->category->id;
            $assigned_training->training_id = $training->id;
            $assigned_training->user_id = $user->id;
            $assigned_training->trainer_id = $trainer_id;
            $assigned_training->start_date = $start_date;
            if ($end_date) $assigned_training->due_date = $end_date;
            $assigned_training->status = 'In Progress';
            $assigned_training->training_passed = 0;
            $assigned_training->user_training_id = $user_training->id;
            $assigned_training->company_id = $this->company->id;
            $assigned_training->survey_id = 0;
            $assigned_training->comments = '';
            $assigned_training->save();

            $activity = new Activity();
            $activity->assigned_training_id = $assigned_training->id;
            $activity->date = $start_date;
            $activity->type = 'Theory';
            $activity->trainer_id = $assigned_training->trainer_id;
            $activity->duration = 0;
            $activity->status = 'Not Completed';
            $activity->save();

            DB::commit();
        }
        catch (\Exception $e) 
        {
            DB::rollBack();
            throw $e;
        }

        return [$user_training, $assigned_training];
    }

    /**
     * view
     */
    public function view(Request $request)
    {
        $data = [];

        $id = $request->id;
        $user_training = UserTraining::find($id);

        if ($user_training) 
        {
            $this->check_company($user_training);
            
            $data['user_training'] = $user_training;

            return view('training.user-training.view', $data);
        } 
        else 
        {
            $this->not_found();
        }
    }

    /**
     * destroy
     */
    public function destroy(Request $request, $locale)
    {
        $id = $request->id;
        $user_training = UserTraining::find($id);

        if ($user_training) 
        {
            $this->check_company($user_training);

            $user_id = $user_training->user_id;

            $user_training->delete();
            
            return redirect()->route('assign-users.view', [$locale, $user_id])->with('success', __('message.successfully_registered'));
        } 
        else 
        {
            $this->not_found();
        }
    }

    public function trainee($locale, $id){
        $model = UserTraining::find($id);

        $trackRecords = TrackRecord::select(['track_record.id','module_id', 'module_name', 'user_training_id', 'task_id', 'task_title', 'trainee', 'trainee_date', 'instructor', 'instructor_date', 'supervisor', 'supervisor_date', 'instructor_id', 'supervisor_id', 'comments', 'comments2', 'comments3', 'user_training.user_id'])
            ->join('user_training', 'track_record.user_training_id', '=', 'user_training.id')
            ->where('user_training_id', '=', $id)
            ->orderBy('module_order', 'ASC')
            ->orderBy('task_order', 'ASC')
            ->distinct()
            ->get();

        $supervisorList = Supervisor::where('user_training_id', $id)->get();
        $instructorsList = Instructor::where('user_training_id', $id)->get();         

        $isTrainee = false;
        $isInstructor = false;
        $isSupervisor = false;
        foreach($trackRecords as $trackRecord){
            if($trackRecord->user_id == auth()->user()->id){
                $isTrainee = true;
            }

             // Check if current user is an instructor for this training
            $instructors = Instructor::where('user_training_id', $trackRecord->user_training_id)
                ->where('user_id', auth()->user()->id)
                ->get();

            $isInstructor = $instructors->contains('user_id', auth()->id());

            // Check if current user is a supervisor for this training
            $supervisors = Supervisor::where('user_training_id', $trackRecord->user_training_id)
                ->where('user_id', auth()->user()->id)
                ->get();

            $isSupervisor = $supervisors->contains('user_id', auth()->id()); 
        }

        // //check role    
        $connected_user = auth()->user();
        $isSuper_trainer = false;        
        $isTrainer = false;
 
        if($connected_user->role_user->role_id == 1){
            $isSuper_trainer = true;
        }

 
        if($connected_user->role_user->role_id == 5){
            $isTrainer = true;
        }
     

        if ($model !== null) {
            return view('training.user-training.trainee', ['model' => $model, 'trackRecords' => $trackRecords, 'supervisors' => $supervisorList, 'id' => $id, 'instructors' => $instructorsList, 'isSupervisor' => $isSupervisor, 'isInstructor' => $isInstructor, 'isSuper_trainer' => $isSuper_trainer, 'isTrainee' => $isTrainee, 'isTrainer' => $isTrainer]);
        }

    }

    public function manageTrainee($locale, $id){
        $model = UserTraining::find($id);

        $trackRecords = TrackRecord::select(['id','module_id', 'module_name', 'user_training_id', 'task_id', 'task_title', 'trainee', 'trainee_date', 'instructor', 'instructor_date', 'supervisor', 'supervisor_date', 'instructor_id', 'supervisor_id', 'comments', 'comments2', 'comments3'])
            ->where('user_training_id', '=', $id)
            ->orderBy('module_order', 'ASC')
            ->orderBy('task_order', 'ASC')
            ->distinct()
            ->get();

        $supervisorList = Supervisor::where('user_training_id', $id)->get();
        $instructorsList = Instructor::where('user_training_id', $id)->get();

     
        $isTrainee = false;
        $isInstructor = false;
        $isSupervisor = false;
        foreach($trackRecords as $trackRecord){
            if($trackRecord->user_id == auth()->user()->id){
                $isTrainee = true;
            }

             // Check if current user is an instructor for this training
            $instructors = Instructor::where('user_training_id', $trackRecord->user_training_id)
                ->where('user_id', auth()->user()->id)
                ->get();

            $isInstructor = $instructors->contains('user_id', auth()->id());

            // Check if current user is a supervisor for this training
            $supervisors = Supervisor::where('user_training_id', $trackRecord->user_training_id)
                ->where('user_id', auth()->user()->id)
                ->get();

            $isSupervisor = $supervisors->contains('user_id', auth()->id()); 
        }

        // //check role    
        $connected_user = auth()->user();
        $isSuper_trainer = false;        
        $isTrainer = false;
 
        if($connected_user->role_user->role_id == 1){
            $isSuper_trainer = true;
        }

 
        if($connected_user->role_user->role_id == 5){
            $isTrainer = true;
        }

        if ($model !== null) {
            return view('training.user-training.trainee', ['model' => $model, 'trackRecords' => $trackRecords, 'supervisors' => $supervisorList, 'id' => $id, 'instructors' => $instructorsList, 'isSupervisor' => $isSupervisor, 'isInstructor' => $isInstructor, 'isSuper_trainer' => $isSuper_trainer, 'isTrainee' => $isTrainee, 'isTrainer' => $isTrainer]);
        }

    }


    public function liveReading($locale, $training_id, $members)
    {
        // Get training and members
        $training = Training::findOrFail($training_id); 
        $memberIds = explode(',', $members);
  
        
        // Create user training records for each member
        $userTrainings = [];
        foreach ($memberIds as $memberId) {
            $userTraining = UserTraining::where([
                'user_id' => $memberId,
                'training_id' => $training_id
            ])->first();

            dd($userTraining);
            
            if ($userTraining) {
                $userTrainings[] = $userTraining->id;
            }
        }

        // Get first module and create track record
        $firstModule = TrainingModule::where('training_id', $training_id)
            ->orderBy('order', 'asc')
            ->first();

        if (!$firstModule) {
            return redirect()->back()->with('error', 'No modules found for this training');
        }

        dd($userTrainings);

        $trackRecord = TrackRecord::where([
            'user_training_id' => $userTrainings[0],
            'module_id' => $firstModule->id,
            'task_order' => 1
        ])->first();

        if (!$trackRecord) {
            return redirect()->back()->with('error', 'No track records found');
        }

        // Create live reading history
        $history = LiveReading::create([
            'user_training_ids' => json_encode($userTrainings),
            'started_at' => now(),
            'completed' => false,
            'duration' => 0
        ]);

        // Start timer for first task
        $trackRecord->last_start = time();
        $trackRecord->save();

        // Get total tasks for first module
        $totalTasks = TrackRecord::where([
            'user_training_id' => $userTrainings[0],
            'module_id' => $firstModule->id
        ])->count();

        return view('training.live-reading', [
            'userTraining' => UserTraining::find($userTrainings[0]),
            'training' => $training,
            'trackRecord' => $trackRecord,
            'totalTasks' => $totalTasks
        ]);
    }
}
