<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller;
use App\Models\TrackRecord;
use Illuminate\Http\Request;

use function Pest\version;

class ScromController extends Controller
{
    /**
     * parse_manifest
     */
    public static function parse_manifest($manifest_path) 
    {
        $manifest_path .= "imsmanifest.xml";
        $manifest_path = substr($manifest_path, 1);
        if (!file_exists($manifest_path)) 
        {
            dd("Error : File imsmanifest.xml not found !");
        }
    
        $xml = simplexml_load_file($manifest_path);
        $ns = $xml->getNamespaces(true);
        
        if (!$xml->resources->resource)
        {
            dd("Error : No SCO detected !");
        }
        
        $scorm_data = [];
        $scorm_data['title'] = (string) $xml->organizations->organization->title;
        $scorm_data['schema'] = (string) $xml->metadata->schema;
        $scorm_data['schemaversion'] = (string) $xml->metadata->schemaversion;
        $scorm_data['version'] = $scorm_data['schemaversion'] == "1.2" ? 1 : 2004;
        foreach ($xml->resources->resource as $resource) {
            $scorm_data['sco'][] = [
                'id' => (string) $resource['identifier'],
                'launch' => (string) $resource['href'],
                'scormType' => (string) $resource['scormType']
            ];
        }

        return $scorm_data;
    }
    
    /**
     * get_scorm_data
     */
    public static function get_scorm_data($scorm_manifest, $track_record)
    {
        $scorm_data = $track_record->scorm_data;
        if (!$scorm_data)
        {
            $version = $scorm_manifest['version'];
            $scorm_data = [];

            $user_id = $track_record->user_training->user->id;
            $user_name = $track_record->user_training->user->fname . ' ' . $track_record->user_training->user->lname;

            // SCORM 1.2
            if ($version == 1)
            {
                $scorm_data["cmi"] = [
                    "cmi.core.student_id" => $user_id, // ID unique de l’apprenant
                    "cmi.core.student_name" => $user_name, // Nom de l’apprenant
                    "cmi.core.lesson_location" => "", // Dernière position visitée
                    "cmi.core.lesson_status" => "", // Statut du module (passed, completed, failed, incomplete, browsed, not attempted)
                    "cmi.core.score.raw" => "", // Score brut de l’apprenant
                    "cmi.core.score.min" => "", // Score minimum requis
                    "cmi.core.score.max" => "", // Score maximum possible
                    "cmi.core.total_time" => "", // Temps total passé dans le module
                    "cmi.core.session_time" => "", // Temps de session actuel
                    "cmi.core.exit" => "", // Mode de sortie (logout, suspend, normal)
                    "cmi.suspend_data" => "", // Données stockées pour reprise
                    "cmi.launch_data" => "", // Données envoyées par le LMS au lancement
                    "cmi.interactions._count" => "", // Nombre d'interactions enregistrées
                    "cmi.interactions.n.id" => "", // ID de la question
                    "cmi.interactions.n.type" => "", // Type de question (true-false, choice, fill-in, etc.)
                    "cmi.interactions.n.student_response" => "", // Réponse de l'utilisateur
                    "cmi.interactions.n.result" => "", // Résultat (correct, wrong, % de réussite)
                ];
                $scorm_data["adl"] = [];
            }
            
            // SCORM 2004
            if ($version == 2004)
            {
                $scorm_data["cmi"] = [
                    "cmi.learner_id" => $user_id, // ID unique de l’apprenant
                    "cmi.learner_name" => $user_name, // Nom de l’apprenant
                    "cmi.location" => "", // Dernière position visitée
                    "cmi.completion_status" => "", // Statut du module (passed, completed, failed, incomplete, unknown)
                    "cmi.success_status" => "", // Succès (passed, failed, unknown)
                    "cmi.score.raw" => "", // Score brut
                    "cmi.score.min" => "", // Score minimum
                    "cmi.score.max" => "", // Score maximum
                    "cmi.score.scaled" => "", // Score entre 0 et 1
                    "cmi.total_time" => "", // Temps total passé
                    "cmi.session_time" => "", // Temps de session
                    "cmi.exit" => "", // Mode de sortie (time-out, suspend, logout, normal)
                    "cmi.suspend_data" => "", // Données stockées pour reprise
                    "cmi.interactions._count" => "", // Nombre d'interactions enregistrées
                    "cmi.interactions.n.id" => "", // ID de la question
                    "cmi.interactions.n.type" => "", // Type de question (true-false, choice, fill-in, etc.)
                    "cmi.interactions.n.timestamp" => "", // Date de l'interaction
                    "cmi.interactions.n.correct_responses._count" => "", // Nombre de réponses correctes
                    "cmi.interactions.n.correct_responses.n.pattern" => "", // Réponse correcte attendue
                    "cmi.interactions.n.learner_response" => "", // Réponse de l'apprenant
                    "cmi.interactions.n.result" => "", // Résultat (correct, incorrect, neutral, % de réussite)
                    "cmi.interactions.n.latency" => "", // Temps de réponse (PT5S)
                ];

                $scorm_data["adl"] = [
                    "adl.nav.request" => "", // Détermine l’action de navigation demandée (continue, previous, choice, exit, exitAll, suspendAll, abandon, abandonAll, _none_).
                    "adl.nav.request_valid.continue" => "", // Indique si la navigation vers l’élément suivant (continue) est valide.
                    "adl.nav.request_valid.previous" => "", // Indique si la navigation vers l’élément précédent (previous) est valide.
                    "adl.nav.request_valid.choice" => "", // Liste des identifiants des éléments vers lesquels l’utilisateur peut naviguer directement.
                    "adl.nav.request_valid.jump" => "", // Liste des identifiants vers lesquels un jump est possible.
                    "adl.nav.request_valid.exit" => "", // Indique si un exit du module est possible.
                    "adl.nav.request_valid.exitAll" => "", // Indique si un exitAll (sortie de tout le cours) est possible.
                    "adl.nav.request_valid.suspendAll" => "", // Indique si suspendAll (pause et sauvegarde de session) est possible.
                    "adl.nav.request_valid.abandon" => "", // Indique si abandon (quitter ce SCO) est possible.
                    "adl.nav.request_valid.abandonAll" => "", // Indique si abandonAll (quitter tout le module) est possible.
                ];
            }

            $scorm_data = json_encode($scorm_data);
        }
        else
        {
            $scorm_data = json_decode($scorm_data, true);;
            if (!array_key_exists("cmi", $scorm_data))
            {
                $new_scorm_data["cmi"] = $scorm_data;
                $new_scorm_data["adl"] = [];
                $scorm_data = $new_scorm_data;
                $scorm_data = json_encode($new_scorm_data);

                $track_record->scorm_data = $scorm_data;
                $track_record->save();
            }
            
            if (is_array($scorm_data))
            {
                $scorm_data = json_encode($scorm_data);
            }
        }

        $scorm_data = str_replace("'", "\'", $scorm_data);

        return $scorm_data;
    }

    /**
     * store
     */
    public function store(Request $request)
    {
        $request_data = $request->all();
        $track_record_id = $request_data["_id"];
        unset($request_data["_id"]);
        unset($request_data["_token"]);

        $track_record = TrackRecord::find($track_record_id);
        $scorm_data = json_decode($track_record->scorm_data, true);
        
        foreach($request_data as $key => $val)
        {
            if (strpos($key, "cmi.") === 0)
            {
                $scorm_data["cmi"][$key] = $val;
            }
            if (strpos($key, "adl.") === 0)
            {
                $scorm_data["adl"][$key] = $val;
            }

            if ($key == "cmi.core.total_time" || $key == "cmi.total_time")
            {
                if ($key == "cmi.core.total_time")
                {
                    $session_time = $this->scormTimeToSecondsWithMilliseconds($scorm_data["cmi"]["cmi.core.session_time"]);
                    $total_time = $this->scormTimeToSeconds($scorm_data["cmi"]["cmi.core.total_time"]);
                }
                if ($key == "cmi.total_time")
                {
                    $session_time = $this->scorm2004TimeToSecondsWithMilliseconds($scorm_data["cmi"]["cmi.session_time"]);
                    $total_time = $this->scormTimeToSeconds($scorm_data["cmi"]["cmi.total_time"]);
                }
                $total_time += $session_time;
                $total_time = $this->secondsToScormTimeWithMilliseconds($total_time);
                $request_data[$key] = $total_time;
                $scorm_data["cmi"][$key] = $total_time;
            }
        }
        $scorm_data = json_encode($scorm_data);
        $track_record->scorm_data = $scorm_data;
        $track_record->save();
        
        echo json_encode($request_data);
    }

    // scormTimeToSeconds
    public function scormTimeToSeconds($scormTime) 
    {
        // Vérifie si le format est correct
        if (!preg_match('/^(\d{2,4}):(\d{2}):(\d{2})(\.\d+)?$/', $scormTime, $matches)) 
        {
            return false; // Format invalide
        }
    
        $hours = intval($matches[1]); // Les heures (peuvent être 2, 3 ou 4 chiffres)
        $minutes = intval($matches[2]); // Les minutes
        $seconds = intval($matches[3]); // Les secondes
    
        return ($hours * 3600) + ($minutes * 60) + $seconds;
    }

    // scormTimeToSecondsWithMilliseconds
    public function scormTimeToSecondsWithMilliseconds($scormTime) 
    {
        if (!preg_match('/^(\d{2,4}):(\d{2}):(\d{2})(\.(\d{1,2}))?$/', $scormTime, $matches)) 
        {
            return false;
        }
    
        $hours = intval($matches[1]);
        $minutes = intval($matches[2]);
        $seconds = intval($matches[3]);
        $milliseconds = isset($matches[5]) ? intval($matches[5]) / 100 : 0; // Converti .CC en fraction de seconde
    
        return ($hours * 3600) + ($minutes * 60) + $seconds + $milliseconds;
    }

    // secondsToScormTime
    public function secondsToScormTime($seconds) {

        $hours = floor($seconds / 3600);
        $minutes = floor(($seconds % 3600) / 60);
        $seconds = $seconds % 60;
    
        // Format SCORM avec 4 chiffres pour les heures
        return sprintf("%04d:%02d:%02d", $hours, $minutes, $seconds);
    }

    // secondsToScormTimeWithMilliseconds    
    public function secondsToScormTimeWithMilliseconds($seconds) 
    {
        $hours = floor($seconds / 3600);
        $minutes = floor(($seconds % 3600) / 60);
        $sec = floor($seconds % 60);
        $milliseconds = round(($seconds - floor($seconds)) * 100); // Convertit la fraction en .CC
    
        return sprintf("%04d:%02d:%02d.%02d", $hours, $minutes, $sec, $milliseconds);
    }

    // secondsToScorm2004Time
    public function secondsToScorm2004Time($seconds) 
    {
        $hours = floor($seconds / 3600);
        $minutes = floor(($seconds % 3600) / 60);
        $sec = $seconds % 60;
    
        return sprintf("PT%01dH%02dM%02dS", $hours, $minutes, $sec);
    }

    // secondsToScorm2004TimeWithMilliseconds
    public function secondsToScorm2004TimeWithMilliseconds($seconds) 
    {
        $hours = floor($seconds / 3600);
        $minutes = floor(($seconds % 3600) / 60);
        $sec = floor($seconds % 60);
        $milliseconds = round(($seconds - floor($seconds)) * 1000); // Convertit la fraction en millisecondes
    
        return sprintf("PT%01dH%02dM%02d.%03dS", $hours, $minutes, $sec, $milliseconds);
    }

    // scorm2004TimeToSeconds
    public function scorm2004TimeToSeconds($scormTime) 
    {
        // Vérifie si le format SCORM 2004 est valide
        if (!preg_match('/PT((\d+)H)?((\d+)M)?((\d+)(\.\d+)?S)?/', $scormTime, $matches)) 
        {
            return false; // Format invalide
        }
    
        $hours = isset($matches[2]) ? intval($matches[2]) : 0;
        $minutes = isset($matches[4]) ? intval($matches[4]) : 0;
        $seconds = isset($matches[6]) ? floatval($matches[6]) : 0;
    
        return ($hours * 3600) + ($minutes * 60) + $seconds;
    }

    // scorm2004TimeToSecondsWithMilliseconds
    public function scorm2004TimeToSecondsWithMilliseconds($scormTime) 
    {
        if (!preg_match('/PT((\d+)H)?((\d+)M)?((\d+)(\.\d+)?S)?/', $scormTime, $matches)) 
        {
            return false;
        }
    
        $hours = isset($matches[2]) ? intval($matches[2]) : 0;
        $minutes = isset($matches[4]) ? intval($matches[4]) : 0;
        $seconds = isset($matches[6]) ? floatval($matches[6]) : 0;
    
        return ($hours * 3600) + ($minutes * 60) + $seconds;
    }
}
