<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller;
use App\Http\Controllers\PDFController;
use App\Models\TrackRecord;
use App\Models\TrackRecordAnswer;
use App\Models\UserTraining;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Storage;
use App\Models\Instructor;
use App\Models\Module;
use App\Models\Supervisor;

class TrackRecordController extends Controller
{
    /**
     * view
     */
    public function view(Request $request)
    {
        $data = [];

        $user_training_id = $request->user_training_id;
        $user_training = UserTraining::find($user_training_id);

        if ($user_training) 
        {
            $this->check_company($user_training);
            
            $data['user_training'] = $user_training;

            $track_records = [];
            $track_record_duration = 0;
            $track_record_id_here = false;
            if (count($user_training->track_records))
            {
                $track_record_id_here = $user_training->track_records[0]->id;
            }
            $found = false;
            foreach ($user_training->track_records as $track_record)
            {
                $track_record->time_taken = $this->sec_to_time_full($track_record->duration);
                $track_records[$track_record->module->id][$track_record->task->id] = $track_record;
                $track_record_duration += $track_record->duration;

                if (($track_record->answer_points == -1 && !$found) || ($track_record->duration && !$found))
                {
                    $track_record_id_here = $track_record->id;
                    $found = true;
                }
            }
            $data['track_record_id_here'] = $track_record_id_here;

            $user_training->track_record_duration_second = $track_record_duration;
            $user_training->track_record_duration = $this->sec_to_time_full($track_record_duration);

            $estimated_time_second = 0;
            $type_modules = [];
            $tasks = [];
            $track_record_modules = [];
            foreach ($user_training->track_records as $track_record)
            {
                $type_modules[$track_record->module->type][$track_record->module->id] = $track_record->module;
                $tasks[$track_record->module->id][] = $track_record->task;
                $estimated_time_second += $track_record->task->duration;
                $track_record_modules[$track_record->module->id][] = $track_record;
            }
            $status_modules = [];
            $completed_training = count($user_training->track_records) ? true : false;
            $completed_tasks = 0;
            foreach ($track_record_modules as $module_id => $list_task)
            {
                $completed = true;
                $timestamp = false;
                foreach ($list_task as $i => $track_record)
                {
                    if ($i == 0)
                    {
                        $timestamp = $track_record->last_stop;
                    }
                    if ($user_training->training->category->id == 2)
                    {
                        if ($track_record->trainee == 0)
                        {
                            $completed = false;
                            $completed_training = false;
                        }
                        else
                        {
                            $completed_tasks++;
                        }
                    }
                    else
                    {
                        if ($track_record->answer_points == -1)
                        {
                            $completed = false;
                            $completed_training = false;
                        }
                    }
                    if ($timestamp < $track_record->last_stop)
                    {
                        $timestamp = $track_record->last_stop;
                    }
                    $track_record->last_stop_date = $track_record->last_stop ? date("Y-m-d", $track_record->last_stop) : '';
                }
                $status_modules[$module_id]['completed'] = $completed;
                $status_modules[$module_id]['date'] = date("Y-m-d", $timestamp);
            }
            $tasks_total = count($user_training->track_records);
            $tasks_status = [$tasks_total, $completed_tasks, $tasks_total - $completed_tasks];
            $data['tasks_status'] = $tasks_status;

            $user_training->estimated_time = $this->sec_to_time_full($estimated_time_second);
            $user_training->estimated_time_second = $estimated_time_second;

            $data['type_modules'] = $type_modules;
            $data['tasks'] = $tasks;
            $data['track_records'] = $track_records;
            $data['status_modules'] = $status_modules;
            $data['completed_training'] = $completed_training;
            
            $documents_training = $this->unserialize($user_training->training->documents);
            $documents_user_training = $this->unserialize($user_training->documents);
            $documents = array_merge($documents_training, $documents_user_training);
            $data['documents'] = $documents;
            $data['documents_training'] = $documents_training;
            $data['documents_user_training'] = $documents_user_training;
            Session::put('downloads', $documents);

            return view('train.view', $data);
        } 
        else 
        {
            $this->not_found();
        }
    }

    /**
     * exam
     */
    public function exam(Request $request, $locale)
    {
        $data = [];

        $track_record_id = $request->track_record_id;
        $track_record = TrackRecord::find($track_record_id);

        if ($track_record) 
        {
            /*
            if ($track_record->module->type == 'Exam' && $track_record->duration) 
            {
                return redirect()->route('track-record.view', [$locale, $track_record->user_training->id]);
            }
            */
            
            if (!session('success'))
            {
                Session::put('answer', 'EXAM');
            }

            $data['track_record'] = $track_record;

            $task = $track_record->task;
            $data_path = $task->data_path;
            $data_path_name = "";
            if (in_array($task->data_type, array('image', 'video', 'scorm')))
            {
                $data_path = $this->unserialize($task->data_path);
                $data_path_name = array_keys($data_path);
                $data_path_name = end($data_path_name);
                $data_path = Storage::url(end($data_path));
            }
            $data['task'] = $task;
            $data['data_path'] = $data_path;
            $data['data_path_name'] = $data_path_name;
            
            $last_start = strtotime('now');
            Session::put('last_start', $last_start);

            $track_records = TrackRecord::select('id')
                                    ->where('user_training_id', $track_record->user_training->id)
                                    ->where('module_id', $track_record->module->id)
                                    ->orderBy('module_order', 'asc')
                                    ->orderBy('task_order', 'asc')
                                    ->get();
            $last_track_record = $track_records[count($track_records) - 1];
            $data['is_last'] = $last_track_record->id == $track_record->id;

            $count = count($track_records);
            foreach ($track_records as $i => $track_record_line)
            {
                $number = $i + 1;
                if ($track_record_line->id == $track_record->id)
                {
                    break;
                }
            }
            $progress_bar = round((100 / $count) * $number);
            $progress_bar = $count == $number ? 100 : $progress_bar;
            $data['progress_bar'] = $progress_bar;
            $data['progress_bar_count'] = "$number / $count";
            
            $documents = $this->unserialize($task->documents);
            $data['documents'] = $documents;
            Session::put('downloads', $documents);

            if ($track_record->task->data_type == 'scorm')
            {
                $scorm_manifest = ScromController::parse_manifest($data_path);
                $scorm_data = ScromController::get_scorm_data($scorm_manifest, $track_record);
                $data['scorm_manifest'] = $scorm_manifest;
                $data['scorm_data'] = $scorm_data;
                $data['scorm_debug'] = $request->debug ? true : false;

                if (!$track_record->scorm_data)
                {
                    $track_record->scorm_data = $scorm_data;
                    $track_record->save();
                }
            }
            return view('train.exam', $data);
        } 
        else 
        {
            $this->not_found();
        }
    }

    /**
     * read
     */
    public function read(Request $request)
    {
        $data = [];

        $track_record_id = $request->track_record_id;
        $track_record = TrackRecord::find($track_record_id);

        if ($track_record) 
        {
            $data['track_record'] = $track_record;

            $task = $track_record->task;
            $data_path = $task->data_path;
            $data_path_name = "";
            if (in_array($task->data_type, array('image', 'video')))
            {
                $data_path = $this->unserialize($task->data_path);
                $data_path_name = array_keys($data_path);
                $data_path_name = end($data_path_name);
                $data_path = Storage::url(end($data_path));
            }
            if (in_array($task->data_type, array('media')))
            {
                $data_path_name = $task->title;
                $data_path = $this->unserialize($task->data_path);
                $new_data_path = [];
                foreach ($data_path as $key_data_path => $val_data_path)
                {
                    $new_data_path[$key_data_path] = Storage::url($val_data_path);
                }
                $data_path = $new_data_path;
            }
            $data['task'] = $task;
            $data['data_path'] = $data_path;
            $data['data_path_name'] = $data_path_name;
            
            $last_start = strtotime('now');
            Session::put('last_start', $last_start);

            $track_records = TrackRecord::select('id')
                                    ->where('user_training_id', $track_record->user_training->id)
                                    ->where('module_id', $track_record->module->id)
                                    ->orderBy('module_order', 'asc')
                                    ->orderBy('task_order', 'asc')
                                    ->get();

            $count = count($track_records);
            foreach ($track_records as $i => $track_record_line)
            {
                $number = $i + 1;
                if ($track_record_line->id == $track_record->id)
                {
                    break;
                }
            }
            $progress_bar = round((100 / $count) * $number);
            $progress_bar = $count == $number ? 100 : $progress_bar;
            $data['progress_bar'] = $progress_bar;
            $data['progress_bar_count'] = "$number / $count";
            
            return view('train.read', $data);
        } 
        else 
        {
            $this->not_found();
        }
    }

    /**
     * store
     */
    public function store(Request $request, $locale)
    {
        if ($request->has('answer'))
        {
            $validation = ['answer' => 'required'];
            $request->validate($validation);
        }
        
        $answer_id = $request->input('answer');

        $last_start = Session::get('last_start');
        $last_stop = strtotime('now');

        $track_record_id = $request->input('track_record_id');

        $track_record = TrackRecord::find($track_record_id);
        if ($track_record) 
        {
            $task = $track_record->task;
            if (in_array($task->answer_type, array(2)))
            {
                if (!$answer_id)
                {
                    $answer_id = [];
                }
            }

            if (in_array($task->answer_type, array(1, 4, 5, 6)))
            {
                foreach ($task->answers as $answer)
                {
                    if (in_array($task->answer_type, array(4, 5)))
                    {
                        if ($answer_id == $answer->answer)
                        {
                            if ($answer->correct == 1)
                            {
                                $track_record->answer_points = $answer_id;
                            }
                            else
                            {
                                $track_record->answer_points = -1 * $answer_id;
                            }
                            break;
                        }
                    }
                    else
                    {
                        if ($answer_id == $answer->id)
                        {
                            if ($answer->correct == 1)
                            {
                                $track_record->answer_points = $answer_id;
                            }
                            else
                            {
                                $track_record->answer_points = -1 * $answer_id;
                            }
                            break;
                        }
                    }
                }
            }
            if (in_array($task->answer_type, array(2, 3)))
            {
                $answer_ids = $answer_id;
                $answered = true;
                foreach ($task->answers as $answer)
                {
                    if ($answer->correct)
                    {
                        if (!in_array($answer->id, $answer_ids))
                        {
                            $answered = false;
                        }
                    }
                }

                $answer_value = 0;
                foreach ($answer_ids as $answer_id)
                {
                    $answer_value += $answer_id;
                }
                if ($answered)
                {
                    $track_record->answer_points = $answer_value;
                }
                else
                {
                    $track_record->answer_points = -1 * $answer_value;
                }
            }
            if (!$task->answer_type || !count($task->answers))
            {
                $track_record->answer_points = $track_record->task->id;
            }

            $duration = $last_stop - $last_start;

            $track_record->last_start = $last_start;
            $track_record->last_stop = $last_stop;
            $track_record->duration = $duration;
            $track_record->save();

            if ($track_record->answer_points && $track_record->answer_points > -1)
            {
                Session::put('answer', 'OK');
            }
            else
            {
                Session::put('answer', 'KO');
            }
            
            if (!$request->has('answer') && (!$track_record->task->answer_type || !count($track_record->task->answers)))
            {
                Session::put('answer', 'OK');
            }
            return redirect()->route('track-record.exam', [$locale, $track_record_id])->with('success', __('message.successfully_registered'));
        }
        else 
        {
            $this->not_found();
        }
    }

    /**
     * store_read
     */
    public function store_read(Request $request, $locale)
    {
        $trainee = $request->input('trainee', 0);

        $last_start = Session::get('last_start');
        $last_stop = strtotime('now');

        $track_record_id = $request->input('track_record_id');

        $track_record = TrackRecord::find($track_record_id);
        if ($track_record) 
        {
            $duration = $last_stop - $last_start;

            $track_record->trainee = $trainee;
            $track_record->last_start = $last_start;
            $track_record->last_stop = $last_stop;
            $track_record->duration = $duration;
            $track_record->save();

            $user_training_id = $track_record->user_training->id;

            if ($track_record->trainee)
            {
                return redirect()->route('track-record.view', [$locale, $user_training_id])->with('success', __('message.successfully_registered'));
            }
            else
            {
                return redirect()->route('track-record.read', [$locale, $track_record_id])->with('success', __('message.successfully_registered'));
            }
        }
        else 
        {
            $this->not_found();
        }
    }

    /**
     * stop
     */
    public function stop(Request $request, $locale)
    {
        Session::put('answer', 'STOP');

        $last_start = Session::get('last_start');
        $last_stop = strtotime('now');

        $track_record_id = $request->input('track_record_id');

        $track_record = TrackRecord::find($track_record_id);
        if ($track_record) 
        {
            $duration = $last_stop - $last_start;

            $track_record->last_start = $last_start;
            $track_record->last_stop = $last_stop;
            $track_record->duration = $duration;
            $track_record->save();

            $user_training_id = $track_record->user_training->id;

            return redirect()->route('track-record.view', [$locale, $user_training_id])->with('success', __('message.successfully_registered'));
        }
        else 
        {
            $this->not_found();
        }
    }

    /**
     * next
     */
    public function next(Request $request, $locale)
    {
        Session::put('answer', 'NEXT');

        $track_record_id = $request->input('track_record_id') ? : $request->track_record_id;

        $track_record = TrackRecord::find($track_record_id);
        if ($track_record) 
        {
            $next_track_records = TrackRecord::select('id')
                                    ->where('user_training_id', $track_record->user_training->id)
                                    ->orderBy('module_order', 'asc')
                                    ->orderBy('task_order', 'asc')
                                    ->get();
            $next_track_record_id = false;
            $next = false;
            foreach ($next_track_records as $i => $next_track_record)
            {
                if ($next_track_record->id == $track_record->id)
                {
                    $next = $i+1;
                }
                if ($i === $next)
                {
                    $next_track_record_id = $next_track_record->id;
                }
            }

            if ($next_track_record_id)
            {
                return redirect()->route('track-record.exam', [$locale, $next_track_record_id])->with('success', __('message.successfully_registered'));
            }
            else
            {
                return redirect()->route('track-record.view', [$locale, $track_record->user_training->id])->with('success', __('message.successfully_registered'));
            }
        }
        else 
        {
            $this->not_found();
        }
    }

    /**
     * review
     */
    public function review(Request $request, $locale)
    {
        $data = [];

        Session::put('answer', 'REVIEW');

        $user_training_id = $request->user_training_id;
        $module_id = $request->module_id;

        $track_records = TrackRecord::where('user_training_id', $user_training_id)
                                ->where('module_id', $module_id)
                                ->where('question', '!=', '')
                                ->whereNotNull('question')
                                ->orderBy('task_order', 'asc')
                                ->get();
        if (count($track_records)) 
        {
            $count_question = count($track_records);
            $count_score_module = 0;
            $passing_grade = 80;
            $module_question_ok = true;
            foreach ($track_records as $track_record)
            {
                if ($track_record->answer_points <= -1)
                {
                    $module_question_ok = false;
                }
                else
                {
                    $count_score_module++;
                }
            }
            if ($count_question == $count_score_module)
            {
                $score_module = 100;
            }
            else
            {
                $score_module = round(($count_score_module * 100) / $count_question, 0);
            }
            $module_exam_ok = $score_module >= $passing_grade;
            $style_score_module = $module_exam_ok ? 'text-success' : 'text-danger';
            $style_passing_grade = 'text-success';

            $track_record = $track_records[0];
            $data['module_question_ok'] = $module_question_ok;
            $data['module_exam_ok'] = $module_exam_ok;
            $data['track_records'] = $track_records;
            $data['user_training'] = $track_record->user_training;
            $data['module'] = $track_record->module;
            $data['score_module'] = '<span class="fs-9 ' . $style_score_module . '">' . $score_module . '%</span>';
            $data['passing_grade'] = '<span class="fs-8 ' . $style_passing_grade . '">' . $passing_grade . '%</span>';

            return view('train.review', $data);
        }
        else 
        {
            $user_training = UserTraining::find($user_training_id);
            $module = Module::find($module_id);

            $data['module_question_ok'] = true;
            $data['user_training'] = $user_training;
            $data['module'] = $module;
            $data['track_records'] = [];

            return view('train.review', $data);
        }
    }

    /**
     * store_documents
     */
    public function store_documents(Request $request, $locale)
    {
        $user_training_id = $request->input('user_training_id');
        $user_training = UserTraining::find($user_training_id);

        if ($user_training)
        {
            if ($request->hasFile('documents')) 
            {
                $file_paths = $this->store_files($request->file('documents'), 'documents');
                $user_training->documents = serialize($file_paths);
                $user_training->save();
            }

            return redirect()->route('track-record.view', [$locale, $user_training->id])->with('success', __('message.successfully_registered'));
        }
        else
        {
            $this->not_found();
        }
    }
    
    public function validateModule(string $locale, int $module_id, int $user_training_id)
    {
        // Get all track records for this module and user training
        $trackRecords = TrackRecord::where('module_id', $module_id)
            ->where('user_training_id', $user_training_id)
            ->get();

        $isSupervisor = true;
        $isInstructor = false;

        foreach ($trackRecords as $trackRecord) {
            // Only update if trainee hasn't already validated
            if($isSupervisor){
                if ($trackRecord->supervisor !== 1) {
                    $trackRecord->supervisor = 1; // Set to "mastered" status
                }
                $trackRecord->supervisor_date = date('Y-m-d H:i:s');
                $trackRecord->save();
            }

            if($isInstructor){
                if ($trackRecord->instructor !== 1) {
                    $trackRecord->instructor = 1; // Set to "mastered" status
                }
                $trackRecord->instructor_date = date('Y-m-d H:i:s');
                $trackRecord->save();
            }
        }

        return redirect()->back()->with('success', __('All tasks in module have been validated'));
    }

    public function test($locale, Request $request, TrackRecord $track_record)
    {
        $isTrainee = true;
        $isInstructor = false;
        $isSupervisor = false;
        $isTrainer = false;


        $userID = auth()->id();
        $trainee = UserTraining::find($track_record->user_training_id);
        $instructors = Instructor::where('user_training_id', $track_record->user_training_id)->get();
        $supervisors = Supervisor::where('user_training_id', $track_record->user_training_id)->get();

        // Logic for checkboxes and user roles

        $tasks = TrackRecord::where('user_training_id', $track_record->user_training_id)
            ->where('module_id', $track_record->module_id)
            ->orderBy('task_order')
            ->get();

        // Déterminez la première et la dernière tâche
        $firstTask = $tasks->first();
        $lastTask = $tasks->last();

        $nextTaskOrder = $track_record->task_order + 1;
        $nextTask = TrackRecord::where(['user_training_id' => $track_record->user_training_id])
            ->where(['module_id' => $track_record->module_id])
            ->where(['task_order' => $nextTaskOrder])
            ->first();

        $prevTaskOrder = $track_record->task_order - 1;
        $prevTask = TrackRecord::where(['user_training_id' => $track_record->user_training_id])
            ->where(['module_id' => $track_record->module_id])
            ->where(['task_order' => $prevTaskOrder])
            ->first();

        //    Déterminez le nom de la tâche courante
        if ($track_record->id == $firstTask->id) {
            $currentTaskName = 'first_task';
        } elseif (!$nextTask) {
            $currentTaskName = 'last_task';
        } else {
            $currentTaskName = 'next_task';
        }

        if($request->isMethod('post')) {
            // if($isTrainee){
            //     $validated = $request->validate([
            //         'trainee' => 'integer',
            //         'comments' => 'string',
            //     ]);
            // }else if($isInstructor){
            //     $validated = $request->validate([
            //         'instructor' => 'integer|nullable',
            //         'comments2' => 'string',
            //     ]);
            // }else{
            //     $validated = $request->validate([
            //         'supervisor' => 'integer|nullable',
            //         'comments3' => 'string',
            //     ]);
            // }

            if($isTrainee){
                $track_record->trainee = $request->trainee;
                $track_record->trainee_date = date('Y-m-d H:i:s');
                $track_record->comments = $request->comments;
            } else if($isInstructor){
                $track_record->instructor = $request->instructor ?? 0;
                $track_record->instructor_id = auth()->id();
                $track_record->instructor_date = date('Y-m-d H:i:s');
                $track_record->comments2 = $request->comments2;
            } else if($isSupervisor){
                $track_record->supervisor = $request->supervisor ?? 0;
                $track_record->supervisor_id = auth()->id();
                $track_record->supervisor_date = date('Y-m-d H:i:s');
                $track_record->comments3 = $request->comments3;
            }
            if($track_record->save() && $nextTask){
                return redirect()->route('track-record.test', [$locale, $nextTask]);
            }else{
                return redirect()->route('user-training.trainee', [$locale, $track_record->user_training_id]);
            }
        }

        return view('train.test', compact('track_record', 'isTrainee', 'isTrainer', 'isInstructor', 'isSupervisor', 'nextTask', 'prevTask', 'currentTaskName'));
    }

}
