<?php

namespace App\Http\Controllers\User;

use App\Http\Controllers\Controller;
use App\Models\Category;
use App\Models\UserTraining;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;

class TrainController extends Controller
{
    /**
     * index
     */
    public function index(Request $request, $locale)
    {
        $data = array();

        $category_id = $request->category_id;
        $category = Category::find($category_id);

        if ($category)
        {
            $data['tutorial_video'] = $this->get_tutorial_video($category, $locale);
            $data['category'] = $category;
            $user_trainings = UserTraining::where('user_id', $this->user->id)
                                ->where('company_id', $this->company->id)
                                ->where('category_id', $category_id)
                                ->orderBy('id')
                                ->get();
            foreach ($user_trainings as $i => $user_training)
            {
                $image = $this->unserialize($user_training->training->image);
                $user_training->training->image = $image;
                $track_record_duration = 0;
                foreach ($user_training->track_records as $track_record)
                {
                    $track_record_duration += $track_record->duration;
                }
                $user_training->track_record_duration_second = $track_record_duration;
                $user_training->track_record_duration = $this->sec_to_time_full($track_record_duration);
                $completed_info = $this->completed_info($user_training);
                $user_training->is_completed = $completed_info['completed'];
                $user_training->date = $completed_info['date'];
                $user_trainings[$i] = $user_training;
            }
            $data["user_trainings"] = $user_trainings;
            $data["category_id"] = $category_id;
        }
        else
        {
            $this->not_found();
        }
        return view('train.index', $data);
    }

    /**
     * completed_info
     */
    protected function completed_info($user_training)
    {
        $completed = count($user_training->track_records) ? true : false;
        $timestamp = false;

        foreach ($user_training->track_records as $i => $track_record)
        {
            if (!$track_record->answer_points)
            {
                $track_record->answer_points = -1;
            }
            if ($i == 0)
            {
                $timestamp = $track_record->last_stop;
            }
            if ($track_record->answer_points == -1)
            {
                $completed = false;
                break;
            }
            if ($timestamp < $track_record->last_stop)
            {
                $timestamp = $track_record->last_stop;
            }
        }

        $date = false;
        if ($timestamp) 
        {
            $date = date("Y-m-d", $timestamp);
        }
        return ['completed' => $completed, 'date' => $date];
    }

    /**
     * get_tutorial_video
     */
    protected function get_tutorial_video($category, $locale)
    {
        $video = "video_$locale";
        $tutorial_video = $category->$video;
        if (!$tutorial_video)
        {
            $tutorial_video = 'videos/default-category-tutorial-video.mp4';
        }
        $tutorial_video = Storage::url($tutorial_video);
        return $tutorial_video;
    }
}
