<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class AssignedTraining extends Model
{
    use HasFactory;

    protected $table = 'assigned_training';

    protected $fillable = [
        'platform_id',
        'category_id',
        'training_id',
        'type',
        'user_id',
        'start_date',
        'due_date',
        'cert_date',
        'attest_date',
        'refresh_date',
        'comments',
        'result',
        'status',
        'training_passed',
        'user_training_id',
        'documents',
        'training_demand_id',
        'presentiel_id',
        'training_group_id',
        'job_training_group_id',
        'practice',
        'theory',
        'user_form_id',
        'company_id',
        'survey_id',
        'completed'
    ];

    /**
     * Get the platform
     * 
     * @return Platform Object
     */
    public function platform(): HasOne
    {
        return $this->hasOne(Platform::class, 'id', 'platform_id')->withDefault();
    }

    /**
     * Get the category
     * 
     * @return Category Object
     */
    public function category(): HasOne
    {
        return $this->hasOne(Category::class, 'id', 'category_id')->withDefault();
    }

    /**
     * Get the training
     * 
     * @return Training Object
     */
    public function training(): HasOne
    {
        return $this->hasOne(Training::class, 'id', 'training_id')->withDefault();
    }

    /**
     * Get the user
     * 
     * @return User Object
     */
    public function user(): HasOne
    {
        return $this->hasOne(User::class, 'id', 'user_id')->withDefault();
    }

    /**
     * Get the trainer
     * 
     * @return Trainer Object
     */
    public function trainer(): HasOne
    {
        return $this->hasOne(Trainer::class, 'id', 'trainer_id')->withDefault();
    }

    /**
     * Get the user_training
     * 
     * @return UserTraining Object
     */
    public function user_training(): HasOne
    {
        return $this->hasOne(UserTraining::class, 'id', 'user_training_id')->withDefault();
    }

    /**
     * Get the training_demand
     * 
     * @return TrainingDemand Object
     */
    public function training_demand(): HasOne
    {
        return $this->hasOne(TrainingDemand::class, 'id', 'training_demand_id')->withDefault();
    }

    /**
     * Get the presentiel
     * 
     * @return Presentiel Object
     */
    public function presentiel(): HasOne
    {
        return $this->hasOne(Presentiel::class, 'id', 'presentiel_id')->withDefault();
    }

    /**
     * Get the training_group
     * 
     * @return TrainingGroup Object
     */
    public function training_group(): HasOne
    {
        return $this->hasOne(TrainingGroup::class, 'id', 'training_group_id')->withDefault();
    }

    /**
     * Get the job_training_group
     * 
     * @return JobTrainingGroup Object
     */
    public function job_training_group(): HasOne
    {
        return $this->hasOne(JobTrainingGroup::class, 'id', 'job_training_group_id')->withDefault();
    }

    /**
     * Get the user_form
     * 
     * @return UserForm Object
     */
    public function user_form(): HasOne
    {
        return $this->hasOne(UserForm::class, 'id', 'user_form_id')->withDefault();
    }

    /**
     * Get the user_forms
     * 
     * @return UserForm Object
     */
    public function user_forms()
    {
        return $this->hasMany(UserForm::class, 'assigned_training_id', 'id')->orderBy('id');
    }

    /**
     * Get the activities
     * 
     * @return Activity Object
     */
    public function activities()
    {
        return $this->hasMany(Activity::class, 'assigned_training_id', 'id')->orderBy('id');
    }
}
