<?php

namespace App\Models;

use App\Models\Trait\ModelForCompanyTrait;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
class DeviceIcons extends Model
{
    use HasFactory, SoftDeletes, ModelForCompanyTrait;

    const STORAGE = "lockouts/device_icons";
    const PUBLIC_STORAGE = "assets/lockouts/device_icons/";

    const DEFAULT_ICON = "no-image.png";

    protected $fillable = ['icon', 'description_fr', 'description_en', 'company_id'];

    public function company()
    {
        return $this->belongsTo(Company::class, 'company_id');
    }

    public function getDescriptionFullAttribute()
    {
        return "FR: {$this->description_fr}<br>EN: {$this->description_en}";
    }

    public function getDescriptionAttribute()
    {
        return "FR: {$this->description_fr} | EN: {$this->description_en}";
    }

    public function getIconAttribute($value)
    {
        if(file_exists(public_path(self::PUBLIC_STORAGE . $value))) {
            return asset(self::PUBLIC_STORAGE  . $value);
        }

        return asset(self::PUBLIC_STORAGE . self::DEFAULT_ICON);
    }

    public function getTitleAttribute($value)
    {
        return file_exists(public_path(self::PUBLIC_STORAGE . $value)) ? $this->type : trans("app.noimage");
    }
}
