<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Devices extends Model
{
    use HasFactory;

    const STORAGE = "lockouts/devices";
    const PUBLIC_STORAGE = "assets/lockouts/devices/";

    const DEFAULT_IMAGE = "no-image.png";

    protected $fillable = [
        'device_id',
        'icon_id',
        'energy_id',
        'position_id',
        'description',
        'emplacement',
        'instruction_id',
        'image',
        'need_lockpad',
        'mechanism_id',
        'position_decad_id',
    ];

    protected $casts = [
        'need_lockpad' => 'boolean',
        'device_id' => 'string',
    ];

    public function icon()
    {
        return $this->belongsTo(DeviceIcons::class, 'icon_id');
    }

    public function mechanism()
    {
        return $this->belongsTo(Mechanisms::class, 'mechanism_id');
    }

    public function instruction()
    {
        return $this->belongsTo(Instructions::class, 'instruction_id');
    }

    public function position()
    {
        return $this->belongsTo(Positions::class, 'position_id');
    }

    public function positon_decad()
    {
        return $this->belongsTo(Positions::class, 'position_decad_id');
    }

    public function energy()
    {
        return $this->belongsTo(Energies::class, 'energy_id');
    }

    public function company()
    {
        return $this->belongsTo(Company::class, 'company_id');
    }

    // public function lockouts()
    // {
    //     return $this->belongsToMany(Lockouts::class, 'lockouts_devices', 'device_id', 'lockout_id')->withPivot('order');
    // }

    public function lockoutItems()
    {
        return $this->morphMany(LockoutItems::class, 'itemable');
    }

    public function getImageAttribute($value)
    {
        return $value ? asset(self::PUBLIC_STORAGE . $value) : asset(self::PUBLIC_STORAGE . self::DEFAULT_IMAGE);
    }
}
