<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class Form extends Model
{
    use HasFactory;

    protected $table = 'forms';

    protected $fillable = [
        'name',
        'category_id',
        'company_id',
        'presentiel',
        'training_id'
    ];

    /**
     * Get the company
     * 
     * @return Company Object
     */
    public function company(): HasOne
    {
        return $this->hasOne(Company::class, 'id', 'company_id')->withDefault();
    }

    /**
     * Get the category
     * 
     * @return Category Object
     */
    public function category(): HasOne
    {
        return $this->hasOne(Category::class, 'id', 'category_id')->withDefault();
    }

    /**
     * Get the training
     * 
     * @return Training Object
     */
    public function training(): HasOne
    {
        return $this->hasOne(Training::class, 'id', 'training_id')->withDefault();
    }

    /**
     * Get the form_fields
     * 
     * @return FormField Object
     */
    public function form_fields()
    {
        return $this->hasMany(FormField::class, 'form_id', 'id')->orderBy('order');
    }
}
