<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Livereading extends Model
{
    use HasFactory;

    /**
     * The table associated with the model.
     *
     * @var string
     */
    protected $table = 'live_readings';

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'company_id',
        'training_id',
        'date',
        'trainees',
        'trainer_id',
        'duration',
        'completed',
    ];

    /**
     * The attributes that should be cast to native types.
     *
     * @var array
     */
    protected $casts = [
        'date' => 'datetime',
        'completed' => 'boolean',
        'trainees' => 'array', // Assumes `trainees` is serialized as an array in Laravel
    ];

    /**
     * Get the related Training model.
     */
    public function training()
    {
        return $this->belongsTo(Training::class, 'training_id', 'id');
    }

    /**
     * Get the related Trainer (User) model.
     */
    public function trainer()
    {
        return $this->belongsTo(User::class, 'trainer_id', 'id');
    }

    /**
     * Check if all user trainings assigned are completed.
     *
     * @return bool
     */
    public function areUserTrainingsAssignedCompleted()
    {
        $traineeIDs = $this->trainees;
        $count = Usertraining::where('training_id', $this->trainingID)
            ->whereIn('user_id', $traineeIDs)
            ->where(function ($query) {
                $query->where('attestation', '>', 0)
                      ->orWhere('certification', '>', 0);
            })
            ->count();

        return $count === count($traineeIDs);
    }
}
