<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;

class Lockouts extends Model
{
    use HasFactory;

    const STORAGE = "lockouts/equipment/images";
    const PUBLIC_STORAGE = "assets/lockouts/equipment/images/";
    const STATUSES = [
        'draft' => 'draft',
        'writed' => 'writed',
        'verified' => 'verified',
        'approved' => 'approved',
        'in_verification' => 'in_verification',
    ];

    const DEFAULT_IMAGE = "no-image.png";

    protected $guarded = [];

    protected $with = ['company', 'location', 'department', 'subDepartment', 'createdBy', 'approvedBy'];

    protected $casts = [
        'affected_at' => 'datetime',
        'approved_at' => 'datetime',
    ];

    protected $fillable = [
        'equipment_id',
        'equipment_description',
        'equipment_image',
        'code',
        'version',
        'equipment_location',
        'title',
        'status',
        'location_id',
        'department_id',
        'sub_department_id',
        'affected_at'
    ];

    public function company()
    {
        return $this->belongsTo(Company::class);
    }

    public function location()
    {
        return $this->belongsTo(Location::class);
    }

    public function department()
    {
        return $this->belongsTo(Department::class);
    }

    public function subDepartment()
    {
        return $this->belongsTo(Subdepartment::class);
    }

    public function createdBy()
    {
        return $this->belongsTo(User::class, 'created_by');
    }

    public function approvedBy()
    {
        return $this->belongsTo(User::class, 'approved_by');
    }

    public function trades()
    {
        return $this->belongsToMany(Trades::class, 'lockouts_trades', 'lockout_id', 'trade_id')->withPivot('order')->orderBy('order');
    }

    public function notesPreparation()
    {
        //filter notes by type preparation
        return $this->belongsToMany(Notes::class, 'lockouts_notes', 'lockout_id', 'note_id')->withPivot('order')->orderBy('order')->where('type', 'PREPA');
    }

    public function notesDeconsignation()
    {
        //filter notes by type deconsignation
        return $this->belongsToMany(Notes::class, 'lockouts_notes', 'lockout_id', 'note_id')->withPivot('order')->orderBy('order')->where('type', 'DECAD');
    }

    // public function notes()
    // {
    //     return $this->belongsToMany(Notes::class, 'lockouts_devices', 'lockout_id', 'note_id')->withPivot('order')->orderBy('order');
    // }

    // public function devices()
    // {
    //     return $this->belongsToMany(Devices::class, 'lockouts_devices', 'lockout_id', 'device_id')->withPivot('order', 'position_operation_id', 'position_cad_id', 'instruction_id', 'mechanism_id')->orderBy('order');
    // }

    public function lockoutItems()
    {
        return $this->hasMany(LockoutItems::class, 'lockout_id')->orderBy('order');
    }

    public function devices()
    {
        return $this->lockoutItems()
            ->where('itemable_type', Devices::class)
            ->with('itemable');
    }

    public function devicesWithLockpad()
    {
        return $this->devices()->get()->filter(function($device) {
            return $device->itemable->need_lockpad;
        });
    }

    public function devicesWithMechanism()
    {
        return $this->devices()->get()->filter(function($device) {
            return $device->itemable->mechanism !== null;
        });
    }

    public function notes()
    {
        return $this->lockoutItems()
            ->where('itemable_type', Notes::class)
            ->with('itemable');
    }

    public function getEquipmentImageAttribute($value)
    {
        return $value ? asset(Lockouts::PUBLIC_STORAGE . $value) : asset(Lockouts::PUBLIC_STORAGE . Lockouts::DEFAULT_IMAGE);
    }

    public function allItems()
    {
        return $this->lockoutItems()
            ->with('itemable')
            ->orderBy('order');
    }

    // Helper pour vérifier le type d'item
    public function isDevice($item)
    {
        return $item->itemable_type === Devices::class;
    }

    public function isNote($item)
    {
        return $item->itemable_type === Notes::class;
    }
}
