<?php

namespace App\Models;

use App\Models\Trait\ModelForCompanyTrait;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;

class Notes extends Model
{
    use HasFactory, SoftDeletes, ModelForCompanyTrait;

    const STORAGE = "lockouts/notes/images";

    const PUBLIC_STORAGE = "assets/lockouts/notes/images/";

    const DEFAULT_IMAGES = "no-image.png";

    /**
     * Les attributs qui sont assignables en masse.
     *
     * @var array
     */
    protected $fillable = [
        'label_fr',
        'label_en',
        'image',
        'company_id',
    ];

    /**
     * Les attributs qui doivent être convertis en types natifs.
     *
     * @var array
     */
    // protected $casts = [
    //     'company' => 'integer',
    // ];

    /**
     * Obtenir l'entreprise associée à cette note.
     */
    public function company()
    {
        return $this->belongsTo(Company::class, 'company');
    }

    public function lockouts()
    {
        return $this->belongsToMany(Lockouts::class, 'lockouts_notes', 'note_id', 'lockout_id')->withPivot('order', 'type')->orderBy('order');
    }

    public function lockoutItems()
    {
        return $this->morphMany(LockoutItems::class, 'itemable');
    }

    // public function lockouts_notes()
    // {
    //     return $this->belongsToMany(Lockouts::class, 'lockouts_devices', 'note_id', 'lockout_id')->withPivot('order', 'type')->orderBy('order');
    // }

    public function getLabelFullAttribute()
    {
        return "FR: {$this->label_fr}<br>EN: {$this->label_en}";
    }

    public function getLabelAttribute()
    {
        return $this->label_fr . ' / ' . $this->label_en;
    }

    public function getImageAttribute($value)
    {
        if(is_null($value)) {
            return null;
        }

        if(file_exists(public_path(self::PUBLIC_STORAGE . $value))) {
            return asset(self::PUBLIC_STORAGE  . $value);
        }

        return asset(self::PUBLIC_STORAGE . self::DEFAULT_IMAGES);
    }

    public function getTitleAttribute($value)
    {
        return file_exists(public_path(self::PUBLIC_STORAGE . $value)) ? $this->type : trans("app.noimage");
    }
}
