<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class Presentiel extends Model
{
    use HasFactory;

    protected $table = 'presentiel';

    protected $fillable = [
        'company_id',
        'training_id',
        'start_date',
        'end_date',
        'time_start',
        'time_stop',
        'trainer_id',
        'training_location_id',
        'is_grantable',
        'cost_redistribution',
        'do_not_send_notification_to_trainees',
        'reccuring_calendar',
        'do_not_send_notification_to_trainer',
        'trainees',
        'documents',
        'training_demand_id'
    ];

    /**
     * Get the training
     * 
     * @return Training Object
     */
    public function training(): HasOne
    {
        return $this->hasOne(Training::class, 'id', 'training_id')->withDefault();
    }

    /**
     * Get the training_location
     * 
     * @return TrainingLocation Object
     */
    public function training_location(): HasOne
    {
        return $this->hasOne(TrainingLocation::class, 'id', 'training_location_id')->withDefault();
    }

    /**
     * Get the trainer
     * 
     * @return Trainer Object
     */
    public function trainer(): HasOne
    {
        return $this->hasOne(Trainer::class, 'id', 'trainer_id')->withDefault();
    }

    /**
     * Get the activities_presentiel
     * 
     * @return ActivityPresentiel Object
     */
    public function activities_presentiel()
    {
        return $this->hasMany(ActivityPresentiel::class, 'presentiel_id', 'id')->orderBy('id');
    }

    /**
     * Get the assigned_training
     * 
     * @return AssignedTraining Object
     */
    public function assigned_training()
    {
        return $this->hasMany(AssignedTraining::class, 'presentiel_id', 'id')->orderBy('id');
    }

    /**
     * Get the training_costs
     * 
     * @return TrainingCost Object
     */
    public function training_costs()
    {
        return $this->hasMany(TrainingCost::class, 'presentiel_id', 'id')->orderBy('id');
    }
}
