<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class Task extends Model
{
    use HasFactory;

    protected $table = 'tasks';

    protected $fillable = [
        'order',
        'title',
        'data_path',
        'data_type',
        'module_id',
        'question',
        'answer_type',
        'prefix',
        'rev',
        'duration',
        'extra_link',
        'timer',
        'documents',
        'serial',
        'scorm_version',
        'scorm_data'
    ];

    /**
     * Get the module
     * 
     * @return Module Object
     */
    public function module(): HasOne
    {
        return $this->hasOne(Module::class, 'id', 'module_id')->withDefault();
    }

    /**
     * Get the answers
     * 
     * @return Answer Object
     */
    public function answers()
    {
        return $this->hasMany(Answer::class, 'task_id', 'id');
    }
}
