<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class TrackRecord extends Model
{
    use HasFactory;

    protected $table = 'track_record';

    protected $fillable = [
        'user_training_id',
        'module_id',
        'module_order',
        'module_name',
        'task_id',
        'task_order',
        'task_title',
        'task_data_path',
        'data_type',
        'trainee',
        'instructor',
        'instructor_id',
        'supervisor',
        'supervisor_id',
        'question',
        'answer_type',
        'answer_points',
        'last_start',
        'last_stop',
        'duration',
        'trainee_date',
        'instructor_date',
        'supervisor_date',
        'playback_position',
        'task_duration',
        'comments',
        'comments2',
        'comments3',
        'scorm_data'
    ];

    /**
     * Get the user_training
     * 
     * @return UserTraining Object
     */
    public function user_training(): HasOne
    {
        return $this->hasOne(UserTraining::class, 'id', 'user_training_id')->withDefault();
    }

    /**
     * Get the module_id
     * 
     * @return Module Object
     */
    public function module(): HasOne
    {
        return $this->hasOne(Module::class, 'id', 'module_id')->withDefault();
    }

    /**
     * Get the task
     * 
     * @return Task Object
     */
    public function task(): HasOne
    {
        return $this->hasOne(Task::class, 'id', 'task_id')->withDefault();
    }

    /**
     * Get the instructor_user
     * 
     * @return User Object
     */
    public function instructor_user(): HasOne
    {
        return $this->hasOne(User::class, 'id', 'instructor_id')->withDefault();
    }

    /**
     * Get the supervisor_user
     * 
     * @return User Object
     */
    public function supervisor_user(): HasOne
    {
        return $this->hasOne(User::class, 'id', 'supervisor_id')->withDefault();
    }
}
