<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class Training extends Model
{
    use HasFactory;

    protected $table = 'training';

    protected $fillable = [
        'category_id',
        'company_id',
        'platform_id',
        'name',
        'prefix',
        'max_attendancy',
        'cost',
        'refresh',
        'result',
        'threshold',
        'requires_theory',
        'requires_practice',
        'duration_th',
        'duration_pr',
        'needs_exam_th',
        'needs_exam_pr',
        'training_level_id',
        'task_video_restrict_fstforward',
        'randomize',
        'disable',
        'self_assign',
        'paid',
        'contractor_assign',
        'department_id',
        'subdepartment_id',
        'location_id',
        'equipement_id',
        'form_id',
        'survey_id',
        'details',
        'signer_id',
        'rev',
        'image',
        'documents',
        'creator_id',
    ];

    /**
     * Get the company
     * 
     * @return Company Object
     */
    public function company(): HasOne
    {
        return $this->hasOne(Company::class, 'id', 'company_id')->withDefault();
    }

    /**
     * Get the category
     * 
     * @return Category Object
     */
    public function category(): HasOne
    {
        return $this->hasOne(Category::class, 'id', 'category_id')->withDefault();
    }

    /**
     * Get the department
     * 
     * @return Department Object
     */
    public function department(): HasOne
    {
        return $this->hasOne(Department::class, 'id', 'department_id')->withDefault();
    }

    /**
     * Get the subdepartment
     * 
     * @return Subdepartment Object
     */
    public function subdepartment(): HasOne
    {
        return $this->hasOne(Subdepartment::class, 'id', 'subdepartment_id')->withDefault();
    }

    /**
     * Get the location
     * 
     * @return Location Object
     */
    public function location(): HasOne
    {
        return $this->hasOne(Location::class, 'id', 'location_id')->withDefault();
    }

    /**
     * Get the platform
     * 
     * @return Platform Object
     */
    public function platform(): HasOne
    {
        return $this->hasOne(Platform::class, 'id', 'platform_id')->withDefault();
    }

    /**
     * Get the training_category
     * 
     * @return TrainingCategory Object
     */
    public function training_category(): HasOne
    {
        return $this->hasOne(TrainingCategory::class, 'id', 'training_category_id')->withDefault();
    }

    /**
     * Get the form
     * 
     * @return Form Object
     */
    public function form(): HasOne
    {
        return $this->hasOne(Form::class, 'id', 'form_id')->withDefault();
    }

    /**
     * Get the training_level
     * 
     * @return TrainingLevel Object
     */
    public function training_level(): HasOne
    {
        return $this->hasOne(TrainingLevel::class, 'id', 'training_level_id')->withDefault();
    }

    /**
     * Get the equipement
     * 
     * @return Equipement Object
     */
    public function equipement(): HasOne
    {
        return $this->hasOne(Equipement::class, 'id', 'equipement_id')->withDefault();
    }

    /**
     * Get the signer
     * 
     * @return Signer Object
     */
    public function signer(): HasOne
    {
        return $this->hasOne(Signer::class, 'id', 'signer_id')->withDefault();
    }

    /**
     * Get the creator
     * 
     * @return User Object
     */
    public function creator(): HasOne
    {
        return $this->hasOne(User::class, 'id', 'user_id')->withDefault();
    }

    /**
     * Get the modules
     * 
     * @return Module Object
     */
    public function modules()
    {
        return $this->belongsToMany(Module::class, 'training_modules', 'training_id', 'module_id')->withPivot('order')->orderBy('order');
    }

    /**
     * Get the forms
     * 
     * @return Form Object
     */
    public function forms()
    {
        return $this->hasMany(Form::class, 'training_id', 'id')->orderBy('form_id');
    }

    /**
     * Get the user_trainings
     * 
     * @return UserTraining Object
     */
    public function user_trainings()
    {
        return $this->hasMany(UserTraining::class, 'training_id', 'id')->orderBy('id');
    }
}
