<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class TrainingDemand extends Model
{
    use HasFactory;

    protected $table = 'training_demand';

    protected $fillable = [
        'applicant_id',
        'demand_date',
        'user_id',
        'training_id',
        'status',
        'decision_date',
        'decision_made',
        'completed_date',
        'not_completed_date',
        'company_id',
        'start_date',
        'location',
        'level2',
        'level3',
        'refusal_raison_level2',
        'refusal_raison_level3',
        'is_closed',
        'team_id',
        'description',
        'presentiel_id'
    ];

    /**
     * Get the applicant
     * 
     * @return User Object
     */
    public function applicant(): HasOne
    {
        return $this->hasOne(User::class, 'id', 'applicant_id')->withDefault();
    }

    /**
     * Get the user
     * 
     * @return User Object
     */
    public function user(): HasOne
    {
        return $this->hasOne(User::class, 'id', 'user_id')->withDefault();
    }

    /**
     * Get the training
     * 
     * @return Training Object
     */
    public function training(): HasOne
    {
        return $this->hasOne(Training::class, 'id', 'training_id')->withDefault();
    }

    /**
     * Get the team
     * 
     * @return Team Object
     */
    public function team(): HasOne
    {
        return $this->hasOne(Team::class, 'id', 'team_id')->withDefault();
    }

    /**
     * Get the presentiel
     * 
     * @return Presentiel Object
     */
    public function presentiel(): HasOne
    {
        return $this->hasOne(Presentiel::class, 'id', 'presentiel_id')->withDefault();
    }

    /**
     * Get the users
     * 
     * @return User Object
     */
    public function users()
    {
        return $this->belongsToMany(User::class, 'training_demand_users', 'training_demand_id', 'user_id');
    }

    /**
     * Get the company
     * 
     * @return Company Object
     */
    public function company(): HasOne
    {
        return $this->hasOne(Company::class, 'id', 'company_id')->withDefault();
    }

    /**
     * Get the training_demand_users
     * 
     * @return TrainingDemandUser Object
     */
    public function training_demand_users()
    {
        return $this->hasMany(TrainingDemandUser::class, 'training_demand_id', 'id')->orderBy('user_id');
    }
}
