<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class TrainingReport extends Model
{
    use HasFactory;

    protected $table = 'training_reports';

    protected $fillable = [
        'name',
        'company_id',
        'start_date',
        'date',
        'date_interval',
        'description',
        'scheduled',
        'trainings',
        'results',
        'recipients',
        'filter_by'
    ];

    /**
     * Get the company
     * 
     * @return Company Object
     */
    public function company(): HasOne
    {
        return $this->hasOne(Company::class, 'id', 'company_id')->withDefault();
    }

    /**
     * Get the training_subreports
     * 
     * @return TrainingSubReport Object
     */
    public function training_subreports()
    {
        return $this->hasMany(TrainingSubReport::class, 'training_report_id', 'id')->orderBy('report_order', 'asc');
    }
}
