<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Notifications\Notifiable;
use App\Models\Matricule;
use App\Models\Company;


class User extends Authenticatable
{
    use HasFactory, Notifiable;

    const NOTIFICATION_TYPE = [
        self::FRIENDLY_REMINDER_NOTIFICATION,
    ];

    const FRIENDLY_REMINDER_NOTIFICATION = 'friendly_reminder';

    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $table = 'users';
    
    protected $fillable = [
        'uid', 'fname', 'lname', 'username', 'password', 'password_ref', 'name', 'email_verified_at',
        'remember_token', 'authkey', 'authkey_expiry', 'password_reset_token', 'status', 'email', 'email2',
        'ref', 'lang', 'company_id', 'department_id', 'work_id', 'sex', 'datestart', 'datestop', 'division',
        'subdepartment_id', 'employer_id', 'location_id', 'section_id', 'minority', 'job_status', 'avatar',
        'dateofbirth', 'person_id', 'password_plain', 'logins', 'cat1', 'cat2', 'cat3', 'cat4', 'cat5',
        'cat6', 'cat7', 'cat8', 'cat9', 'iscontractor', 'nas', 'cross_id', 'use_automatic_matricule',
        'stripe_session_id', 'stripe_session_users', 'team_id', 'is_procedure_resp', 'is_lockout_resp',
        'access_demande_courriel', 'access_contact_courriel', 'access_traininggroup',
        'access_traininggroup_done', 'access_traininggroup_date', 'stripe_customer_id',
        'employe_category_id', 'place_of_birth_id', 'is_deleted', 'failed_logins', 'goal_resp_level',
        'capterra_review', 'capterra_review_date', 'security_question_id', 'security_answer',
        'last_login', 'is_adminreq', 'port_attach_id', 'adress', 'phone_number', 'position',
        'emergency_contact', 'emergency_phone_number', 'supervisor', 'last_login_at', 'last_login_ip','person_id'
    ];

    /**
     * 
     */
    protected $primaryKey = 'id';

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * Get the attributes that should be cast.
     *
     * @return array<string, string>
     */
    protected function casts(): array
    {
        return [
            'email_verified_at' => 'datetime',
            'password' => 'hashed',
        ];
    }

    /**
     * Get the company
     * 
     * @return Company Object
     */
    public function company(): HasOne
    {
        return $this->hasOne(Company::class, 'id', 'company_id')->withDefault();
    }

    /**
     * Get the employer
     * 
     * @return Employer Object
     */
    public function employer(): HasOne
    {
        return $this->hasOne(Employer::class, 'id', 'employer_id')->withDefault();
    }

    /**
     * Get the location
     * 
     * @return Location Object
     */
    public function location(): HasOne
    {
        return $this->hasOne(Location::class, 'id', 'location_id')->withDefault();
    }

    /**
     * Get the work
     * 
     * @return Work Object
     */
    public function work(): HasOne
    {
        return $this->hasOne(Work::class, 'id', 'work_id')->withDefault();
    }

    /**
     * Get the section
     * 
     * @return Section Object
     */
    public function section(): HasOne
    {
        return $this->hasOne(Section::class, 'id', 'section_id')->withDefault();
    }

    /**
     * Get the department
     * 
     * @return Department Object
     */
    public function Department(): HasOne
    {
        return $this->hasOne(Department::class, 'id', 'department_id')->withDefault();
    }

    /**
     * Get the subdepartment
     * 
     * @return Subdepartment Object
     */
    public function subdepartment(): HasOne
    {
        return $this->hasOne(Subdepartment::class, 'id', 'subdepartment_id')->withDefault();
    }

    /**
     * Get the role
     * 
     * @return Role Object
     */
    public function role_user(): HasOne
    {
        return $this->hasOne(UsersRoles::class, 'user_id', 'id');
    }

    /**
     * Get the team
     * 
     * @return Team Object
     */
    public function team(): HasOne
    {
        return $this->hasOne(Team::class, 'id', 'team_id')->withDefault();
    }
    
    /**
     * Get the employe_category
     * 
     * @return EmployeCategory Object
     */
    public function employe_category(): HasOne
    {
        return $this->hasOne(EmployeCategory::class, 'id', 'employe_category_id')->withDefault();
    }

    /**
     * Get the place_of_birth
     * 
     * @return PlaceOfBirth Object
     */
    public function place_of_birth(): HasOne
    {
        return $this->hasOne(PlaceOfBirth::class, 'id', 'place_of_birth_id')->withDefault();
    }

    /**
     * Get the place_of_birth
     * 
     * @return PortOfRegistry Object
     */
    public function portOfRegistry(): HasOne
    {
        return $this->hasOne(PortOfRegistry::class, 'id', 'port_of_registry_id')->withDefault();
    }

    /**
     * Get the assigned_training
     * 
     * @return AssignedTraining Object
     */
    public function assigned_training()
    {
        return $this->hasMany(AssignedTraining::class, 'user_id', 'id')->orderBy('category_id');
    }

    /**
     * Get the user_forms
     * 
     * @return UserForm Object
     */
    public function user_forms()
    {
        return $this->hasMany(UserForm::class, 'user_id', 'id')->orderBy('category_id');
    }

    /**
     * Get the user_reports
     * 
     * @return UserReport Object
     */
    public function user_reports()
    {
        return $this->hasMany(UserReport::class, 'user_id', 'id')->orderBy('id');
    }

    /**
     * Get the user_trainings
     * 
     * @return UserTraining Object
     */
    public function user_trainings()
    {
        return $this->hasMany(UserTraining::class, 'user_id', 'id')->orderBy('category_id');
    }

    public function generate_matricule_id($company_id = NULL){
        if(!$company_id){
            $company_id = $this->company_id;
        } 
        $prefix = '';
        $company_model = Company::where('id', $company_id)->first();

        if($company_model->id == -1){
            $prefix = '';
        } else {
            if($company_model->companyName == 'Forage Nordik Drilling'){
                $prefix = "FNK";
            } 
            elseif($company_model->companyName == 'Groupe Lebel - Pohénégamook'){
                $prefix = "POH";
            } 
            elseif($company_model->companyName == 'Groupe Lebel - Kamouraska'){
                $prefix = "KAM";
            } 
            elseif($company_model->companyName == 'Groupe Minier Windfall'){
                $prefix = "GMW";
            } 
            elseif($company_model->companyName == 'PH Tech'){
                $prefix = "PHT";
            } 
            else {
                $prefix = mb_substr($company_model->name, 0, 3);
            }
        }

        $matricule = Matricule::where('companyID', $company_id)->first();
        $counter = $matricule->counter;

        while($d = User::where(['company_id' => $company_id, 'id' => $prefix . str_pad($counter, 5, '0', STR_PAD_LEFT)])->first()){
            $counter++;
        }
        $matricule->counter = $counter;
        $matricule->save();

        return strtoupper($prefix . str_pad($counter, 5, '0', STR_PAD_LEFT));
    }
}
