<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class UserForm extends Model
{
    use HasFactory;

    protected $table = 'user_forms';

    protected $fillable = [
        'user_id',
        'form_id',
        'category_id',
        'completed',
        'completed_date',
        'active',
        'notification',
        'answers',
        'modified',
        'images',
        'assigned_training_id',
        'signature',
        'signature_i',
        'signature_s'
    ];

    /**
     * Get the user
     * 
     * @return User Object
     */
    public function user(): HasOne
    {
        return $this->hasOne(User::class, 'id', 'user_id')->withDefault();
    }
    
    /**
     * Get the form
     * 
     * @return Form Object
     */
    public function form(): HasOne
    {
        return $this->hasOne(Form::class, 'id', 'form_id')->withDefault();
    }

    /**
     * Get the category
     * 
     * @return Category Object
     */
    public function category(): HasOne
    {
        return $this->hasOne(Category::class, 'id', 'category_id')->withDefault();
    }

    /**
     * Get the assigned_training
     * 
     * @return AssignedTraining Object
     */
    public function assigned_training(): HasOne
    {
        return $this->hasOne(AssignedTraining::class, 'id', 'assigned_training_id')->withDefault();
    }
}
