<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;

class UserTraining extends Model
{
    use HasFactory;

    protected $table = 'user_training';

    protected $fillable = [
        'cat_name',
        'cat_num',
        'category_id',
        'training_id',
        'name',
        'user_id',
        'attestation',
        'attest_assigner',
        'attest_date',
        'certification',
        'cert_assigner',
        'cert_date',
        'assigned_date',
        'progress',
        'fname',
        'lname',
        'active',
        'counter',
        'notification',
        'notification30',
        'notification0',
        'prev_attestations',
        'documents',
        'refresh_date',
        'platform_id',
        'training_group_id',
        'job_training_group_id',
        'last_update',
        'activate_on',
        'due_date',
        'th_duration_taken',
        'pr_duration_taken',
        'company_id',
        'survey_id',
        'form_id',
        'reassigned'
    ];

    /**
     * Get the category
     * 
     * @return Category Object
     */
    public function category(): HasOne
    {
        return $this->hasOne(Category::class, 'id', 'category_id')->withDefault();
    }

    /**
     * Get the training
     * 
     * @return Training Object
     */
    public function training(): HasOne
    {
        return $this->hasOne(Training::class, 'id', 'training_id')->withDefault();
    }

    /**
     * Get the user
     * 
     * @return User Object
     */
    public function user(): HasOne
    {
        return $this->hasOne(User::class, 'id', 'user_id')->withDefault();
    }

    /**
     * Get the platform
     * 
     * @return Platform Object
     */
    public function platform(): HasOne
    {
        return $this->hasOne(Platform::class, 'id', 'platform_id')->withDefault();
    }

    /**
     * Get the training_group
     * 
     * @return TrainingGroup Object
     */
    public function training_group(): HasOne
    {
        return $this->hasOne(TrainingGroup::class, 'id', 'training_group_id')->withDefault();
    }

    /**
     * Get the job_training_group
     * 
     * @return JobTrainingGroup Object
     */
    public function job_training_group(): HasOne
    {
        return $this->hasOne(JobTrainingGroup::class, 'id', 'job_training_group_id')->withDefault();
    }

    /**
     * Get the company
     * 
     * @return Company Object
     */
    public function company(): HasOne
    {
        return $this->hasOne(Company::class, 'id', 'company_id')->withDefault();
    }

    /**
     * Get the form
     * 
     * @return Form Object
     */
    public function form(): HasOne
    {
        return $this->hasOne(Form::class, 'id', 'form_id')->withDefault();
    }

    /**
     * Get the user_training_modules
     * 
     * @return UserTrainingModule Object
     */
    public function user_training_modules()
    {
        return $this->hasMany(UserTrainingModule::class, 'user_training_id', 'id')->orderBy('module_id');
    }

    /**
     * Get the track_records
     * 
     * @return TrackRecord Object
     */
    public function track_records()
    {
        return $this->hasMany(TrackRecord::class, 'user_training_id', 'id')->orderBy('module_order')->orderBy('task_order');
    }

    public static function calculateProgress($user_training_id){
        $isTrainee = true;
        $isInstructor = true;
        $isSupervisor = true;
        $progress = 0;
        // Count total tasks and completed tasks for this training
        $totalTasks = TrackRecord::where('user_training_id', $user_training_id)
            ->count();

        $completedTasks = TrackRecord::where('user_training_id', $user_training_id)
            ->where(function($query) use ($isTrainee, $isInstructor, $isSupervisor) {
                if($isTrainee) {
                    $query->where('trainee', 3);
                } else if($isInstructor) {
                    $query->where('instructor', 1);
                } else if($isSupervisor) {
                    $query->where('supervisor', 1);
                }
            })
            ->count();
        // Count completed tasks for each role
        $completedTasksA = TrackRecord::where('user_training_id', $user_training_id)
            ->where('trainee', 3)
            ->count();

        $completedTasksI = TrackRecord::where('user_training_id', $user_training_id)
            ->where('instructor', 1)
            ->count();

        $completedTasksS = TrackRecord::where('user_training_id', $user_training_id)
            ->where('supervisor', 1)
            ->count();

        // Calculate percentages
        $progress_a = $totalTasks > 0 ? ($completedTasksA / $totalTasks) * 100 : 0;
        $progress_i = $totalTasks > 0 ? ($completedTasksI / $totalTasks) * 100 : 0;
        $progress_s = $totalTasks > 0 ? ($completedTasksS / $totalTasks) * 100 : 0;

        return [
            'progress_a' => $progress_a,
            'progress_i' => $progress_i,
            'progress_s' => $progress_s
        ];
    }
}
