<?php

namespace App\Providers;

use App\Models\Langs;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\View;
use Illuminate\Support\ServiceProvider;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Lang;
use Illuminate\Support\Facades\Schema;

class AppServiceProvider extends ServiceProvider
{
    /**
     * Register any application services.
     */
    public function register(): void
    {
        //
    }

    /**
     * Bootstrap any application services.
     */
    public function boot(): void
    {
        Paginator::useBootstrapFive();
        
        View::composer('layouts.app-bo', function ($view) 
        {
            $menus_left = array();
            $menus_top = array();
            $menus_icon = array();

            $langs = Langs::all();
            $view->with("langs", $langs);
            
            $uri = $this->app->request->getRequestUri();
            $uris = explode("/", $uri);
            $lang = $uris[1];
            $lang = Langs::where("code", "=", $lang)->first();
            if (!$lang)
            {
                $lang = Lang::locale();
                $new_uri = $uri;
            }
            else
            {
                $lang = $lang->code;
                unset($uris[1]);
                $new_uri = implode("/", $uris);
            }
            App::setlocale($lang);

            $view->with("uri", $new_uri);

            if (Auth::check())
            {
                $user_id = Auth::user()->id;

                $menus = DB::table('menus')
                            ->leftJoin('roles_menus', 'roles_menus.menu_id', 'menus.id')
                            ->leftJoin('users_roles', 'users_roles.role_id', 'roles_menus.role_id')
                            ->where('users_roles.user_id', '=', $user_id)
                            ->where('menus.status', '=', 1)
                            ->select('menus.*')
                            ->orderBy('menus.position', 'asc')
                            ->orderBy('menus.number', 'asc')
                            ->orderBy('menus.id', 'asc')
                            ->get();
                
                $menus_active = "";
                foreach($menus as $menu)
                {
                    $menu_level1 = $menu->menu_level1;
                    $menu_level2 = $menu->menu_level2;
                    $menu_level3 = $menu->menu_level3;
                    $position = $menu->position;
                    $icon = $menu->icon;
                    $link = $menu->link;
                    $link = str_replace("{locale}", App::currentLocale(), $link);

                    if ($position == "LEFT" && in_array($menu_level1, array($this->app->request->segment(2), $this->app->request->segment(3))))
                    {
                        $menus_active = $menu_level1;
                    }

                    $menus_icon[$menu_level1] = $icon;
                    $menus_icon[$menu_level1.$menu_level2] = $icon;
                    $menus_icon[$menu_level1.$menu_level2.$menu_level3] = $icon;
                    if ($menu_level1 == $menu_level2)
                    {
                        $menus_icon[$menu_level1.$menu_level3] = $icon;
                    }

                    if ($menu_level1 && $menu_level2 && $menu_level3)
                    {
                        if ($position == "LEFT")
                        {
                            $menus_left[$menu_level1][$menu_level2][$menu_level3] = $link;
                        }
                        else if ($position == "TOP")
                        {
                            $menus_top[$menu_level1][$menu_level2][$menu_level3] = $link;
                        }
                    }
                    else if ($menu_level1 && $menu_level2)
                    {
                        if ($position == "LEFT")
                        {
                            $menus_left[$menu_level1][$menu_level2] = $link;
                        }
                        else if ($position == "TOP")
                        {
                            $menus_top[$menu_level1][$menu_level2] = $link;
                        }
                    }
                    else if ($menu_level1)
                    {
                        if ($position == "LEFT")
                        {
                            $menus_left[$menu_level1] = $link;
                        }
                        else if ($position == "TOP")
                        {
                            $menus_top[$menu_level1] = $link;
                        }
                    }
                }
            }
            $new_menus_left = array();
            foreach ($menus_left as $key1 => $val1)
            {
                if (is_string($val1))
                {
                    $new_menus_left[$key1] = $val1;
                }
                else
                {
                    if (count($val1) == 1)
                    {
                        $new_menus_left[$key1] = end($val1);
                    }
                    else
                    {
                        $new_menus_left[$key1] = $val1;
                    }
                }
            }
            $menus_left = $new_menus_left;
            $list = array(
                'menus_left' => $menus_left, 
                'menus_top' => $menus_top, 
                'menus_icon' => $menus_icon, 
                'menus_active' => $menus_active
            );
            $view->with('menus', $list);
        });

        Schema::defaultStringLength(191);
    }
}
